@@ -1,5 +1,5 @@
 /*
- * Copyright 2008-2018 Aerospike, Inc.
+ * Copyright 2008-2022 Aerospike, Inc.
  *
  * Portions may be licensed to Aerospike, Inc. under one or more contributor
  * license agreements.
@@ -28,5 +28,5 @@
  * FUNCTIONS
  *****************************************************************************/
 
-as_rec * map_rec_new();
+as_rec * map_rec_new(void);
 as_rec * map_rec_init(as_rec *);
