@@ -1,5 +1,5 @@
 /*
- * Copyright 2008-2018 Aerospike, Inc.
+ * Copyright 2008-2022 Aerospike, Inc.
  *
  * Portions may be licensed to Aerospike, Inc. under one or more contributor
  * license agreements.
@@ -28,7 +28,7 @@
  * TYPES
  *****************************************************************************/
 
-typedef as_val * (* producer_callback)();
+typedef as_val * (* producer_callback)(void);
 
 /*****************************************************************************
  * FUNCTIONS
