@@ -1,5 +1,5 @@
 /*
- * Copyright 2008-2022 Aerospike, Inc.
+ * Copyright 2008-2023 Aerospike, Inc.
  *
  * Portions may be licensed to Aerospike, Inc. under one or more contributor
  * license agreements.
@@ -20,37 +20,32 @@
  * @defgroup scan_operations Scan Operations
  * @ingroup client_operations
  *
- * Aerospike Scan Operations provide the ability to scan all record of a 
- * namespace and set in an Aerospike database. 
+ * Aerospike Scan Operations provide the ability to scan all record of a namespace and set
+ * in an Aerospike database.
  *
  * ## Usage
  *
- * Before you can execute a scan, you first need to define a scan using 
- * as_scan. See as_scan for details on defining scans.
+ * Before you can execute a scan, you first need to define a scan using as_scan. See as_scan
+ * for details on defining scans.
  *
- * Once you have a scan defined, then you can execute the scan
- * using either:
+ * Once you have a scan defined, then you can execute the scan using either:
  *
- * - aerospike_scan_foreach() — Execute a scan on the database, then process 
- * 	the results.
- * - aerospike_scan_background() — Send a scan to the database, and not wait 
- * 	for completed. The scan is given an id, which can be used to query the
- * 	scan status.
+ * - aerospike_scan_foreach() — Execute a scan on the database, then process the results.
+ * - aerospike_scan_background() — Send a scan to the database, and not wait for completed.
+ *   The scan is given an id, which can be used to query the scan status.
  *
- * When aerospike_scan_foreach() is executed, it will process the results
- * and create records on the stack. Because the records are on the stack, 
- * they will only be available within the context of the callback function.
+ * When aerospike_scan_foreach() is executed, it will process the results and create records
+ * on the stack. Because the records are on the stack, they will only be available within the
+ * context of the callback function.
  *
- * When aerospike_scan_background() is executed, the client will not wait for 
- * results from the database. Instead, the client will be given a scan_id, 
- * which can be used to query the scan status on the database via 
- * aerospike_scan_info().
+ * When aerospike_scan_background() is executed, the client will not wait for results from the
+ * database. Instead, the client will be given a scan_id, which can be used to query the scan
+ * status on the database via aerospike_scan_info().
  *
  * ## Walk-through
  * 
- * First, we build a scan using as_scan. The scan will be on the "test"
- * namespace and "demo" set. We will select only bins "a" and "b" to be returned 
- * for each record.
+ * First, we build a scan using as_scan. The scan will be on the "test" namespace and "demo" set.
+ * We will select only bins "a" and "b" to be returned for each record.
  * 
  * ~~~~~~~~~~{.c}
  * as_scan scan;
@@ -66,27 +61,27 @@
  * 
  * ~~~~~~~~~~{.c}
  * if (aerospike_scan_foreach(&as, &err, NULL, &scan, callback, NULL) != AEROSPIKE_OK) {
- * 	   fprintf(stderr, "error(%d) %s at [%s:%d]", err.code, err.message, err.file, err.line);
+ *     printf("error(%d) %s at [%s:%d]", err.code, err.message, err.file, err.line);
  * }
  * ~~~~~~~~~~
  * 
  * The callback provided to the function above is implemented as:
  * 
  * ~~~~~~~~~~{.c}
- * bool callback(const as_val * val, void* udata)
+ * bool callback(const as_val* val, void* udata)
  * {
- * 	   as_record* rec = as_record_fromval(val);
- * 	   if ( !rec ) return false;
- * 	   fprintf("record contains %d bins", as_record_numbins(rec));
- * 	   return true;
+ *     if (!val) {
+ *         return false; // Scan complete.
+ *     }
+ *
+ *     as_record* rec = as_record_fromval(val);
+ *     // Process record
+ *     // Do not call as_record_destroy() because the calling function will do that for you.
+ *     return true;
  * }
  * ~~~~~~~~~~
- * 
- * An as_scan is simply a scan definition, so it does not contain any state,
- * allowing it to be reused for multiple scan operations. 
- * 
- * When you are finished with the scan, you should destroy the resources 
- * allocated to it:
+ *
+ * When you are finished with the scan, you should destroy the resources allocated to it:
  *
  * ~~~~~~~~~~{.c}
  * as_scan_destroy(&scan);
@@ -116,16 +111,6 @@ extern "C" {
  * Multiple threads will likely be calling this callback in parallel.  Therefore,
  * your callback implementation should be thread safe.
  *
- * The following functions accept the callback:
- * - aerospike_scan_foreach()
- * - aerospike_scan_node()
- * 
- * ~~~~~~~~~~{.c}
- * bool my_callback(const as_val * val, void* udata) {
- * 	   return true;
- * }
- * ~~~~~~~~~~
- *
  * @param val 			The value received from the query.
  * @param udata 		User-data provided to the calling function.
  *
@@ -139,19 +124,22 @@ typedef bool (*aerospike_scan_foreach_callback)(const as_val* val, void* udata);
  * Asynchronous scan user callback.  This function is called for each record returned.
  * This function is also called once when the scan completes or an error has occurred.
  *
- * @param err			This error structure is only populated when the command fails. Null on success.
- * @param record 		Returned record.  Use as_val_reserve() on record to prevent calling function from destroying.
- * 						The record will be null on final scan completion or scan error.
+ * @param err			This error structure is only populated when the command fails.
+ *                      NULL on success.
+ * @param record 		Returned record. The record will be NULL on final scan completion or scan
+ *                      error.
  * @param udata 		User data that is forwarded from asynchronous command function.
- * @param event_loop 	Event loop that this command was executed on.  Use this event loop when running
- * 						nested asynchronous commands when single threaded behavior is desired for the
- * 						group of commands.
+ * @param event_loop 	Event loop that this command was executed on.  Use this event loop when
+ *                      running nested asynchronous commands when single threaded behavior is
+ *                      desired for the group of commands.
  *
  * @return `true` to continue to the next value. Otherwise, the scan will end.
  *
  * @ingroup scan_operations
  */
-typedef bool (*as_async_scan_listener)(as_error* err, as_record* record, void* udata, as_event_loop* event_loop);
+typedef bool (*as_async_scan_listener)(
+	as_error* err, as_record* record, void* udata, as_event_loop* event_loop
+	);
 
 /******************************************************************************
  * FUNCTIONS
@@ -170,10 +158,10 @@ typedef bool (*as_async_scan_listener)(as_error* err, as_record* record, void* u
  * uint64_t scanid = 0;
  * 
  * if (aerospike_scan_background(&as, &err, NULL, &scan, &scanid) != AEROSPIKE_OK) {
- *     fprintf(stderr, "error(%d) %s at [%s:%d]", err.code, err.message, err.file, err.line);
+ *     printf("error(%d) %s at [%s:%d]", err.code, err.message, err.file, err.line);
  * }
  * else {
- * 	   printf("Running background scan job: %ll", scanid);
+ *     printf("Running background scan job: %ll", scanid);
  * }
  * as_scan_destroy(&scan);
  * ~~~~~~~~~~
@@ -183,9 +171,9 @@ typedef bool (*as_async_scan_listener)(as_error* err, as_record* record, void* u
  *
  * @param as			The aerospike instance to use for this operation.
  * @param err			The as_error to be populated if an error occurs.
- * @param policy		The policy to use for this operation. If NULL, then the default policy will be used.
+ * @param policy		Scan policy configuration parameters, pass in NULL for default.
  * @param scan 			The scan to execute against the cluster.
- * @param scan_id		The id for the scan job, which can be used for querying the status of the scan.
+ * @param scan_id		The id for the scan job, which can be used for obtaining scan status.
  *
  * @return AEROSPIKE_OK on success. Otherwise an error occurred.
  *
@@ -207,7 +195,7 @@ aerospike_scan_background(
  *
  * @param as			The aerospike instance to use for this operation.
  * @param err			The as_error to be populated if an error occurs.
- * @param policy		The policy to use for this operation. If NULL, then the default policy will be used.
+ * @param policy		Scan policy configuration parameters, pass in NULL for default.
  * @param scan_id		The id for the scan job.
  * @param interval_ms	The polling interval in milliseconds. If zero, 1000 ms is used.
  *
@@ -228,7 +216,7 @@ aerospike_scan_wait(
  * as_scan_info scan_info;
  * 
  * if (aerospike_scan_info(&as, &err, NULL, &scan, scan_id, &scan_info) != AEROSPIKE_OK) {
- * 	   fprintf(stderr, "error(%d) %s at [%s:%d]", err.code, err.message, err.file, err.line);
+ *     printf("error(%d) %s at [%s:%d]", err.code, err.message, err.file, err.line);
  * }
  * else {
  *     printf("Scan id=%ll, status=%d percent=%d", scan_id, scan_info.status, scan_info.progress_pct);
@@ -237,7 +225,7 @@ aerospike_scan_wait(
  * 
  * @param as			The aerospike instance to use for this operation.
  * @param err			The as_error to be populated if an error occurs.
- * @param policy		The policy to use for this operation. If NULL, then the default policy will be used.
+ * @param policy		Scan policy configuration parameters, pass in NULL for default.
  * @param scan_id		The id for the scan job to check the status of.
  * @param info			Information about this scan, to be populated by this operation.
  *
@@ -259,18 +247,30 @@ aerospike_scan_info(
  * If "scan.concurrent" is true (default false), the callback code must be thread-safe.
  *
  * ~~~~~~~~~~{.c}
+ * bool callback(const as_val* val, void* udata)
+ * {
+ *     if (!val) {
+ *         return false; // Scan complete.
+ *     }
+ *
+ *     as_record* rec = as_record_fromval(val);
+ *     // Process record
+ *     // Do not call as_record_destroy() because the calling function will do that for you.
+ *     return true;
+ * }
+ *
  * as_scan scan;
  * as_scan_init(&scan, "test", "demo");
  * 
  * if (aerospike_scan_foreach(&as, &err, NULL, &scan, callback, NULL) != AEROSPIKE_OK) {
- * 	   fprintf(stderr, "error(%d) %s at [%s:%d]", err.code, err.message, err.file, err.line);
+ *     printf("error(%d) %s at [%s:%d]", err.code, err.message, err.file, err.line);
  * }
  * as_scan_destroy(&scan);
  * ~~~~~~~~~~
  * 
  * @param as			The aerospike instance to use for this operation.
  * @param err			The as_error to be populated if an error occurs.
- * @param policy		The policy to use for this operation. If NULL, then the default policy will be used.
+ * @param policy		Scan policy configuration parameters, pass in NULL for default.
  * @param scan			The scan to execute against the cluster.
  * @param callback		The function to be called for each record scanned.
  * @param udata			User-data to be passed to the callback.
@@ -292,6 +292,18 @@ aerospike_scan_foreach(
  * been scanned, then callback will be called with a NULL value for the record.
  *
  * ~~~~~~~~~~{.c}
+ * bool callback(const as_val* val, void* udata)
+ * {
+ *     if (!val) {
+ *         return false; // Scan complete.
+ *     }
+ *
+ *     as_record* rec = as_record_fromval(val);
+ *     // Process record
+ *     // Do not call as_record_destroy() because the calling function will do that for you.
+ *     return true;
+ * }
+ *
  * char* node_names = NULL;
  * int n_nodes = 0;
  * as_cluster_get_node_names(as->cluster, &n_nodes, &node_names);
@@ -303,7 +315,7 @@ aerospike_scan_foreach(
  * as_scan_init(&scan, "test", "demo");
  *
  * if (aerospike_scan_node(&as, &err, NULL, &scan, node_names[0], callback, NULL) != AEROSPIKE_OK ) {
- * 	   fprintf(stderr, "error(%d) %s at [%s:%d]", err.code, err.message, err.file, err.line);
+ *     printf("error(%d) %s at [%s:%d]", err.code, err.message, err.file, err.line);
  * }
  *
  * free(node_names);
@@ -312,7 +324,7 @@ aerospike_scan_foreach(
  *
  * @param as			The aerospike instance to use for this operation.
  * @param err			The as_error to be populated if an error occurs.
- * @param policy		The policy to use for this operation. If NULL, then the default policy will be used.
+ * @param policy		Scan policy configuration parameters, pass in NULL for default.
  * @param scan			The scan to execute against the cluster.
  * @param node_name		The node name to scan.
  * @param callback		The function to be called for each record scanned.
@@ -337,6 +349,18 @@ aerospike_scan_node(
  * If "scan.concurrent" is true (default false), the callback code must be thread-safe.
  *
  * ~~~~~~~~~~{.c}
+ * bool callback(const as_val* val, void* udata)
+ * {
+ *     if (!val) {
+ *         return false; // Scan complete.
+ *     }
+ *
+ *     as_record* rec = as_record_fromval(val);
+ *     // Process record
+ *     // Do not call as_record_destroy() because the calling function will do that for you.
+ *     return true;
+ * }
+ *
  * as_scan scan;
  * as_scan_init(&scan, "test", "demo");
  *
@@ -344,14 +368,14 @@ aerospike_scan_node(
  * as_partition_filter_set_range(&pf, 0, 1024);
  * 
  * if (aerospike_scan_partitions(&as, &err, NULL, &scan, &pf, callback, NULL) != AEROSPIKE_OK) {
- * 	   fprintf(stderr, "error(%d) %s at [%s:%d]", err.code, err.message, err.file, err.line);
+ *     printf("error(%d) %s at [%s:%d]", err.code, err.message, err.file, err.line);
  * }
  * as_scan_destroy(&scan);
  * ~~~~~~~~~~
  * 
  * @param as			The aerospike instance to use for this operation.
  * @param err			The as_error to be populated if an error occurs.
- * @param policy		The policy to use for this operation. If NULL, then the default policy will be used.
+ * @param policy		Scan policy configuration parameters, pass in NULL for default.
  * @param scan			The scan to execute against the cluster.
  * @param pf			Partition filter.
  * @param callback		The function to be called for each record scanned.
@@ -379,19 +403,19 @@ aerospike_scan_partitions(
  * ~~~~~~~~~~{.c}
  * bool my_listener(as_error* err, as_record* record, void* udata, as_event_loop* event_loop)
  * {
- * 	   if (err) {
- * 	       printf("Scan failed: %d %s\n", err->code, err->message);
- * 		   return false;
- * 	   }
+ *     if (err) {
+ *         printf("Scan failed: %d %s\n", err->code, err->message);
+ *         return false;
+ *     }
  *
- * 	   if (! record) {
- * 	       printf("Scan ended\n");
- * 	       return false;
- * 	   }
+ *     if (! record) {
+ *         printf("Scan ended\n");
+ *         return false;
+ *     }
  *
- * 	   // Process record
- * 	   // Do not call as_record_destroy() because the calling function will do that for you.
- * 	   return true;
+ *     // Process record
+ *     // Do not call as_record_destroy() because the calling function will do that for you.
+ *     return true;
  * }
  *
  * as_scan scan;
@@ -403,12 +427,13 @@ aerospike_scan_partitions(
  *
  * @param as			The aerospike instance to use for this operation.
  * @param err			The as_error to be populated if an error occurs.
- * @param policy		The policy to use for this operation. If NULL, then the default policy will be used.
+ * @param policy		Scan policy configuration parameters, pass in NULL for default.
  * @param scan			The scan to execute against the cluster.
  * @param scan_id		The id for the scan job.  Use NULL if the scan_id will not be used.
  * @param listener		The function to be called for each record scanned.
  * @param udata			User-data to be passed to the callback.
- * @param event_loop 	Event loop assigned to run this command. If NULL, an event loop will be chosen by round-robin.
+ * @param event_loop 	Event loop assigned to run this command. If NULL, an event loop will be 
+ *                      chosen by round-robin.
  *
  * @return AEROSPIKE_OK if async scan succesfully queued. Otherwise an error.
  *
@@ -429,19 +454,19 @@ aerospike_scan_async(
  * ~~~~~~~~~~{.c}
  * bool my_listener(as_error* err, as_record* record, void* udata, as_event_loop* event_loop)
  * {
- * 	   if (err) {
- * 	       printf("Scan failed: %d %s\n", err->code, err->message);
- * 		   return false;
- * 	   }
- *
- * 	   if (! record) {
- * 	       printf("Scan ended\n");
- * 	       return false;
+ *     if (err) {
+ *         printf("Scan failed: %d %s\n", err->code, err->message);
+ *         return false;
  *     }
  *
- * 	   // Process record
- * 	   // Do not call as_record_destroy() because the calling function will do that for you.
- * 	   return true;
+ *     if (! record) {
+ *         printf("Scan ended\n");
+ *         return false;
+ *     }
+ *
+ *     // Process record
+ *     // Do not call as_record_destroy() because the calling function will do that for you.
+ *     return true;
  * }
  *
  * char* node_names = NULL;
@@ -449,7 +474,7 @@ aerospike_scan_async(
  * as_cluster_get_node_names(as->cluster, &n_nodes, &node_names);
  *
  * if (n_nodes <= 0)
- * 	   return <error>;
+ *     return <error>;
  *
  * as_scan scan;
  * as_scan_init(&scan, "test", "demo");
@@ -462,13 +487,14 @@ aerospike_scan_async(
  *
  * @param as			The aerospike instance to use for this operation.
  * @param err			The as_error to be populated if an error occurs.
- * @param policy		The policy to use for this operation. If NULL, then the default policy will be used.
+ * @param policy		Scan policy configuration parameters, pass in NULL for default.
  * @param scan			The scan to execute against the cluster.
  * @param scan_id		The id for the scan job.  Use NULL if the scan_id will not be used.
  * @param node_name		The node name to scan.
  * @param listener		The function to be called for each record scanned.
  * @param udata			User-data to be passed to the callback.
- * @param event_loop 	Event loop assigned to run this command. If NULL, an event loop will be chosen by round-robin.
+ * @param event_loop 	Event loop assigned to run this command. If NULL, an event loop will be
+ *                      chosen by round-robin.
  *
  * @return AEROSPIKE_OK if async scan succesfully queued. Otherwise an error.
  *
@@ -492,19 +518,19 @@ aerospike_scan_node_async(
  * ~~~~~~~~~~{.c}
  * bool my_listener(as_error* err, as_record* record, void* udata, as_event_loop* event_loop)
  * {
- * 	   if (err) {
- * 	       printf("Scan failed: %d %s\n", err->code, err->message);
- * 		   return false;
- * 	   }
+ *     if (err) {
+ *         printf("Scan failed: %d %s\n", err->code, err->message);
+ *         return false;
+ *     }
  *
- * 	   if (! record) {
- * 	       printf("Scan ended\n");
- * 	       return false;
- * 	   }
+ *     if (! record) {
+ *         printf("Scan ended\n");
+ *         return false;
+ *     }
  *
- * 	   // Process record
- * 	   // Do not call as_record_destroy() because the calling function will do that for you.
- * 	   return true;
+ *     // Process record
+ *     // Do not call as_record_destroy() because the calling function will do that for you.
+ *     return true;
  * }
  *
  * as_scan scan;
@@ -519,12 +545,13 @@ aerospike_scan_node_async(
  *
  * @param as			The aerospike instance to use for this operation.
  * @param err			The as_error to be populated if an error occurs.
- * @param policy		The policy to use for this operation. If NULL, then the default policy will be used.
+ * @param policy		Scan policy configuration parameters, pass in NULL for default.
  * @param scan			The scan to execute against the cluster.
  * @param pf			Partition filter.
  * @param listener		The function to be called for each record scanned.
  * @param udata			User-data to be passed to the callback.
- * @param event_loop 	Event loop assigned to run this command. If NULL, an event loop will be chosen by round-robin.
+ * @param event_loop 	Event loop assigned to run this command. If NULL, an event loop will be
+ *                      chosen by round-robin.
  *
  * @return AEROSPIKE_OK if async scan succesfully queued. Otherwise an error.
  *
