@@ -1,5 +1,5 @@
 /*
- * Copyright 2008-2022 Aerospike, Inc.
+ * Copyright 2008-2024 Aerospike, Inc.
  *
  * Portions may be licensed to Aerospike, Inc. under one or more contributor
  * license agreements.
@@ -38,6 +38,7 @@ extern "C" {
 #define AS_COMMAND_FLAGS_READ 1
 #define AS_COMMAND_FLAGS_BATCH 2
 #define AS_COMMAND_FLAGS_LINEARIZE 4
+#define AS_COMMAND_FLAGS_SPLIT_RETRY 8
 
 // Field IDs
 #define AS_FIELD_NAMESPACE 0
@@ -79,7 +80,7 @@ extern "C" {
 #define AS_MSG_INFO2_GENERATION_GT		(1 << 3) // apply write if new generation >= old, good for restore
 #define AS_MSG_INFO2_DURABLE_DELETE		(1 << 4) // transaction resulting in record deletion leaves tombstone (Enterprise only).
 #define AS_MSG_INFO2_CREATE_ONLY		(1 << 5) // write record only if it doesn't exist
-// (Note:  Bit 6 is unused.)
+#define AS_MSG_INFO2_RELAX_AP_LONG_QUERY (1 << 6) // treat as long query, but relax read consistency.
 #define AS_MSG_INFO2_RESPOND_ALL_OPS	(1 << 7) // return a result for every operation.
 
 // Message info3 bits
@@ -186,9 +187,10 @@ typedef struct as_command_s {
 	uint32_t iteration;
 	uint32_t sent;
 	uint8_t flags;
-	bool master;
-	bool master_sc;   // Used in batch only.
-	bool split_retry; // Used in batch only.
+	uint8_t replica_size;
+	uint8_t replica_index;
+	uint8_t replica_index_sc; // Used in batch only.
+	as_latency_type latency_type;
 } as_command;
 
 /**
@@ -364,8 +366,8 @@ as_command_write_header_write(
 uint8_t*
 as_command_write_header_read(
 	uint8_t* cmd, const as_policy_base* policy, as_policy_read_mode_ap read_mode_ap,
-	as_policy_read_mode_sc read_mode_sc, uint32_t timeout, uint16_t n_fields, uint16_t n_bins,
-	uint8_t read_attr, uint8_t info_attr
+	as_policy_read_mode_sc read_mode_sc, int read_ttl, uint32_t timeout, uint16_t n_fields,
+	uint16_t n_bins, uint8_t read_attr, uint8_t write_attr, uint8_t info_attr
 	);
 
 /**
@@ -375,7 +377,8 @@ as_command_write_header_read(
 uint8_t*
 as_command_write_header_read_header(
 	uint8_t* cmd, const as_policy_base* policy, as_policy_read_mode_ap read_mode_ap,
-	as_policy_read_mode_sc read_mode_sc, uint16_t n_fields, uint16_t n_bins, uint8_t read_attr
+	as_policy_read_mode_sc read_mode_sc, int read_ttl, uint16_t n_fields, uint16_t n_bins,
+	uint8_t read_attr
 	);
 
 /**
@@ -664,18 +667,19 @@ as_task_id_resolve(uint64_t* task_id_ptr)
 
 /**
  * @private
- * Replica round robin iterator.
+ * Return command's starting replica index for AS_POLICY_REPLICA_ANY.
  */
-extern uint32_t g_replica_rr;
+uint8_t
+as_replica_index_any(void);
 
 /**
  * @private
- * Should command target master on first iteration.
+ * Return starting replica index for read commands.
  */
-static inline bool
-as_command_target_master(as_policy_replica replica)
+static inline uint8_t
+as_replica_index_init_read(as_policy_replica replica)
 {
-	return (replica != AS_POLICY_REPLICA_ANY || (as_faa_uint32(&g_replica_rr, 1) & 1));
+	return (replica != AS_POLICY_REPLICA_ANY)? 0 : as_replica_index_any();
 }
 
 /**
