@@ -1,5 +1,5 @@
 /*
- * Copyright 2008-2022 Aerospike, Inc.
+ * Copyright 2008-2024 Aerospike, Inc.
  *
  * Portions may be licensed to Aerospike, Inc. under one or more contributor
  * license agreements.
@@ -26,9 +26,9 @@
 extern "C" {
 #endif
 
-/******************************************************************************
- * MACROS
- *****************************************************************************/
+//---------------------------------
+// Macros
+//---------------------------------
 
 #ifdef __linux__
 /**
@@ -60,14 +60,14 @@ extern "C" {
  */
 #define AS_PASSWORD_SIZE 64
 
-/******************************************************************************
- * TYPES
- *****************************************************************************/
+//---------------------------------
+// Types
+//---------------------------------
 
 /**
  * IP translation table.
  *
- * @ingroup as_config_object
+ * @relates as_config
  */
 typedef struct as_addr_map_s {
 	
@@ -86,7 +86,7 @@ typedef struct as_addr_map_s {
 /**
  * Authentication mode.
  *
- * @ingroup as_config_object
+ * @relates as_config
  */
 typedef enum as_auth_mode_e {
 	/**
@@ -120,7 +120,7 @@ typedef enum as_auth_mode_e {
 /**
  * Cluster event notification type.
  *
- * @ingroup as_config_object
+ * @relates as_config
  */
 typedef enum as_cluster_event_type_e {
 	/**
@@ -142,7 +142,7 @@ typedef enum as_cluster_event_type_e {
 /**
  * Cluster event notification data.
  *
- * @ingroup as_config_object
+ * @relates as_config
  */
 typedef struct as_cluster_event_s {
 	/**
@@ -171,14 +171,14 @@ typedef struct as_cluster_event_s {
  * as_cluster_event is placed on the stack before calling.
  * Do not free node_name or node_address.
  *
- * @ingroup as_config_object
+ * @relates as_config
  */
 typedef void (*as_cluster_event_callback) (as_cluster_event* event);
 
 /**
  * lua module config
  *
- * @ingroup as_config_object
+ * @relates as_config
  */
 typedef struct as_config_lua_s {
 
@@ -199,7 +199,7 @@ typedef struct as_config_lua_s {
 /**
  * TLS module config
  *
- * @ingroup as_config_object
+ * @relates as_config
  */
 typedef struct as_config_tls_s {
 
@@ -213,9 +213,17 @@ typedef struct as_config_tls_s {
 	 * Path to a trusted CA certificate file.
 	 * By default TLS will use system standard trusted CA certificates.
 	 * Use as_config_tls_set_cafile() to set this field.
+	 * If cafile is populated, castring is ignored.
 	 */
 	char* cafile;
 
+	/**
+	 * String containing trusted CA certificate(s).
+	 * Use as_config_tls_set_castring() to set this field.
+	 * If cafile is populated, castring is ignored.
+	 */
+	char* castring;
+
 	/**
 	 * Path to a directory of trusted certificates.
 	 * See the OpenSSL SSL_CTX_load_verify_locations manual page for
@@ -272,8 +280,8 @@ typedef struct as_config_tls_s {
 	/**
 	 * Path to the client's key for mutual authentication.
 	 * By default mutual authentication is disabled.
-	 *
 	 * Use as_config_tls_set_keyfile() to set this field.
+	 * If keyfile is populated, keystring is ignored.
 	 */
 	char* keyfile;
 
@@ -285,14 +293,30 @@ typedef struct as_config_tls_s {
 	 */
 	char* keyfile_pw;
 
+	/**
+	 * Client's key string for mutual authentication.
+	 * By default mutual authentication is disabled.
+	 * Use as_config_tls_set_keystring() to set this field.
+	 * If keyfile is populated, keystring is ignored.
+	 */
+	char* keystring;
+
 	/**
 	 * Path to the client's certificate chain file for mutual authentication.
 	 * By default mutual authentication is disabled.
-	 *
 	 * Use as_config_tls_set_certfile() to set this field.
+	 * If certfile is populated, certstring is ignored.
 	 */
 	char* certfile;
 
+	/**
+	 * Client's certificate chain file string for mutual authentication.
+	 * By default mutual authentication is disabled.
+	 * Use as_config_tls_set_certstring() to set this field.
+	 * If certfile is populated, certstring is ignored.
+	 */
+	char* certstring;
+
 	/**
 	 * Enable CRL checking for the certificate chain leaf certificate.
 	 * An error occurs if a suitable CRL cannot be found.
@@ -421,9 +445,11 @@ typedef struct as_config_s {
 	char password[AS_PASSWORD_SIZE];
 	
 	/**
-	 * Expected cluster name.  If not null, server nodes must return this cluster name in order to
-	 * join the client's view of the cluster. Should only be set when connecting to servers that
-	 * support the "cluster-name" info command.  Use as_config_set_cluster_name() to set this field.
+	 * Expected cluster name.  If populated, the cluster_name must match the cluster-name field
+	 * in the service section in each server configuration.  This ensures that the specified
+	 * seed nodes belong to the expected cluster on startup.  If not, the client will refuse
+	 * to add the node to the client's view of the cluster.
+	 *
 	 * Default: NULL
 	 */
 	char* cluster_name;
@@ -726,7 +752,7 @@ typedef struct as_config_s {
 	 * Each shared memory segment contains state for one Aerospike cluster.  If there are
 	 * multiple Aerospike clusters, a different shm_key must be defined for each cluster.
 	 * 
-	 * Default: 0xA8000000
+	 * Default: 0xA9000000
 	 */
 	int shm_key;
 	
@@ -754,9 +780,9 @@ typedef struct as_config_s {
 	uint32_t shm_takeover_threshold_sec;
 } as_config;
 
-/******************************************************************************
- * FUNCTIONS
- *****************************************************************************/
+//---------------------------------
+// Functions
+//---------------------------------
 
 /**
  * Initialize the configuration to default values.
@@ -890,6 +916,17 @@ as_config_tls_set_cafile(as_config* config, const char* cafile)
 	as_config_set_string(&config->tls.cafile, cafile);
 }
 
+/**
+ * Set string containing trusted CA certificate(s).
+ *
+ * @relates as_config
+ */
+static inline void
+as_config_tls_set_castring(as_config* config, const char* castring)
+{
+	as_config_set_string(&config->tls.castring, castring);
+}
+
 /**
  * Set TLS path to a directory of trusted certificates.
  *
@@ -956,6 +993,17 @@ as_config_tls_set_keyfile_pw(as_config* config, const char* pw)
 	as_config_set_string(&config->tls.keyfile_pw, pw);
 }
 
+/**
+ * Set client's key string for mutual authentication.
+ *
+ * @relates as_config
+ */
+static inline void
+as_config_tls_set_keystring(as_config* config, const char* keystring)
+{
+	as_config_set_string(&config->tls.keystring, keystring);
+}
+
 /**
  * Set TLS path to the client's certificate chain file for mutual authentication.
  *
@@ -967,6 +1015,17 @@ as_config_tls_set_certfile(as_config* config, const char* certfile)
 	as_config_set_string(&config->tls.certfile, certfile);
 }
 
+/**
+ * Set client's certificate chain file string for mutual authentication.
+ *
+ * @relates as_config
+ */
+static inline void
+as_config_tls_set_certstring(as_config* config, const char* certstring)
+{
+	as_config_set_string(&config->tls.certstring, certstring);
+}
+
 /**
  * Add TLS host to seed the cluster.
  * The host address and TLS name will be copied.
