@@ -1,5 +1,5 @@
 /*
- * Copyright 2008-2022 Aerospike, Inc.
+ * Copyright 2008-2024 Aerospike, Inc.
  *
  * Portions may be licensed to Aerospike, Inc. under one or more contributor
  * license agreements.
@@ -57,17 +57,14 @@ extern "C" {
 #define AS_ASYNC_STATE_QUEUE_ERROR 11
 #define AS_ASYNC_STATE_RETRY 12
 
-#define AS_ASYNC_FLAGS_MASTER 1
+#define AS_ASYNC_FLAGS_DESERIALIZE 1
 #define AS_ASYNC_FLAGS_READ 2
 #define AS_ASYNC_FLAGS_HAS_TIMER 4
 #define AS_ASYNC_FLAGS_USING_SOCKET_TIMER 8
 #define AS_ASYNC_FLAGS_EVENT_RECEIVED 16
 #define AS_ASYNC_FLAGS_FREE_BUF 32
 #define AS_ASYNC_FLAGS_LINEARIZE 64
-#define AS_ASYNC_FLAGS_MASTER_SC 128
-
-#define AS_ASYNC_FLAGS2_DESERIALIZE 1
-#define AS_ASYNC_FLAGS2_HEAP_REC 2
+#define AS_ASYNC_FLAGS_HEAP_REC 128
 
 #define AS_ASYNC_AUTH_RETURN_CODE 1
 
@@ -143,6 +140,7 @@ typedef struct as_event_command {
 	cf_ll_element pipe_link;
 	
 	uint8_t* buf;
+	uint64_t begin; // Used for metrics
 	uint32_t command_sent_counter;
 	uint32_t write_offset;
 	uint32_t write_len;
@@ -155,7 +153,11 @@ typedef struct as_event_command {
 	uint8_t proto_type_rcv;
 	uint8_t state;
 	uint8_t flags;
-	uint8_t flags2;
+	uint8_t replica_size;
+	uint8_t replica_index;
+	uint8_t replica_index_sc; // Used in batch only.
+	as_latency_type latency_type;
+	bool metrics_enabled;
 } as_event_command;
 
 typedef struct {
@@ -190,6 +192,9 @@ as_event_command_execute(as_event_command* cmd, as_error* err);
 void
 as_event_command_schedule(as_event_command* cmd);
 
+void
+as_event_connection_complete(as_event_command* cmd);
+
 bool
 as_event_proto_parse(as_event_command* cmd, as_proto* proto);
 
@@ -762,7 +767,7 @@ as_event_release_async_connection(as_event_command* cmd)
 {
 	as_async_conn_pool* pool = &cmd->node->async_conn_pools[cmd->event_loop->index];
 	as_event_release_connection(cmd->conn, pool);
-	as_node_incr_error_count(cmd->node);
+	as_node_incr_error_rate(cmd->node);
 }
 
 static inline void
@@ -784,7 +789,7 @@ as_event_connection_timeout(as_event_command* cmd, as_async_conn_pool* pool)
 		if (conn->watching > 0) {
 			as_event_stop_watcher(cmd, conn);
 			as_event_release_connection(conn, pool);
-			as_node_incr_error_count(cmd->node);
+			as_node_incr_error_rate(cmd->node);
 		}
 		else {
 			cf_free(conn);
