@@ -1,5 +1,5 @@
 /*
- * Copyright 2008-2022 Aerospike, Inc.
+ * Copyright 2008-2024 Aerospike, Inc.
  *
  * Portions may be licensed to Aerospike, Inc. under one or more contributor
  * license agreements.
@@ -59,9 +59,9 @@
 extern "C" {
 #endif
 
-/******************************************************************************
- * MACROS
- *****************************************************************************/
+//---------------------------------
+// Macros
+//---------------------------------
 
 /**
  * Default socket idle timeout value
@@ -133,9 +133,9 @@ extern "C" {
  */
 #define AS_POLICY_COMMIT_LEVEL_DEFAULT AS_POLICY_COMMIT_LEVEL_ALL
 
-/******************************************************************************
- * TYPES
- *****************************************************************************/
+//---------------------------------
+// Types
+//---------------------------------
 
 struct as_exp;
 
@@ -172,18 +172,18 @@ typedef enum as_policy_retry_e {
 typedef enum as_policy_gen_e {
 
 	/**
-	 * Write a record, regardless of generation.
+	 * Do not use record generation to restrict writes.
 	 */
 	AS_POLICY_GEN_IGNORE,
 
 	/**
-	 * Write a record, ONLY if generations are equal
+	 * Update/delete record if expected generation is equal to server generation. Otherwise, fail.
 	 */
 	AS_POLICY_GEN_EQ,
 
 	/**
-	 * Write a record, ONLY if local generation is 
-	 * greater-than remote generation
+	 * Update/delete record if expected generation greater than the server generation. 
+	 * Otherwise, fail. This is useful for restore after backup.
 	 */
 	AS_POLICY_GEN_GT
 
@@ -266,7 +266,6 @@ typedef enum as_policy_exists_e {
  * Replica Policy
  *
  * Defines algorithm used to determine the target node for a command.
- * Scan and query are not affected by this replica algorithm.
  *
  * @ingroup client_policies
  */
@@ -279,7 +278,7 @@ typedef enum as_policy_replica_e {
 
 	/**
 	 * Distribute reads across nodes containing key's master and replicated partition
-	 * in round-robin fashion.  Currently restricted to master and one prole.
+	 * in round-robin fashion.
 	 */
 	AS_POLICY_REPLICA_ANY,
 
@@ -288,7 +287,6 @@ typedef enum as_policy_replica_e {
 	 * If connection fails, all commands try nodes containing replicated partitions.
 	 * If socketTimeout is reached, reads also try nodes containing replicated partitions,
 	 * but writes remain on master node.
-	 * Currently restricted to master and one prole.
 	 */
 	AS_POLICY_REPLICA_SEQUENCE,
 
@@ -383,6 +381,49 @@ typedef enum as_policy_commit_level_e {
 
 } as_policy_commit_level;
 
+/**
+ * Expected query duration. The server treats the query in different ways depending on the expected duration.
+ * This enum is ignored for aggregation queries, background queries and server versions &lt; 6.0.
+ *
+ * @ingroup client_policies
+ */
+typedef enum as_query_duration_e {
+
+	/**
+	 * The query is expected to return more than 100 records per node. The server optimizes for a
+	 * large record set in the following ways:
+	 * <ul>
+	 * <li>Allow query to be run in multiple threads using the server's query threading configuration.</li>
+	 * <li>Do not relax read consistency for AP namespaces.</li>
+	 * <li>Add the query to the server's query monitor.</li>
+	 * <li>Do not add the overall latency to the server's latency histogram.</li>
+	 * <li>Do not allow server timeouts.</li>
+	 * </ul>
+	 */
+	AS_QUERY_DURATION_LONG,
+
+	/**
+	 * The query is expected to return less than 100 records per node. The server optimizes for a
+	 * small record set in the following ways:
+	 * <ul>
+	 * <li>Always run the query in one thread and ignore the server's query threading configuration.</li>
+	 * <li>Allow query to be inlined directly on the server's service thread.</li>
+	 * <li>Relax read consistency for AP namespaces.</li>
+	 * <li>Do not add the query to the server's query monitor.</li>
+	 * <li>Add the overall latency to the server's latency histogram.</li>
+	 * <li>Allow server timeouts. The default server timeout for a short query is 1 second.</li>
+	 * </ul>
+	 */
+	AS_QUERY_DURATION_SHORT,
+	
+	/**
+	 * Treat query as a LONG query, but relax read consistency for AP namespaces.
+	 * This value is treated exactly like LONG for server versions &lt; 7.1.
+	 */
+	AS_QUERY_DURATION_LONG_RELAX_AP
+
+} as_query_duration;
+
 /**
  * Generic policy fields shared among all policies.
  *
@@ -535,6 +576,27 @@ typedef struct as_policy_read_s {
 	 */
 	as_policy_read_mode_sc read_mode_sc;
 
+	/**
+	 * Determine how record TTL (time to live) is affected on reads. When enabled, the server can
+	 * efficiently operate as a read-based LRU cache where the least recently used records are expired.
+	 * The value is expressed as a percentage of the TTL sent on the most recent write such that a read
+	 * within this interval of the record’s end of life will generate a touch.
+	 *
+	 * For example, if the most recent write had a TTL of 10 hours and read_touch_ttl_percent is set to
+	 * 80, the next read within 8 hours of the record's end of life (equivalent to 2 hours after the most
+	 * recent write) will result in a touch, resetting the TTL to another 10 hours.
+	 *
+	 * Values:
+	 * <ul>
+	 * <li> 0 : Use server config default-read-touch-ttl-pct for the record's namespace/set.</li>
+	 * <li>-1 : Do not reset record TTL on reads.</li>
+	 * <li>1 - 100 : Reset record TTL on reads when within this percentage of the most recent write TTL.</li>
+	 * </ul>
+	 *
+	 * Default: 0
+	 */
+	int read_touch_ttl_percent;
+	
 	/**
 	 * Should raw bytes representing a list or map be deserialized to as_list or as_map.
 	 * Set to false for backup programs that just need access to raw bytes.
@@ -593,6 +655,20 @@ typedef struct as_policy_write_s {
 	 */
 	as_policy_exists exists;
 	
+	/**
+	 * The default time-to-live (expiration) of the record in seconds. This field will 
+	 * only be used if "as_record.ttl" is set to AS_RECORD_CLIENT_DEFAULT_TTL. The
+	 * as_record instance is passed in to write functions along with as_policy_write.
+	 *
+	 * There are also special values that can be set in the record ttl:
+	 * <ul>
+	 * <li>AS_RECORD_DEFAULT_TTL: Use the server default ttl from the namespace.</li>
+	 * <li>AS_RECORD_NO_EXPIRE_TTL: Do not expire the record.</li>
+	 * <li>AS_RECORD_NO_CHANGE_TTL: Keep the existing record ttl when the record is updated.</li>
+	 * </ul>
+	 */
+	uint32_t ttl;
+
 	/**
 	 * Minimum record size beyond which it is compressed and sent to the server.
 	 */
@@ -638,18 +714,15 @@ typedef struct as_policy_apply_s {
 	as_policy_commit_level commit_level;
 
 	/**
-	 * The time-to-live (expiration) of the record in seconds.
-	 * There are also special values that can be set in the record TTL:
-	 * (*) ZERO (defined as AS_RECORD_DEFAULT_TTL), which means that the
-	 *    record will adopt the default TTL value from the namespace.
-	 * (*) 0xFFFFFFFF (also, -1 in a signed 32 bit int)
-	 *    (defined as AS_RECORD_NO_EXPIRE_TTL), which means that the record
-	 *    will get an internal "void_time" of zero, and thus will never expire.
-	 * (*) 0xFFFFFFFE (also, -2 in a signed 32 bit int)
-	 *    (defined as AS_RECORD_NO_CHANGE_TTL), which means that the record
-	 *    ttl will not change when the record is updated.
+	 * The time-to-live (expiration) of the record in seconds. Note that ttl
+	 * is only used on write/update calls.
 	 *
-	 * Note that the TTL value will be employed ONLY on write/update calls.
+	 * There are also special values that can be set in the record ttl:
+	 * <ul>
+	 * <li>AS_RECORD_DEFAULT_TTL: Use the server default ttl from the namespace.</li>
+	 * <li>AS_RECORD_NO_EXPIRE_TTL: Do not expire the record.</li>
+	 * <li>AS_RECORD_NO_CHANGE_TTL: Keep the existing record ttl when the record is updated.</li>
+	 * </ul>
 	 */
 	uint32_t ttl;
 
@@ -714,6 +787,42 @@ typedef struct as_policy_operate_s {
 	 */
 	as_policy_exists exists;
 
+	/**
+	 * The default time-to-live (expiration) of the record in seconds. This field will 
+	 * only be used if one or more of the  operations is a write operation and  if "as_operations.ttl"
+	 * is set to AS_RECORD_CLIENT_DEFAULT_TTL. The as_operations instance is passed in to
+	 * operate functions along with as_policy_operate.
+	 *
+	 * There are also special values that can be set in the record ttl:
+	 * <ul>
+	 * <li>AS_RECORD_DEFAULT_TTL: Use the server default ttl from the namespace.</li>
+	 * <li>AS_RECORD_NO_EXPIRE_TTL: Do not expire the record.</li>
+	 * <li>AS_RECORD_NO_CHANGE_TTL: Keep the existing record ttl when the record is updated.</li>
+	 * </ul>
+	 */
+	uint32_t ttl;
+
+	/**
+	 * Determine how record TTL (time to live) is affected on reads. When enabled, the server can
+	 * efficiently operate as a read-based LRU cache where the least recently used records are expired.
+	 * The value is expressed as a percentage of the TTL sent on the most recent write such that a read
+	 * within this interval of the record’s end of life will generate a touch.
+	 *
+	 * For example, if the most recent write had a TTL of 10 hours and read_touch_ttl_percent is set to
+	 * 80, the next read within 8 hours of the record's end of life (equivalent to 2 hours after the most
+	 * recent write) will result in a touch, resetting the TTL to another 10 hours.
+	 *
+	 * Values:
+	 * <ul>
+	 * <li> 0 : Use server config default-read-touch-ttl-pct for the record's namespace/set.</li>
+	 * <li>-1 : Do not reset record TTL on reads.</li>
+	 * <li>1 - 100 : Reset record TTL on reads when within this percentage of the most recent write TTL.</li>
+	 * </ul>
+	 *
+	 * Default: 0
+	 */
+	int read_touch_ttl_percent;
+
 	/**
 	 * Should raw bytes representing a list or map be deserialized to as_list or as_map.
 	 * Set to false for backup programs that just need access to raw bytes.
@@ -821,6 +930,27 @@ typedef struct as_policy_batch_s {
 	 */
 	as_policy_read_mode_sc read_mode_sc;
 
+	/**
+	 * Determine how record TTL (time to live) is affected on reads. When enabled, the server can
+	 * efficiently operate as a read-based LRU cache where the least recently used records are expired.
+	 * The value is expressed as a percentage of the TTL sent on the most recent write such that a read
+	 * within this interval of the record’s end of life will generate a touch.
+	 *
+	 * For example, if the most recent write had a TTL of 10 hours and read_touch_ttl_percent is set to
+	 * 80, the next read within 8 hours of the record's end of life (equivalent to 2 hours after the most
+	 * recent write) will result in a touch, resetting the TTL to another 10 hours.
+	 *
+	 * Values:
+	 * <ul>
+	 * <li> 0 : Use server config default-read-touch-ttl-pct for the record's namespace/set.</li>
+	 * <li>-1 : Do not reset record TTL on reads.</li>
+	 * <li>1 - 100 : Reset record TTL on reads when within this percentage of the most recent write TTL.</li>
+	 * </ul>
+	 *
+	 * Default: 0
+	 */
+	int read_touch_ttl_percent;
+
 	/**
 	 * Determine if batch commands to each server are run in parallel threads.
 	 *
@@ -921,7 +1051,7 @@ typedef struct as_policy_batch_read_s {
 	 * transaction is ignored. This can be used to eliminate a client/server roundtrip
 	 * in some cases.
 	 *
-	 * aerospike_destroy() automatically calls as_exp_destroy() on all global default 
+	 * aerospike_destroy() automatically calls as_exp_destroy() on all global default
 	 * policy filter expression instances. The user is responsible for calling as_exp_destroy()
 	 * on filter expressions when setting temporary transaction policies.
 	 *
@@ -941,6 +1071,27 @@ typedef struct as_policy_batch_read_s {
 	 */
 	as_policy_read_mode_sc read_mode_sc;
 
+	/**
+	 * Determine how record TTL (time to live) is affected on reads. When enabled, the server can
+	 * efficiently operate as a read-based LRU cache where the least recently used records are expired.
+	 * The value is expressed as a percentage of the TTL sent on the most recent write such that a read
+	 * within this interval of the record’s end of life will generate a touch.
+	 *
+	 * For example, if the most recent write had a TTL of 10 hours and read_touch_ttl_percent is set to
+	 * 80, the next read within 8 hours of the record's end of life (equivalent to 2 hours after the most
+	 * recent write) will result in a touch, resetting the TTL to another 10 hours.
+	 *
+	 * Values:
+	 * <ul>
+	 * <li> 0 : Use server config default-read-touch-ttl-pct for the record's namespace/set.</li>
+	 * <li>-1 : Do not reset record TTL on reads.</li>
+	 * <li>1 - 100 : Reset record TTL on reads when within this percentage of the most recent write TTL.</li>
+	 * </ul>
+	 *
+	 * Default: 0
+	 */
+	int read_touch_ttl_percent;
+
 } as_policy_batch_read;
 
 /**
@@ -953,7 +1104,7 @@ typedef struct as_policy_batch_write_s {
 	 * transaction is ignored. This can be used to eliminate a client/server roundtrip
 	 * in some cases.
 	 *
-	 * aerospike_destroy() automatically calls as_exp_destroy() on all global default 
+	 * aerospike_destroy() automatically calls as_exp_destroy() on all global default
 	 * policy filter expression instances. The user is responsible for calling as_exp_destroy()
 	 * on filter expressions when setting temporary transaction policies.
 	 *
@@ -982,6 +1133,20 @@ typedef struct as_policy_batch_write_s {
 	 */
 	as_policy_exists exists;
 
+	/**
+	 * The default time-to-live (expiration) of the record in seconds. This field will only be
+	 * used if "as_operations.ttl" is set to AS_RECORD_CLIENT_DEFAULT_TTL. The as_operations
+	 * instance is passed in to batch write functions along with as_policy_batch_write.
+	 *
+	 * There are also special values that can be set in the record ttl:
+	 * <ul>
+	 * <li>AS_RECORD_DEFAULT_TTL: Use the server default ttl from the namespace.</li>
+	 * <li>AS_RECORD_NO_EXPIRE_TTL: Do not expire the record.</li>
+	 * <li>AS_RECORD_NO_CHANGE_TTL: Keep the existing record ttl when the record is updated.</li>
+	 * </ul>
+	 */
+	uint32_t ttl;
+
 	/**
 	 * If the transaction results in a record deletion, leave a tombstone for the record.
 	 * This prevents deleted records from reappearing after node failures.
@@ -1023,18 +1188,15 @@ typedef struct as_policy_batch_apply_s {
 	as_policy_commit_level commit_level;
 
 	/**
-	 * The time-to-live (expiration) of the record in seconds.
-	 * There are also special values that can be set in the record TTL:
-	 * (*) ZERO (defined as AS_RECORD_DEFAULT_TTL), which means that the
-	 *    record will adopt the default TTL value from the namespace.
-	 * (*) 0xFFFFFFFF (also, -1 in a signed 32 bit int)
-	 *    (defined as AS_RECORD_NO_EXPIRE_TTL), which means that the record
-	 *    will get an internal "void_time" of zero, and thus will never expire.
-	 * (*) 0xFFFFFFFE (also, -2 in a signed 32 bit int)
-	 *    (defined as AS_RECORD_NO_CHANGE_TTL), which means that the record
-	 *    ttl will not change when the record is updated.
+	 * The time-to-live (expiration) of the record in seconds. Note that ttl
+	 * is only used on write/update calls.
 	 *
-	 * Note that the TTL value will be employed ONLY on write/update calls.
+	 * There are also special values that can be set in the record ttl:
+	 * <ul>
+	 * <li>AS_RECORD_DEFAULT_TTL: Use the server default ttl from the namespace.</li>
+	 * <li>AS_RECORD_NO_EXPIRE_TTL: Do not expire the record.</li>
+	 * <li>AS_RECORD_NO_CHANGE_TTL: Keep the existing record ttl when the record is updated.</li>
+	 * </ul>
 	 */
 	uint32_t ttl;
 
@@ -1119,6 +1281,19 @@ typedef struct as_policy_query_s {
 	 */
 	uint32_t info_timeout;
 
+	/**
+	 * Algorithm used to determine target node.
+	 */
+	as_policy_replica replica;
+	
+	/**
+	 * Expected query duration. The server treats the query in different ways depending on the expected duration.
+	 * This field is ignored for aggregation queries, background queries and server versions &lt; 6.0.
+	 *
+	 * Default: AS_QUERY_DURATION_LONG
+	 */
+	as_query_duration expected_duration;
+
 	/**
 	 * Terminate query if cluster is in migration state. If the server supports partition
 	 * queries or the query filter is null (scan), this field is ignored.
@@ -1136,12 +1311,20 @@ typedef struct as_policy_query_s {
 	bool deserialize;
 
 	/**
-	 * Is query expected to return less than 100 records.
+	 * This field is deprecated and will eventually be removed. Use expected_duration instead.
+	 *
+	 * For backwards compatibility: If short_query is true, the query is treated as a short query and
+	 * expected_duration is ignored. If short_query is false, expected_duration is used
+	 * and defaults to AS_QUERY_DURATION_LONG.
+	 *
+	 * Is query expected to return less than 100 records per node.
 	 * If true, the server will optimize the query for a small record set.
 	 * This field is ignored for aggregation queries, background queries
 	 * and server versions &lt; 6.0.
 	 *
 	 * Default: false
+	 *
+	 * @deprecated Use expected_duration instead.
 	 */
 	bool short_query;
 
@@ -1177,6 +1360,24 @@ typedef struct as_policy_scan_s {
 	 */
 	uint32_t records_per_second;
 
+	/**
+	 * Algorithm used to determine target node.
+	 */
+	as_policy_replica replica;
+
+	/**
+	 * The default time-to-live (expiration) of the record in seconds. This field will only be
+	 * used on background scan writes if "as_scan.ttl" is set to AS_RECORD_CLIENT_DEFAULT_TTL.
+	 *
+	 * There are also special values that can be set in the record ttl:
+	 * <ul>
+	 * <li>AS_RECORD_DEFAULT_TTL: Use the server default ttl from the namespace.</li>
+	 * <li>AS_RECORD_NO_EXPIRE_TTL: Do not expire the record.</li>
+	 * <li>AS_RECORD_NO_CHANGE_TTL: Keep the existing record ttl when the record is updated.</li>
+	 * </ul>
+	 */
+	uint32_t ttl;
+
 	/**
 	 * If the transaction results in a record deletion, leave a tombstone for the record.
 	 * This prevents deleted records from reappearing after node failures.
@@ -1231,7 +1432,7 @@ typedef struct as_policy_admin_s {
  * 
  * This is utilized by as_config to define default values for policies.
  *
- * @ingroup as_config_t
+ * @ingroup client_policies
  */
 typedef struct as_policies_s {
 
@@ -1307,9 +1508,9 @@ typedef struct as_policies_s {
 
 } as_policies;
 
-/******************************************************************************
- * FUNCTIONS
- *****************************************************************************/
+//---------------------------------
+// Functions
+//---------------------------------
 
 /**
  * Initialize base defaults for reads.
@@ -1382,6 +1583,7 @@ as_policy_read_init(as_policy_read* p)
 	p->replica = AS_POLICY_REPLICA_DEFAULT;
 	p->read_mode_ap = AS_POLICY_READ_MODE_AP_DEFAULT;
 	p->read_mode_sc = AS_POLICY_READ_MODE_SC_DEFAULT;
+	p->read_touch_ttl_percent = 0;
 	p->deserialize = true;
 	p->async_heap_rec = false;
 	return p;
@@ -1418,6 +1620,7 @@ as_policy_write_init(as_policy_write* p)
 	p->commit_level = AS_POLICY_COMMIT_LEVEL_DEFAULT;
 	p->gen = AS_POLICY_GEN_DEFAULT;
 	p->exists = AS_POLICY_EXISTS_DEFAULT;
+	p->ttl = 0; // AS_RECORD_DEFAULT_TTL
 	p->compression_threshold = AS_POLICY_COMPRESSION_THRESHOLD_DEFAULT;
 	p->durable_delete = false;
 	return p;
@@ -1456,6 +1659,8 @@ as_policy_operate_init(as_policy_operate* p)
 	p->commit_level = AS_POLICY_COMMIT_LEVEL_DEFAULT;
 	p->gen = AS_POLICY_GEN_DEFAULT;
 	p->exists = AS_POLICY_EXISTS_DEFAULT;
+	p->ttl = 0; // AS_RECORD_DEFAULT_TTL
+	p->read_touch_ttl_percent = 0;
 	p->deserialize = true;
 	p->durable_delete = false;
 	p->async_heap_rec = false;
@@ -1560,6 +1765,7 @@ as_policy_batch_init(as_policy_batch* p)
 	p->replica = AS_POLICY_REPLICA_SEQUENCE;
 	p->read_mode_ap = AS_POLICY_READ_MODE_AP_DEFAULT;
 	p->read_mode_sc = AS_POLICY_READ_MODE_SC_DEFAULT;
+	p->read_touch_ttl_percent = 0;
 	p->concurrent = false;
 	p->allow_inline = true;
 	p->allow_inline_ssd = false;
@@ -1609,6 +1815,7 @@ as_policy_batch_read_init(as_policy_batch_read* p)
 	p->filter_exp = NULL;
 	p->read_mode_ap = AS_POLICY_READ_MODE_AP_DEFAULT;
 	p->read_mode_sc = AS_POLICY_READ_MODE_SC_DEFAULT;
+	p->read_touch_ttl_percent = 0;
 	return p;
 }
 
@@ -1624,6 +1831,7 @@ as_policy_batch_write_init(as_policy_batch_write* p)
 	p->commit_level = AS_POLICY_COMMIT_LEVEL_DEFAULT;
 	p->gen = AS_POLICY_GEN_DEFAULT;
 	p->exists = AS_POLICY_EXISTS_DEFAULT;
+	p->ttl = 0; // AS_RECORD_DEFAULT_TTL
 	p->durable_delete = false;
 	return p;
 }
@@ -1638,7 +1846,7 @@ as_policy_batch_apply_init(as_policy_batch_apply* p)
 	p->filter_exp = NULL;
 	p->key = AS_POLICY_KEY_DEFAULT;
 	p->commit_level = AS_POLICY_COMMIT_LEVEL_DEFAULT;
-	p->ttl = 0;
+	p->ttl = 0; // AS_RECORD_DEFAULT_TTL
 	p->durable_delete = false;
 	return p;
 }
@@ -1673,6 +1881,8 @@ as_policy_scan_init(as_policy_scan* p)
 	as_policy_base_query_init(&p->base);
 	p->max_records = 0;
 	p->records_per_second = 0;
+	p->replica = AS_POLICY_REPLICA_SEQUENCE;
+	p->ttl = 0; // AS_RECORD_DEFAULT_TTL
 	p->durable_delete = false;
 	return p;
 }
@@ -1704,6 +1914,8 @@ as_policy_query_init(as_policy_query* p)
 {
 	as_policy_base_query_init(&p->base);
 	p->info_timeout = 10000;
+	p->replica = AS_POLICY_REPLICA_SEQUENCE;
+	p->expected_duration = AS_QUERY_DURATION_LONG;
 	p->fail_on_cluster_change = false;
 	p->deserialize = true;
 	p->short_query = false;
