@@ -1,5 +1,5 @@
 /*
- * Copyright 2008-2022 Aerospike, Inc.
+ * Copyright 2008-2024 Aerospike, Inc.
  *
  * Portions may be licensed to Aerospike, Inc. under one or more contributor
  * license agreements.
@@ -29,88 +29,137 @@
 extern "C" {
 #endif
 
-/******************************************************************************
- * MACROS
- *****************************************************************************/
+//---------------------------------
+// Macros
+//---------------------------------
 
 /**
- * Macro for setting setting the STRING_EQUAL predicate.
+ * Filter on string bins.
  *
  * ~~~~~~~~~~{.c}
  * as_query_where(query, "bin1", as_string_equals("abc"));
  * ~~~~~~~~~~
  *
  * @relates as_query
+ * @ingroup query_operations
  */
 #define as_string_equals(__val) AS_PREDICATE_EQUAL, AS_INDEX_TYPE_DEFAULT, AS_INDEX_STRING, __val
 
 /**
- * Macro for setting setting the INTEGER_EQUAL predicate.
+ * Filter on blob bins.
+ * Requires server version 7.0+.
+ *
+ * ~~~~~~~~~~{.c}
+ * // as_blob_equals(uint8_t* bytes, uint32_t size, bool free)
+ * as_query_where(query, "bin1", as_blob_equals(bytes, size, true));
+ * ~~~~~~~~~~
+ *
+ * @relates as_query
+ * @ingroup query_operations
+ */
+#define as_blob_equals(__val, __size, __free) AS_PREDICATE_EQUAL, AS_INDEX_TYPE_DEFAULT, AS_INDEX_BLOB, __val, __size, __free
+
+/**
+ * Filter on integer bins.
  *
  * ~~~~~~~~~~{.c}
  * as_query_where(query, "bin1", as_integer_equals(123));
  * ~~~~~~~~~~
  *
  * @relates as_query
+ * @ingroup query_operations
  */
 #define as_integer_equals(__val) AS_PREDICATE_EQUAL, AS_INDEX_TYPE_DEFAULT, AS_INDEX_NUMERIC, (int64_t)__val
 
 /**
- * Macro for setting setting the INTEGER_RANGE predicate.
+ * Ranger filter on integer bins.
  *
  * ~~~~~~~~~~{.c}
  * as_query_where(query, "bin1", as_integer_range(1,100));
  * ~~~~~~~~~~
  * 
  * @relates as_query
- * @ingroup query_object
+ * @ingroup query_operations
  */
 #define as_integer_range(__min, __max) AS_PREDICATE_RANGE, AS_INDEX_TYPE_DEFAULT, AS_INDEX_NUMERIC, (int64_t)__min, (int64_t)__max
 
 /**
- * Macro for setting setting the RANGE predicate.
+ * Range filter on list/map elements.
  *
  * ~~~~~~~~~~{.c}
  * as_query_where(query, "bin1", as_range(LIST,NUMERIC,1,100));
  * ~~~~~~~~~~
  * 
  * @relates as_query
- * @ingroup query_object
+ * @ingroup query_operations
  */
 #define as_range(indextype, datatype, __min, __max) AS_PREDICATE_RANGE, AS_INDEX_TYPE_ ##indextype, AS_INDEX_ ##datatype, __min, __max
 
 /**
- * Macro for setting setting the CONTAINS predicate.
+ * Contains filter on list/map elements.
  *
  * ~~~~~~~~~~{.c}
  * as_query_where(query, "bin1", as_contains(LIST,STRING,"val"));
  * ~~~~~~~~~~
  * 
  * @relates as_query
- * @ingroup query_object
+ * @ingroup query_operations
  */
 #define as_contains(indextype, datatype, __val) AS_PREDICATE_EQUAL, AS_INDEX_TYPE_ ##indextype, AS_INDEX_ ##datatype, __val
 
 /**
- * Macro for setting setting the EQUALS predicate.
+ * Contains blob filter on list/map elements.
+ * Requires server version 7.0+.
+ *
+ * ~~~~~~~~~~{.c}
+ * // as_blob_contains(type, uint8_t* bytes, uint32_t size, bool free)
+ * as_query_where(query, "bin1", as_blob_equals(LIST, bytes, size, true));
+ * ~~~~~~~~~~
+ *
+ * @relates as_query
+ * @ingroup query_operations
+ */
+#define as_blob_contains(indextype, __val, __size, __free) AS_PREDICATE_EQUAL, AS_INDEX_TYPE_ ##indextype, AS_INDEX_BLOB, __val, __size, __free
+
+/**
+ * Filter specified type on bins.
  *
  * ~~~~~~~~~~{.c}
  * as_query_where(query, "bin1", as_equals(NUMERIC,5));
  * ~~~~~~~~~~
  * 
  * @relates as_query
- * @ingroup query_object
+ * @ingroup query_operations
  */
 #define as_equals(datatype, __val) AS_PREDICATE_EQUAL, AS_INDEX_TYPE_DEFAULT, AS_INDEX_ ##datatype, __val
 
+/**
+ * Within filter on GEO bins.
+ *
+ * ~~~~~~~~~~{.c}
+ * as_query_where(query, "bin1", as_geo_within(region));
+ * ~~~~~~~~~~
+ * 
+ * @relates as_query
+ * @ingroup query_operations
+ */
 #define as_geo_within(__val) AS_PREDICATE_RANGE, AS_INDEX_TYPE_DEFAULT, AS_INDEX_GEO2DSPHERE, __val
 
+/**
+ * Contains filter on GEO bins.
+ *
+ * ~~~~~~~~~~{.c}
+ * as_query_where(query, "bin1", as_geo_contains(region));
+ * ~~~~~~~~~~
+ * 
+ * @relates as_query
+ * @ingroup query_operations
+ */
 #define as_geo_contains(__val) AS_PREDICATE_RANGE, AS_INDEX_TYPE_DEFAULT, AS_INDEX_GEO2DSPHERE, __val
 
-
-/******************************************************************************
- * TYPES 	
- *****************************************************************************/
+//---------------------------------
+// Types
+//---------------------------------
 
 struct as_operations_s;
 
@@ -118,34 +167,23 @@ struct as_operations_s;
  * Union of supported predicates
  */
 typedef union as_predicate_value_u {
-	
-	/**
-	 * String Value
-	 */
-	char * string;
-
-	/**
-	 * Integer Value
-	 */
 	int64_t integer;
 
-	/**
-	 * Integer Range Value
-	 */
 	struct {
+		char* string;
+		bool _free;
+	} string_val;
 
-		/**
-		 * Minimum value
-		 */
+	struct {
+		uint8_t* bytes;
+		uint32_t bytes_size;
+		bool _free;
+	} blob_val;
+
+	struct {
 		int64_t min;
-
-		/**
-		 * Maximum value
-		 */
 		int64_t max;
-
 	} integer_range;
-
 } as_predicate_value;
 
 /**
@@ -183,6 +221,11 @@ typedef struct as_predicate_s {
 	 */
 	uint32_t ctx_size;
 
+	/**
+	 * Should ctx be destroyed on as_query_destroy(). Default: false.
+	 */
+	bool ctx_free;
+
 	/**
 	 * The predicate type, dictates which values to use from the union
 	 */
@@ -304,7 +347,7 @@ typedef struct as_query_predicates_s {
 	/**
 	 * Sequence of entries
 	 */
-	as_predicate * 	entries;
+	as_predicate* entries;
 
 } as_query_predicates;
 
@@ -413,7 +456,7 @@ typedef struct as_query_predicates_s {
  * as_query_apply(query, "udf_module", "udf_function", arglist);
  * ~~~~~~~~~~
  *
- * @ingroup client_objects
+ * @ingroup query_operations
  */
 typedef struct as_query_s {
 
@@ -464,7 +507,7 @@ typedef struct as_query_s {
 	as_query_predicates where;
 
 	/**
-	 * UDF to apply to results of the background query.
+	 * UDF to apply to results of a background query or a foreground aggregation query.
 	 *
 	 * Should be set via `as_query_apply()`.
 	 */
@@ -500,18 +543,16 @@ typedef struct as_query_s {
 	uint32_t records_per_second;
 
 	/**
-	 * The time-to-live (expiration) of the record in seconds.
-	 * There are also special values that can be set in the record TTL:
-	 * (*) ZERO (defined as AS_RECORD_DEFAULT_TTL), which means that the
-	 *    record will adopt the default TTL value from the namespace.
-	 * (*) 0xFFFFFFFF (also, -1 in a signed 32 bit int)
-	 *    (defined as AS_RECORD_NO_EXPIRE_TTL), which means that the record
-	 *    will get an internal "void_time" of zero, and thus will never expire.
-	 * (*) 0xFFFFFFFE (also, -2 in a signed 32 bit int)
-	 *    (defined as AS_RECORD_NO_CHANGE_TTL), which means that the record
-	 *    ttl will not change when the record is updated.
+	 * The time-to-live (expiration) of the record in seconds. Note that ttl
+	 * is only used on background query writes.
 	 *
-	 * Note that the TTL value will be employed ONLY on background query writes.
+	 * There are also special values that can be set in the record ttl:
+	 * <ul>
+	 * <li>AS_RECORD_DEFAULT_TTL: Use the server default ttl from the namespace.</li>
+	 * <li>AS_RECORD_NO_EXPIRE_TTL: Do not expire the record.</li>
+	 * <li>AS_RECORD_NO_CHANGE_TTL: Keep the existing record ttl when the record is updated.</li>
+	 * <li>AS_RECORD_CLIENT_DEFAULT_TTL: Use the default client ttl in as_policy_write.</li>
+	 * </ul>
 	 */
 	uint32_t ttl;
 
@@ -531,9 +572,9 @@ typedef struct as_query_s {
 
 } as_query;
 
-/******************************************************************************
- * INSTANCE FUNCTIONS
- *****************************************************************************/
+//---------------------------------
+// Instance Functions
+//---------------------------------
 
 /**
  * Initialize a stack allocated as_query.
@@ -550,6 +591,7 @@ typedef struct as_query_s {
  * @return On success, the initialized query. Otherwise NULL.
  *
  * @relates as_query
+ * @ingroup query_operations
  */
 AS_EXTERN as_query*
 as_query_init(as_query* query, const char* ns, const char* set);
@@ -567,7 +609,7 @@ as_query_init(as_query* query, const char* ns, const char* set);
  * @return On success, the new query. Otherwise NULL.
  *
  * @relates as_query
- * @ingroup query_object
+ * @ingroup query_operations
  */
 AS_EXTERN as_query*
 as_query_new(const char* ns, const char* set);
@@ -576,14 +618,14 @@ as_query_new(const char* ns, const char* set);
  * Destroy the query and associated resources.
  * 
  * @relates as_query
- * @ingroup query_object
+ * @ingroup query_operations
  */
 AS_EXTERN void
 as_query_destroy(as_query* query);
 
-/******************************************************************************
- * SELECT FUNCTIONS
- *****************************************************************************/
+//---------------------------------
+// Select Functions
+//---------------------------------
 
 /** 
  * Initializes `as_query.select` with a capacity of `n` using `alloca`
@@ -600,7 +642,7 @@ as_query_destroy(as_query* query);
  * @param __n		The number of bins to allocate.
  *
  * @relates as_query
- * @ingroup query_object
+ * @ingroup query_operations
  */
 #define as_query_select_inita(__query, __n) \
 	do { \
@@ -631,7 +673,7 @@ as_query_destroy(as_query* query);
  * @return On success, the initialized. Otherwise an error occurred.
  *
  * @relates as_query
- * @ingroup query_object
+ * @ingroup query_operations
  */
 AS_EXTERN bool
 as_query_select_init(as_query* query, uint16_t n);
@@ -648,20 +690,19 @@ as_query_select_init(as_query* query, uint16_t n);
  * as_query_select(&query, "bin2");
  * ~~~~~~~~~~
  *
- * @param query 	The query to modify.
+ * @param query		The query to modify.
  * @param bin 		The name of the bin to select.
  *
  * @return On success, true. Otherwise an error occurred.
- *
  * @relates as_query
- * @ingroup query_object
+ * @ingroup query_operations
  */
 AS_EXTERN bool
 as_query_select(as_query* query, const char * bin);
 
-/******************************************************************************
- * WHERE FUNCTIONS
- *****************************************************************************/
+//---------------------------------
+// Where Functions
+//---------------------------------
 
 /** 
  * Initializes `as_query.where` with a capacity of `n` using `alloca()`.
@@ -679,6 +720,7 @@ as_query_select(as_query* query, const char * bin);
  * @return On success, true. Otherwise an error occurred.
  *
  * @relates as_query
+ * @ingroup query_operations
  */
 #define as_query_where_inita(__query, __n) \
 	do { \
@@ -708,6 +750,7 @@ as_query_select(as_query* query, const char * bin);
  * @return On success, true. Otherwise an error occurred.
  *
  * @relates as_query
+ * @ingroup query_operations
  */
 AS_EXTERN bool
 as_query_where_init(as_query* query, uint16_t n);
@@ -739,6 +782,7 @@ as_query_where_init(as_query* query, uint16_t n);
  * @return On success, true. Otherwise an error occurred.
  *
  * @relates as_query
+ * @ingroup query_operations
  */
 AS_EXTERN bool
 as_query_where(
@@ -777,6 +821,7 @@ as_query_where(
  * @return On success, true. Otherwise an error occurred.
  *
  * @relates as_query
+ * @ingroup query_operations
  */
 AS_EXTERN bool
 as_query_where_with_ctx(
@@ -784,9 +829,9 @@ as_query_where_with_ctx(
 	as_index_type itype, as_index_datatype dtype, ...
 	);
 
-/******************************************************************************
- * QUERY MODIFIER FUNCTIONS
- *****************************************************************************/
+//---------------------------------
+// Background Query Functions
+//---------------------------------
 
 /**
  * Apply a function to the results of the query.
@@ -803,19 +848,20 @@ as_query_where_with_ctx(
  * @return On success, true. Otherwise an error occurred.
  *
  * @relates as_query
+ * @ingroup query_operations
  */
 AS_EXTERN bool
 as_query_apply(as_query* query, const char* module, const char* function, const as_list* arglist);
 
-/******************************************************************************
- * QUERY PAGINATE
- *****************************************************************************/
+//---------------------------------
+// Paginate Functions
+//---------------------------------
 
 /**
  * Set if records should be read in pages in conjunction with max_records policy.
  * 
  * @relates as_query
- * @ingroup as_query_object
+ * @ingroup query_operations
  */
 static inline void
 as_query_set_paginate(as_query* query, bool paginate)
@@ -828,7 +874,7 @@ as_query_set_paginate(as_query* query, bool paginate)
  * The query will resume from this point.
  *
  * @relates as_query
- * @ingroup as_query_object
+ * @ingroup query_operations
  */
 static inline void
 as_query_set_partitions(as_query* query, as_partitions_status* parts_all)
@@ -841,7 +887,7 @@ as_query_set_partitions(as_query* query, as_partitions_status* parts_all)
  * return all records?
  *
  * @relates as_query
- * @ingroup as_query_object
+ * @ingroup query_operations
  */
 static inline bool
 as_query_is_done(as_query* query)
@@ -849,6 +895,52 @@ as_query_is_done(as_query* query)
 	return query->parts_all && query->parts_all->done;
 }
 
+//---------------------------------
+// Serialization Functions
+//---------------------------------
+
+/**
+ * Serialize query definition to bytes.
+ *
+ * @relates as_query
+ * @ingroup query_operations
+ */
+AS_EXTERN bool
+as_query_to_bytes(const as_query* query, uint8_t** bytes, uint32_t* bytes_size);
+
+/**
+ * Deserialize bytes to query definition. Query definition is assumed to be on the stack.
+ * as_query_destroy() should be called when done with the query definition.
+ *
+ * @returns true on success and false on failure.
+ *
+ * @relates as_query
+ * @ingroup query_operations
+ */
+AS_EXTERN bool
+as_query_from_bytes(as_query* query, const uint8_t* bytes, uint32_t bytes_size);
+
+/**
+ * Create query definition on the heap and deserialize bytes to that query definition.
+ * as_query_destroy() should be called when done with the query definition.
+ *
+ * @returns query definition on success and NULL on failure.
+ * 
+ * @relates as_query
+ * @ingroup query_operations
+ */
+AS_EXTERN as_query*
+as_query_from_bytes_new(const uint8_t* bytes, uint32_t bytes_size);
+
+/**
+ * Compare query objects.
+ * @private
+ * @relates as_query
+ * @ingroup query_operations
+ */
+AS_EXTERN bool
+as_query_compare(as_query* q1, as_query* q2);
+
 #ifdef __cplusplus
 } // end extern "C"
 #endif
