@@ -1,5 +1,5 @@
 /*
- * Copyright 2008-2022 Aerospike, Inc.
+ * Copyright 2008-2023 Aerospike, Inc.
  *
  * Portions may be licensed to Aerospike, Inc. under one or more contributor
  * license agreements.
@@ -186,17 +186,14 @@ typedef struct as_record_s {
 
 	/**
 	 * The time-to-live (expiration) of the record in seconds.
-	 * There are also special values that can be set in the record TTL:
-	 * (*) ZERO (defined as AS_RECORD_DEFAULT_TTL), which means that the
-	 *     record will adopt the default TTL value from the namespace.
-	 * (*) 0xFFFFFFFF (also, -1 in a signed 32 bit int)
-	 *     (defined as AS_RECORD_NO_EXPIRE_TTL), which means that the record
-	 *     will get an internal "void_time" of zero, and thus will never expire.
-	 * (*) 0xFFFFFFFE (also, -2 in a signed 32 bit int)
-	 *     (defined as AS_RECORD_NO_CHANGE_TTL), which means that the record
-	 *     ttl will not change when the record is updated.
 	 *
-	 * Note that the TTL value will be employed ONLY on write/update calls.
+	 * There are also special values that can be set in the record ttl:
+	 * <ul>
+	 * <li>AS_RECORD_DEFAULT_TTL: Use the server default ttl from the namespace.</li>
+	 * <li>AS_RECORD_NO_EXPIRE_TTL: Do not expire the record.</li>
+	 * <li>AS_RECORD_NO_CHANGE_TTL: Keep the existing record ttl when the record is updated.</li>
+	 * <li>AS_RECORD_CLIENT_DEFAULT_TTL: Use the default client ttl in as_policy_write.</li>
+	 * </ul>
 	 */
 	uint32_t ttl;
 
@@ -208,24 +205,28 @@ typedef struct as_record_s {
 } as_record;
 
 /**
- * When the record is given a TTL value of ZERO, it will adopt the TTL value
- * that is the default TTL value for the namespace (defined in the config file).
+ * Use the server default ttl for the namespace in the aerospke server config file.
  */
 #define AS_RECORD_DEFAULT_TTL 0
 
 /**
- * When the record is given a TTL value of 0xFFFFFFFF, it will set the internal
- * void_time value (the absolute clock time value that shows when a record
- * will expire) to zero, which means the record will never expire
+ * Set the internal void_time value (the absolute clock time value that shows when a record
+ * will expire) to zero, which means the record will never expire.
  */
 #define AS_RECORD_NO_EXPIRE_TTL 0xFFFFFFFF
 
 /**
- * When the record is given a TTL value of 0xFFFFFFFE, the TTL will not change
- * when a record is updated.
+ * Do not change the  ttl when the record is updated.
  */
 #define AS_RECORD_NO_CHANGE_TTL 0xFFFFFFFE
 
+/**
+ * Use the applicable client policy ttl (as_policy_write, as_policy_operate, as_policy_scan,
+ * as_policy_batch_write, ...). If the policy is not defined for the transaction, use the default
+ * in as_config.policies.
+ */
+#define AS_RECORD_CLIENT_DEFAULT_TTL 0xFFFFFFFD
+
 /******************************************************************************
  * MACROS
  *****************************************************************************/
