@@ -1,5 +1,5 @@
 /*
- * Copyright 2008-2022 Aerospike, Inc.
+ * Copyright 2008-2024 Aerospike, Inc.
  *
  * Portions may be licensed to Aerospike, Inc. under one or more contributor
  * license agreements.
@@ -44,6 +44,7 @@
 typedef struct as_read_info_s {
 	as_policy_replica replica;
 	uint8_t flags;
+	uint8_t replica_index;
 } as_read_info;
 
 /******************************************************************************
@@ -79,6 +80,8 @@ as_command_init_read(
 	cmd->udata = udata;
 	cmd->buf_size = size;
 	cmd->partition_id = pi->partition_id;
+	cmd->latency_type = AS_LATENCY_TYPE_READ;
+	as_cluster_add_tran(cluster);
 
 	if (pi->sc_mode) {
 		switch (read_mode_sc) {
@@ -103,7 +106,8 @@ as_command_init_read(
 		cmd->replica = replica;
 		cmd->flags = AS_COMMAND_FLAGS_READ;
 	}
-	cmd->master = as_command_target_master(cmd->replica);
+	cmd->replica_size = pi->replica_size;
+	cmd->replica_index = as_replica_index_init_read(cmd->replica);
 }
 
 static inline as_status
@@ -139,7 +143,10 @@ as_command_init_write(
 	cmd->partition_id = pi->partition_id;
 	cmd->flags = 0;
 	cmd->replica = as_command_write_replica(replica);
-	cmd->master = true;
+	cmd->replica_size = pi->replica_size;
+	cmd->replica_index = 0;
+	cmd->latency_type = AS_LATENCY_TYPE_WRITE;
+	as_cluster_add_tran(cluster);
 }
 
 static inline void
@@ -171,9 +178,7 @@ as_event_command_init_read(
 		ri->flags =  AS_ASYNC_FLAGS_READ;
 	}
 
-	if (as_command_target_master(ri->replica)) {
-		ri->flags |= AS_ASYNC_FLAGS_MASTER;
-	}
+	ri->replica_index = as_replica_index_init_read(ri->replica);
 }
 
 static inline uint32_t
@@ -224,7 +229,8 @@ aerospike_key_get(
 	uint8_t* buf = as_command_buffer_init(size);
 	uint32_t timeout = as_command_server_timeout(&policy->base);
 	uint8_t* p = as_command_write_header_read(buf, &policy->base, policy->read_mode_ap,
-		policy->read_mode_sc, timeout, n_fields, 0, AS_MSG_INFO1_READ | AS_MSG_INFO1_GET_ALL, 0);
+		policy->read_mode_sc, policy->read_touch_ttl_percent, timeout, n_fields, 0,
+		AS_MSG_INFO1_READ | AS_MSG_INFO1_GET_ALL, 0, 0);
 
 	p = as_command_write_key(p, policy->key, key);
 	p = as_command_write_filter(&policy->base, filter_size, p);
@@ -269,13 +275,14 @@ aerospike_key_get_async(
 	size += filter_size;
 
 	as_event_command* cmd = as_async_record_command_create(
-		cluster, &policy->base, ri.replica, pi.ns, pi.partition, policy->deserialize,
-		policy->async_heap_rec, ri.flags, listener, udata, event_loop, pipe_listener,
-		size, as_event_command_parse_result);
+		cluster, &policy->base, &pi, ri.replica, ri.replica_index, policy->deserialize,
+		policy->async_heap_rec, ri.flags, listener, udata, event_loop, pipe_listener, size,
+		as_event_command_parse_result, AS_LATENCY_TYPE_READ);
 
 	uint32_t timeout = as_command_server_timeout(&policy->base);
 	uint8_t* p = as_command_write_header_read(cmd->buf, &policy->base, policy->read_mode_ap,
-		policy->read_mode_sc, timeout, n_fields, 0, AS_MSG_INFO1_READ | AS_MSG_INFO1_GET_ALL, 0);
+		policy->read_mode_sc, policy->read_touch_ttl_percent, timeout, n_fields, 0,
+		AS_MSG_INFO1_READ | AS_MSG_INFO1_GET_ALL, 0, 0);
 
 	p = as_command_write_key(p, policy->key, key);
 	p = as_command_write_filter(&policy->base, filter_size, p);
@@ -323,7 +330,8 @@ aerospike_key_select(
 	uint8_t* buf = as_command_buffer_init(size);
 	uint32_t timeout = as_command_server_timeout(&policy->base);
 	uint8_t* p = as_command_write_header_read(buf, &policy->base, policy->read_mode_ap,
-				policy->read_mode_sc, timeout, n_fields, nvalues, AS_MSG_INFO1_READ, 0);
+				policy->read_mode_sc, policy->read_touch_ttl_percent, timeout, n_fields, nvalues,
+				AS_MSG_INFO1_READ, 0, 0);
 
 	p = as_command_write_key(p, policy->key, key);
 	p = as_command_write_filter(&policy->base, filter_size, p);
@@ -381,13 +389,14 @@ aerospike_key_select_async(
 	}
 
 	as_event_command* cmd = as_async_record_command_create(
-		cluster, &policy->base, ri.replica, pi.ns, pi.partition, policy->deserialize,
-		policy->async_heap_rec, ri.flags, listener, udata, event_loop, pipe_listener,
-		size, as_event_command_parse_result);
+		cluster, &policy->base, &pi, ri.replica, ri.replica_index, policy->deserialize,
+		policy->async_heap_rec, ri.flags, listener, udata, event_loop, pipe_listener, size,
+		as_event_command_parse_result, AS_LATENCY_TYPE_READ);
 
 	uint32_t timeout = as_command_server_timeout(&policy->base);
 	uint8_t* p = as_command_write_header_read(cmd->buf, &policy->base, policy->read_mode_ap,
-					policy->read_mode_sc, timeout, n_fields, nvalues, AS_MSG_INFO1_READ, 0);
+					policy->read_mode_sc, policy->read_touch_ttl_percent, timeout, n_fields, nvalues,
+					AS_MSG_INFO1_READ, 0, 0);
 
 	p = as_command_write_key(p, policy->key, key);
 	p = as_command_write_filter(&policy->base, filter_size, p);
@@ -427,7 +436,8 @@ aerospike_key_exists(
 
 	uint8_t* buf = as_command_buffer_init(size);
 	uint8_t* p = as_command_write_header_read_header(buf, &policy->base, policy->read_mode_ap,
-		policy->read_mode_sc, n_fields, 0, AS_MSG_INFO1_READ | AS_MSG_INFO1_GET_NOBINDATA);
+		policy->read_mode_sc, policy->read_touch_ttl_percent, n_fields, 0,
+		AS_MSG_INFO1_READ | AS_MSG_INFO1_GET_NOBINDATA);
 
 	p = as_command_write_key(p, policy->key, key);
 	p = as_command_write_filter(&policy->base, filter_size, p);
@@ -472,12 +482,13 @@ aerospike_key_exists_async(
 	size += filter_size;
 
 	as_event_command* cmd = as_async_record_command_create(
-		cluster, &policy->base, ri.replica, pi.ns, pi.partition, false, policy->async_heap_rec,
-		ri.flags, listener, udata, event_loop, pipe_listener,
-		size, as_event_command_parse_result);
+		cluster, &policy->base, &pi, ri.replica, ri.replica_index, false, policy->async_heap_rec,
+		ri.flags, listener, udata, event_loop, pipe_listener, size, as_event_command_parse_result,
+		AS_LATENCY_TYPE_READ);
 
 	uint8_t* p = as_command_write_header_read_header(cmd->buf, &policy->base, policy->read_mode_ap,
-		policy->read_mode_sc, n_fields, 0, AS_MSG_INFO1_READ | AS_MSG_INFO1_GET_NOBINDATA);
+		policy->read_mode_sc, policy->read_touch_ttl_percent, n_fields, 0,
+		AS_MSG_INFO1_READ | AS_MSG_INFO1_GET_NOBINDATA);
 
 	p = as_command_write_key(p, policy->key, key);
 	p = as_command_write_filter(&policy->base, filter_size, p);
@@ -533,9 +544,10 @@ as_put_write(void* udata, uint8_t* buf)
 	as_put* put = udata;
 	const as_policy_write* policy = put->policy;
 	as_record* rec = put->rec;
+	uint32_t ttl = (rec->ttl == AS_RECORD_CLIENT_DEFAULT_TTL)? policy->ttl : rec->ttl;
 
 	uint8_t* p = as_command_write_header_write(buf, &policy->base, policy->commit_level,
-		policy->exists, policy->gen, rec->gen, rec->ttl, put->n_fields, put->n_bins,
+		policy->exists, policy->gen, rec->gen, ttl, put->n_fields, put->n_bins,
 		policy->durable_delete, 0, AS_MSG_INFO2_WRITE, 0);
 
 	p = as_command_write_key(p, policy->key, put->key);
@@ -635,7 +647,7 @@ aerospike_key_put_async_ex(
 	if (compression_threshold == 0 || (put.size <= compression_threshold)) {
 		// Send uncompressed command.
 		as_event_command* cmd = as_async_write_command_create(
-			cluster, &policy->base, policy->replica, pi.ns, pi.partition, listener, udata, event_loop,
+			cluster, &policy->base, &pi, policy->replica, listener, udata, event_loop,
 			pipe_listener, put.size, as_event_command_parse_header);
 
 		cmd->write_len = (uint32_t)as_put_write(&put, cmd->buf);
@@ -660,7 +672,7 @@ aerospike_key_put_async_ex(
 		// Allocate command with compressed upper bound.
 		size_t comp_size = as_command_compress_max_size(size);
 		as_event_command* cmd = as_async_write_command_create(
-			cluster, &policy->base, policy->replica, pi.ns, pi.partition, listener, udata, event_loop,
+			cluster, &policy->base, &pi, policy->replica, listener, udata, event_loop,
 			pipe_listener, comp_size, as_event_command_parse_header);
 
 		// Compress buffer and execute.
@@ -768,7 +780,7 @@ aerospike_key_remove_async_ex(
 	size += filter_size;
 
 	as_event_command* cmd = as_async_write_command_create(
-		cluster, &policy->base, policy->replica, pi.ns, pi.partition, listener, udata, event_loop,
+		cluster, &policy->base, &pi, policy->replica, listener, udata, event_loop,
 		pipe_listener, size, as_event_command_parse_header);
 
 	uint8_t* p = as_command_write_header_write(cmd->buf, &policy->base, policy->commit_level,
@@ -917,9 +929,20 @@ as_operate_write(void* udata, uint8_t* buf)
 	as_operate* oper = udata;
 	const as_policy_operate* policy = oper->policy;
 	const as_operations* ops = oper->ops;
+	uint32_t ttl;
+	
+	if (oper->write_attr & AS_MSG_INFO2_WRITE) {
+		ttl = (ops->ttl == AS_RECORD_CLIENT_DEFAULT_TTL)? policy->ttl : ops->ttl;
+	}
+	else {
+		// ttl is an unsigned 32 bit integer in the wire protocol, but it still
+		// works if a negative read_touch_ttl_percent is used. The server casts
+		// ttl back to a signed integer when all operations are read operations.
+		ttl = (uint32_t)policy->read_touch_ttl_percent;
+	}
 
 	uint8_t* p = as_command_write_header_write(buf, &policy->base, policy->commit_level,
-		policy->exists, policy->gen, ops->gen, ops->ttl, oper->n_fields,
+		policy->exists, policy->gen, ops->gen, ttl, oper->n_fields,
 		oper->n_operations, policy->durable_delete, oper->read_attr, oper->write_attr,
 		oper->info_attr);
 
@@ -1033,22 +1056,22 @@ aerospike_key_operate_async(
 
 	as_event_command* cmd;
 
-	if (! (policy->base.compress && oper.size > AS_COMPRESS_THRESHOLD)) {
+	if (! (policy->base.compress && oper.size > AS_COMPRESS_THRESHOLD)) {		
 		// Send uncompressed command.
 		if (oper.write_attr & AS_MSG_INFO2_WRITE) {
 			cmd = as_async_record_command_create(
-				cluster, &policy->base, policy->replica, pi.ns, pi.partition, policy->deserialize,
-				policy->async_heap_rec, AS_ASYNC_FLAGS_MASTER, listener, udata, event_loop,
-				pipe_listener, oper.size, as_event_command_parse_result);
+				cluster, &policy->base, &pi, policy->replica, 0, policy->deserialize,
+				policy->async_heap_rec, 0, listener, udata, event_loop, pipe_listener, oper.size,
+				as_event_command_parse_result, AS_LATENCY_TYPE_WRITE);
 		}
 		else {
 			as_read_info ri;
 			as_event_command_init_read(policy->replica, policy->read_mode_sc, pi.sc_mode, &ri);
 
 			cmd = as_async_record_command_create(
-				cluster, &policy->base, ri.replica, pi.ns, pi.partition, policy->deserialize,
+				cluster, &policy->base, &pi, ri.replica, ri.replica_index, policy->deserialize,
 				policy->async_heap_rec, ri.flags, listener, udata, event_loop, pipe_listener,
-				oper.size, as_event_command_parse_result);
+				oper.size, as_event_command_parse_result, AS_LATENCY_TYPE_READ);
 		}
 
 		cmd->write_len = (uint32_t)as_operate_write(&oper, cmd->buf);
@@ -1065,18 +1088,18 @@ aerospike_key_operate_async(
 
 		if (oper.write_attr & AS_MSG_INFO2_WRITE) {
 			cmd = as_async_record_command_create(
-				cluster, &policy->base, policy->replica, pi.ns, pi.partition, policy->deserialize,
-				policy->async_heap_rec, AS_ASYNC_FLAGS_MASTER, listener, udata, event_loop,
-				pipe_listener, comp_size, as_event_command_parse_result);
+				cluster, &policy->base, &pi, policy->replica, 0, policy->deserialize,
+				policy->async_heap_rec, 0, listener, udata, event_loop, pipe_listener, comp_size,
+				as_event_command_parse_result, AS_LATENCY_TYPE_WRITE);
 		}
 		else {
 			as_read_info ri;
 			as_event_command_init_read(policy->replica, policy->read_mode_sc, pi.sc_mode, &ri);
 
 			cmd = as_async_record_command_create(
-				cluster, &policy->base, ri.replica, pi.ns, pi.partition, policy->deserialize,
+				cluster, &policy->base, &pi, ri.replica, ri.replica_index, policy->deserialize,
 				policy->async_heap_rec, ri.flags, listener, udata, event_loop, pipe_listener,
-				comp_size, as_event_command_parse_result);
+				comp_size, as_event_command_parse_result, AS_LATENCY_TYPE_READ);
 		}
 
 		// Compress buffer and execute.
@@ -1216,8 +1239,8 @@ aerospike_key_apply_async(
 
 	if (! (policy->base.compress && size > AS_COMPRESS_THRESHOLD)) {
 		// Send uncompressed command.
-		as_event_command* cmd = as_async_value_command_create(cluster, &policy->base,
-			policy->replica, pi.ns, pi.partition, listener, udata, event_loop, pipe_listener, size,
+		as_event_command* cmd = as_async_value_command_create(cluster, &policy->base, &pi,
+			policy->replica, listener, udata, event_loop, pipe_listener, size,
 			as_event_command_parse_success_failure);
 
 		cmd->write_len = (uint32_t)as_apply_write(&ap, cmd->buf);
@@ -1239,9 +1262,9 @@ aerospike_key_apply_async(
 		// Allocate command with compressed upper bound.
 		size_t comp_size = as_command_compress_max_size(size);
 
-		as_event_command* cmd = as_async_value_command_create(cluster, &policy->base,
-			policy->replica, pi.ns, pi.partition, listener, udata, event_loop, pipe_listener,
-			comp_size, as_event_command_parse_success_failure);
+		as_event_command* cmd = as_async_value_command_create(cluster, &policy->base, &pi,
+			policy->replica, listener, udata, event_loop, pipe_listener, comp_size,
+			as_event_command_parse_success_failure);
 
 		// Compress buffer and execute.
 		status = as_command_compress(err, buf, size, cmd->buf, &comp_size);
