@@ -1,5 +1,5 @@
 /*
- * Copyright 2008-2022 Aerospike, Inc.
+ * Copyright 2008-2024 Aerospike, Inc.
  *
  * Portions may be licensed to Aerospike, Inc. under one or more contributor
  * license agreements.
@@ -41,6 +41,8 @@
 #include <aerospike/as_udf_context.h>
 #include <aerospike/mod_lua.h>
 
+extern bool as_op_is_write[];
+
 /******************************************************************************
  * TYPES
  *****************************************************************************/
@@ -97,6 +99,7 @@ typedef struct as_async_query_executor {
 	as_async_query_record_listener listener;
 	as_cluster* cluster;
 	as_partition_tracker* pt;
+	uint64_t parent_id;
 	uint8_t* cmd_buf;
 	uint32_t cmd_size;
 	uint32_t cmd_size_pre;
@@ -138,6 +141,12 @@ typedef struct as_query_builder {
  * STATIC FUNCTIONS
  *****************************************************************************/
 
+static inline void
+as_query_log_iter(uint64_t parent_id, uint64_t task_id, uint32_t iter)
+{
+	as_log_debug("Query parent=%" PRIu64 " task=%" PRIu64 " iter=%u", parent_id, task_id, iter);
+}
+
 static int
 as_query_aerospike_log(const as_aerospike* as, const char * file, const int line, const int level, const char * msg)
 {
@@ -320,13 +329,18 @@ as_query_parse_record_async(
 	*pp = as_command_parse_key(*pp, msg->n_fields, &rec.key, &bval);
 
 	as_status status = as_command_parse_bins(pp, err, &rec, msg->n_ops,
-		qc->command.flags2 & AS_ASYNC_FLAGS2_DESERIALIZE);
+		qc->command.flags & AS_ASYNC_FLAGS_DESERIALIZE);
 
 	if (status != AEROSPIKE_OK) {
 		as_record_destroy(&rec);
 		return status;
 	}
 
+	if (as_partition_tracker_reached_max_records_async(qe->pt, qc->np)) {
+		as_record_destroy(&rec);
+		return AEROSPIKE_OK;
+	}
+
 	bool rv = qe->listener(NULL, &rec, qe->executor.udata, qe->executor.event_loop);
 
 	if (! rv) {
@@ -457,6 +471,11 @@ as_query_parse_record(uint8_t** pp, as_msg* msg, as_query_task* task, as_error*
 			return status;
 		}
 
+		if (as_partition_tracker_reached_max_records_sync(task->pt, task->np)) {
+			as_record_destroy(&rec);
+			return AEROSPIKE_OK;
+		}
+
 		if (task->callback) {
 			bool rv = task->callback((as_val*)&rec, task->udata);
 
@@ -559,6 +578,27 @@ as_query_write_range_string(uint8_t* p, char* begin, char* end)
 	return p;
 }
 
+static uint8_t*
+as_query_write_range_blob(uint8_t* p, uint8_t* bytes, uint32_t size)
+{
+	// Write particle type.
+	*p++ = AS_BYTES_BLOB;
+	
+	// Write begin value.
+	*(uint32_t*)p = cf_swap_to_be32(size);
+	p += sizeof(uint32_t);
+	memcpy(p, bytes, size);
+	p += size;
+
+	// Write end value.
+	*(uint32_t*)p = cf_swap_to_be32(size);
+	p += sizeof(uint32_t);
+	memcpy(p, bytes, size);
+	p += size;
+
+	return p;
+}
+
 static uint8_t*
 as_query_write_range_geojson(uint8_t* p, char* begin, char* end)
 {
@@ -670,21 +710,32 @@ as_query_command_size(
 		// bin name size(1) + particle type size(1) + begin particle size(4) + end particle size(4) = 10
 		filter_size += (uint32_t)strlen(pred->bin) + 10;
 		
-		switch(pred->type) {
+		switch (pred->type) {
 			case AS_PREDICATE_EQUAL:
-				if (pred->dtype == AS_INDEX_STRING) {
-					filter_size += (uint32_t)strlen(pred->value.string) * 2;
-				}
-				else if (pred->dtype == AS_INDEX_NUMERIC) {
-					filter_size += sizeof(int64_t) * 2;
+				switch (pred->dtype) {
+					case AS_INDEX_STRING:
+						filter_size += (uint32_t)strlen(pred->value.string_val.string) * 2;
+						break;
+
+					case AS_INDEX_NUMERIC:
+						filter_size += sizeof(int64_t) * 2;
+						break;
+
+					case AS_INDEX_BLOB:
+						filter_size += pred->value.blob_val.bytes_size * 2;
+						break;
+
+					default:
+						break;
 				}
 				break;
+
 			case AS_PREDICATE_RANGE:
 				if (pred->dtype == AS_INDEX_NUMERIC) {
 					filter_size += sizeof(int64_t) * 2;
 				}
 				else if (pred->dtype == AS_INDEX_GEO2DSPHERE) {
-					filter_size += (uint32_t)strlen(pred->value.string) * 2;
+					filter_size += (uint32_t)strlen(pred->value.string_val.string) * 2;
 				}
 				break;
 		}
@@ -772,6 +823,12 @@ as_query_command_size(
 
 		for (uint16_t i = 0; i < ops->binops.size; i++) {
 			as_binop* op = &ops->binops.entries[i];
+
+			if (!as_op_is_write[op->op]) {
+				return as_error_set_message(err, AEROSPIKE_ERR_PARAM,
+					"Read operations not allowed in background query");
+			}
+
 			as_status status = as_command_bin_size(&op->bin, qb->opsbuffers, &qb->size, err);
 
 			if (status != AEROSPIKE_OK) {
@@ -803,32 +860,43 @@ as_query_command_init(
 	if (query_policy) {
 		// Foreground query.
 		uint8_t read_attr = AS_MSG_INFO1_READ;
+		uint8_t write_attr = 0;
 
 		if (query->no_bins) {
 			read_attr |= AS_MSG_INFO1_GET_NOBINDATA;
 		}
 
-		if (query_policy->short_query) {
+		if (query_policy->short_query || query_policy->expected_duration == AS_QUERY_DURATION_SHORT) {
 			read_attr |= AS_MSG_INFO1_SHORT_QUERY;
 		}
+		else if (query_policy->expected_duration == AS_QUERY_DURATION_LONG_RELAX_AP) {
+			write_attr |= AS_MSG_INFO2_RELAX_AP_LONG_QUERY;
+		}
 
 		uint8_t info_attr = qb->is_new ? AS_MSG_INFO3_PARTITION_DONE : 0;
 
 		p = as_command_write_header_read(cmd, base_policy, AS_POLICY_READ_MODE_AP_ONE,
-			AS_POLICY_READ_MODE_SC_SESSION, base_policy->total_timeout, qb->n_fields, qb->n_ops,
-			read_attr, info_attr);
+			AS_POLICY_READ_MODE_SC_SESSION, -1, base_policy->total_timeout, qb->n_fields, qb->n_ops,
+			read_attr, write_attr, info_attr);
 	}
 	else if (query->ops) {
 		// Background query with operations.
 		uint32_t ttl = (query->ttl)? query->ttl : query->ops->ttl;
+
+		if (ttl == AS_RECORD_CLIENT_DEFAULT_TTL) {
+			ttl = write_policy->ttl;
+		}
+
 		p = as_command_write_header_write(cmd, base_policy, write_policy->commit_level,
 			write_policy->exists, AS_POLICY_GEN_IGNORE, 0, ttl, qb->n_fields, qb->n_ops,
 			write_policy->durable_delete, 0, AS_MSG_INFO2_WRITE, 0);
 	}
 	else {
 		// Background query with UDF.
+		uint32_t ttl = (query->ttl == AS_RECORD_CLIENT_DEFAULT_TTL)? write_policy->ttl : query->ttl;
+
 		p = as_command_write_header_write(cmd, base_policy, write_policy->commit_level,
-			write_policy->exists, AS_POLICY_GEN_IGNORE, 0, query->ttl, qb->n_fields, qb->n_ops,
+			write_policy->exists, AS_POLICY_GEN_IGNORE, 0, ttl, qb->n_fields, qb->n_ops,
 			write_policy->durable_delete, 0, AS_MSG_INFO2_WRITE, 0);
 	}
 
@@ -877,21 +945,38 @@ as_query_command_init(
 		*len_ptr = (uint8_t)(s - (uint8_t*)pred->bin);
 		
 		// Write particle type and range values.
-		switch(pred->type) {
+		switch (pred->type) {
 			case AS_PREDICATE_EQUAL:
-				if (pred->dtype == AS_INDEX_STRING) {
-					p = as_query_write_range_string(p, pred->value.string, pred->value.string);
-				}
-				else if (pred->dtype == AS_INDEX_NUMERIC) {
-					p = as_query_write_range_integer(p, pred->value.integer, pred->value.integer);
+				switch (pred->dtype) {
+					case AS_INDEX_STRING: {
+						char* str = pred->value.string_val.string;
+						p = as_query_write_range_string(p, str, str);
+						break;
+					}
+
+					case AS_INDEX_NUMERIC: {
+						p = as_query_write_range_integer(p, pred->value.integer, pred->value.integer);
+						break;
+					}
+
+					case AS_INDEX_BLOB: {
+						p = as_query_write_range_blob(p, pred->value.blob_val.bytes, pred->value.blob_val.bytes_size);
+						break;
+					}
+
+					default: {
+						break;
+					}
 				}
 				break;
+
 			case AS_PREDICATE_RANGE:
 				if (pred->dtype == AS_INDEX_NUMERIC) {
 					p = as_query_write_range_integer(p, pred->value.integer_range.min, pred->value.integer_range.max);
 				}
 				else if (pred->dtype == AS_INDEX_GEO2DSPHERE) {
-					p = as_query_write_range_geojson(p, pred->value.string, pred->value.string);
+					char* str = pred->value.string_val.string;
+					p = as_query_write_range_geojson(p, str, str);
 				}
 				break;
 		}
@@ -1049,7 +1134,9 @@ as_query_command_execute_old(as_query_task* task)
 	cmd.partition_id = 0; // Not referenced when node set.
 	cmd.replica = AS_POLICY_REPLICA_MASTER;
 	cmd.flags = flags;
-	cmd.master = true;
+	cmd.replica_size = 1;
+	cmd.replica_index = 0;
+	cmd.latency_type = AS_LATENCY_TYPE_QUERY;
 
 	as_command_start_timer(&cmd);
 
@@ -1149,7 +1236,9 @@ as_query_command_execute_new(as_query_task* task)
 	cmd.partition_id = 0; // Not referenced when node set.
 	cmd.replica = AS_POLICY_REPLICA_MASTER;
 	cmd.flags = flags;
-	cmd.master = true;
+	cmd.replica_size = 1;
+	cmd.replica_index = 0;
+	cmd.latency_type = AS_LATENCY_TYPE_QUERY;
 
 	as_command_start_timer(&cmd);
 
@@ -1218,6 +1307,7 @@ as_query_worker_new(void* data)
 static as_status
 as_query_execute(as_query_task* task, const as_query* query, as_nodes* nodes)
 {
+	as_cluster_add_tran(task->cluster);
 	as_status status = AEROSPIKE_OK;
 
 	if (task->query_policy && task->query_policy->fail_on_cluster_change) {
@@ -1337,8 +1427,7 @@ as_query_aggregate(void* data)
 	
 	as_udf_context ctx = {
 		.as = &as,
-		.timer = NULL,
-		.memtracker = NULL
+		.timer = NULL
 	};
 
 	// The callback stream provides the ability to write to a user callback function
@@ -1386,10 +1475,13 @@ as_query_partitions(
 	as_cluster* cluster, as_error* err, const as_policy_query* policy, const as_query* query,
 	as_partition_tracker* pt, aerospike_query_foreach_callback callback, void* udata)
 {
-	as_status status;
+	as_cluster_add_tran(cluster);
+	uint64_t parent_id = as_random_get_uint64();
+	as_status status = AEROSPIKE_OK;
 
 	while (true) {
 		uint64_t task_id = as_random_get_uint64();
+		as_query_log_iter(parent_id, task_id, pt->iteration);
 		status = as_partition_tracker_assign(pt, cluster, query->ns, err);
 
 		if (status != AEROSPIKE_OK) {
@@ -1398,6 +1490,10 @@ as_query_partitions(
 
 		uint32_t n_nodes = pt->node_parts.size;
 
+		if (pt->iteration > 1) {
+			as_cluster_add_retries(cluster, n_nodes);
+		}
+
 		// Initialize task.
 		uint32_t error_mutex = 0;
 
@@ -1630,8 +1726,10 @@ as_query_partition_execute_async(
 		cmd->type = AS_ASYNC_TYPE_QUERY_PARTITION;
 		cmd->proto_type = AS_MESSAGE_TYPE;
 		cmd->state = AS_ASYNC_STATE_UNREGISTERED;
-		cmd->flags = AS_ASYNC_FLAGS_MASTER;
-		cmd->flags2 = qe->deserialize ? AS_ASYNC_FLAGS2_DESERIALIZE : 0;
+		cmd->flags = qe->deserialize ? AS_ASYNC_FLAGS_DESERIALIZE : 0;
+		cmd->replica_size = 1;
+		cmd->replica_index = 0;
+		cmd->latency_type = AS_LATENCY_TYPE_QUERY;
 		ee->commands[i] = cmd;
 	}
 
@@ -1670,6 +1768,7 @@ as_query_partition_async(
 	as_event_loop* event_loop
 	)
 {
+	as_cluster_add_tran(cluster);
 	pt->sleep_between_retries = 0;
 	as_status status = as_partition_tracker_assign(pt, cluster, query->ns, err);
 
@@ -1685,7 +1784,9 @@ as_query_partition_async(
 		as_queue_inita(&opsbuffers, sizeof(as_buffer), query->ops->binops.size);
 	}
 
+	uint64_t parent_id = as_random_get_uint64();
 	uint64_t task_id = as_random_get_uint64();
+	as_query_log_iter(parent_id, task_id, pt->iteration);
 
 	// Create command builder without partition fields.
 	// The partition fields will be added later.
@@ -1711,6 +1812,7 @@ as_query_partition_async(
 	qe->listener = listener;
 	qe->cluster = cluster;
 	qe->pt = pt;
+	qe->parent_id = parent_id;
 	qe->cmd_buf = cmd_buf;
 	qe->cmd_size = (uint32_t)cmd_size;
 	qe->cmd_size_pre = qb.cmd_size_pre;
@@ -1749,6 +1851,7 @@ as_query_partition_retry_async(as_async_query_executor* qe_old, as_error* err)
 	qe->listener = qe_old->listener;
 	qe->cluster = qe_old->cluster;
 	qe->pt = qe_old->pt;
+	qe->parent_id = qe_old->parent_id;
 	qe->cmd_buf = qe_old->cmd_buf;
 	qe->cmd_size = qe_old->cmd_size;
 	qe->cmd_size_pre = qe_old->cmd_size_pre;
@@ -1761,6 +1864,7 @@ as_query_partition_retry_async(as_async_query_executor* qe_old, as_error* err)
 
 	// Must change task_id each round. Otherwise, server rejects command.
 	uint64_t task_id = as_random_get_uint64();
+	as_query_log_iter(qe->parent_id, task_id, qe->pt->iteration);
 	*(uint64_t*)(qe->cmd_buf + qe->task_id_offset) = task_id;
 
 	uint32_t n_nodes = qe->pt->node_parts.size;
@@ -1782,6 +1886,7 @@ as_query_partition_retry_async(as_async_query_executor* qe_old, as_error* err)
 	ee->queued = 0;
 	ee->notify = true;
 	ee->valid = true;
+	as_cluster_add_retry(qe->cluster);
 
 	return as_query_partition_execute_async(qe, qe->pt, err);
 }
@@ -1858,7 +1963,7 @@ aerospike_query_foreach(
 
 		as_partition_tracker pt;
 		as_partition_tracker_init_nodes(&pt, cluster, &policy->base, query->max_records,
-			&query->parts_all, query->paginate, n_nodes);
+			policy->replica, &query->parts_all, query->paginate, n_nodes);
 
 		status = as_query_partitions(cluster, err, policy, query, &pt, callback, udata);
 
@@ -2011,7 +2116,7 @@ aerospike_query_partitions(
 
 	as_partition_tracker pt;
 	status = as_partition_tracker_init_filter(&pt, cluster, &policy->base, query->max_records,
-		&query->parts_all, query->paginate, n_nodes, pf, err);
+		policy->replica, &query->parts_all, query->paginate, n_nodes, pf, err);
 
 	if (status != AEROSPIKE_OK) {
 		return status;
@@ -2056,7 +2161,7 @@ aerospike_query_async(
 
 		as_partition_tracker* pt = cf_malloc(sizeof(as_partition_tracker));
 		as_partition_tracker_init_nodes(pt, cluster, &policy->base, query->max_records,
-			&query->parts_all, query->paginate, n_nodes);
+			policy->replica, &query->parts_all, query->paginate, n_nodes);
 		return as_query_partition_async(cluster, err, policy, query, pt, listener, udata, event_loop);
 	}
 
@@ -2070,6 +2175,8 @@ aerospike_query_async(
 	}
 
 	uint64_t task_id = as_random_get_uint64();
+	as_query_log_iter(0, task_id, 1);
+
 	as_queue opsbuffers;
 
 	if (query->ops) {
@@ -2123,6 +2230,7 @@ aerospike_query_async(
 	exec->valid = true;
 	executor->listener = listener;
 	executor->info_timeout = policy->info_timeout;
+	executor->parent_id = 0;
 
 	// Allocate enough memory to cover, then, round up memory size in 8KB increments to allow socket
 	// read to reuse buffer.
@@ -2153,8 +2261,10 @@ aerospike_query_async(
 		cmd->type = AS_ASYNC_TYPE_QUERY;
 		cmd->proto_type = AS_MESSAGE_TYPE;
 		cmd->state = AS_ASYNC_STATE_UNREGISTERED;
-		cmd->flags = AS_ASYNC_FLAGS_MASTER;
-		cmd->flags2 = policy->deserialize ? AS_ASYNC_FLAGS2_DESERIALIZE : 0;
+		cmd->flags = policy->deserialize ? AS_ASYNC_FLAGS_DESERIALIZE : 0;
+		cmd->replica_size = 1;
+		cmd->replica_index = 0;
+		cmd->latency_type = AS_LATENCY_TYPE_QUERY;
 		memcpy(cmd->buf, cmd_buf, size);
 		exec->commands[i] = cmd;
 	}
@@ -2226,7 +2336,7 @@ aerospike_query_partitions_async(
 
 	as_partition_tracker* pt = cf_malloc(sizeof(as_partition_tracker));
 	status = as_partition_tracker_init_filter(pt, cluster, &policy->base, query->max_records,
-		&query->parts_all, query->paginate, n_nodes, pf, err);
+		policy->replica, &query->parts_all, query->paginate, n_nodes, pf, err);
 
 	if (status != AEROSPIKE_OK) {
 		cf_free(pt);
