@@ -1,5 +1,5 @@
 /*
- * Copyright 2008-2022 Aerospike, Inc.
+ * Copyright 2008-2024 Aerospike, Inc.
  *
  * Portions may be licensed to Aerospike, Inc. under one or more contributor
  * license agreements.
@@ -21,7 +21,7 @@
 #include <aerospike/as_exp.h>
 #include <aerospike/as_job.h>
 #include <aerospike/as_key.h>
-#include <aerospike/as_log.h>
+#include <aerospike/as_log_macros.h>
 #include <aerospike/as_msgpack.h>
 #include <aerospike/as_operations.h>
 #include <aerospike/as_partition_tracker.h>
@@ -67,6 +67,7 @@ typedef struct as_async_scan_executor {
 	as_async_scan_listener listener;
 	as_cluster* cluster;
 	as_partition_tracker* pt;
+	uint64_t parent_id;
 	uint8_t* cmd_buf;
 	uint32_t cmd_size;
 	uint32_t cmd_size_pre;
@@ -102,6 +103,12 @@ typedef struct as_scan_builder {
  * STATIC FUNCTIONS
  *****************************************************************************/
 
+static inline void
+as_scan_log_iter(uint64_t parent_id, uint64_t task_id, uint32_t iter)
+{
+	as_log_debug("Scan parent=%" PRIu64 " task=%" PRIu64 " iter=%u", parent_id, task_id, iter);
+}
+
 static as_status
 as_scan_partition_retry_async(as_async_scan_executor* se, as_error* err);
 
@@ -187,13 +194,18 @@ as_scan_parse_record_async(
 	*pp = as_command_parse_key(*pp, msg->n_fields, &rec.key, &bval);
 
 	as_status status = as_command_parse_bins(pp, err, &rec, msg->n_ops,
-											 sc->command.flags2 & AS_ASYNC_FLAGS2_DESERIALIZE);
+											 sc->command.flags & AS_ASYNC_FLAGS_DESERIALIZE);
 
 	if (status != AEROSPIKE_OK) {
 		as_record_destroy(&rec);
 		return status;
 	}
 
+	if (as_partition_tracker_reached_max_records_async(se->pt, sc->np)) {
+		as_record_destroy(&rec);
+		return AEROSPIKE_OK;
+	}
+
 	bool rv = se->listener(0, &rec, se->executor.udata, se->executor.event_loop);
 
 	if (! rv) {
@@ -293,6 +305,11 @@ as_scan_parse_record(uint8_t** pp, as_msg* msg, as_scan_task* task, as_error* er
 		return status;
 	}
 
+	if (as_partition_tracker_reached_max_records_sync(task->pt, task->np)) {
+		as_record_destroy(&rec);
+		return AEROSPIKE_OK;
+	}
+
 	if (task->callback) {
 		bool rv = task->callback((as_val*)&rec, task->udata);
 
@@ -483,14 +500,21 @@ as_scan_command_init(
 	if (scan->ops) {
 		// Background scan with operations.
 		uint32_t ttl = (scan->ttl)? scan->ttl : scan->ops->ttl;
+
+		if (ttl == AS_RECORD_CLIENT_DEFAULT_TTL) {
+			ttl = policy->ttl;
+		}
+
 		p = as_command_write_header_write(cmd, &policy->base, AS_POLICY_COMMIT_LEVEL_ALL,
 				AS_POLICY_EXISTS_IGNORE, AS_POLICY_GEN_IGNORE, 0, ttl, sb->n_fields, n_ops,
 				policy->durable_delete, 0, AS_MSG_INFO2_WRITE, 0);
 	}
 	else if (scan->apply_each.function[0]) {
 		// Background scan with UDF.
+		uint32_t ttl = (scan->ttl == AS_RECORD_CLIENT_DEFAULT_TTL)? policy->ttl : scan->ttl;
+
 		p = as_command_write_header_write(cmd, &policy->base, AS_POLICY_COMMIT_LEVEL_ALL,
-				AS_POLICY_EXISTS_IGNORE, AS_POLICY_GEN_IGNORE, 0, scan->ttl, sb->n_fields, n_ops,
+				AS_POLICY_EXISTS_IGNORE, AS_POLICY_GEN_IGNORE, 0, ttl, sb->n_fields, n_ops,
 				policy->durable_delete, 0, AS_MSG_INFO2_WRITE, 0);
 	}
 	else {
@@ -505,8 +529,8 @@ as_scan_command_init(
 		int info_attr = cluster->has_partition_query? AS_MSG_INFO3_PARTITION_DONE : 0;
 
 		p = as_command_write_header_read(cmd, &policy->base, AS_POLICY_READ_MODE_AP_ONE,
-				AS_POLICY_READ_MODE_SC_SESSION, policy->base.total_timeout, sb->n_fields, n_ops,
-				read_attr, info_attr);
+				AS_POLICY_READ_MODE_SC_SESSION, -1, policy->base.total_timeout, sb->n_fields, n_ops,
+				read_attr, 0, info_attr);
 	}
 	
 	if (scan->ns[0]) {
@@ -661,7 +685,9 @@ as_scan_command_execute(as_scan_task* task)
 	cmd.partition_id = 0; // Not referenced when node set.
 	cmd.replica = AS_POLICY_REPLICA_MASTER;
 	cmd.flags = AS_COMMAND_FLAGS_READ;
-	cmd.master = true;
+	cmd.replica_size = 1;
+	cmd.replica_index = 0;
+	cmd.latency_type = AS_LATENCY_TYPE_QUERY;
 
 	as_command_start_timer(&cmd);
 
@@ -735,6 +761,7 @@ as_scan_generic(
 	aerospike_scan_foreach_callback callback, void* udata, uint64_t* task_id_ptr
 	)
 {
+	as_cluster_add_tran(cluster);
 	as_status status = as_scan_validate(err, policy, scan);
 
 	if (status != AEROSPIKE_OK) {
@@ -851,10 +878,13 @@ as_scan_partitions(
 	as_cluster* cluster, as_error* err, const as_policy_scan* policy, const as_scan* scan,
 	as_partition_tracker* pt, aerospike_scan_foreach_callback callback, void* udata)
 {
-	as_status status;
+	as_cluster_add_tran(cluster);
+	uint64_t parent_id = as_random_get_uint64();
+	as_status status = AEROSPIKE_OK;
 
 	while (true) {
 		uint64_t task_id = as_random_get_uint64();
+		as_scan_log_iter(parent_id, task_id, pt->iteration);
 		status = as_partition_tracker_assign(pt, cluster, scan->ns, err);
 
 		if (status != AEROSPIKE_OK) {
@@ -862,6 +892,10 @@ as_scan_partitions(
 		}
 
 		uint32_t n_nodes = pt->node_parts.size;
+		
+		if (pt->iteration > 1) {
+			as_cluster_add_retries(cluster, n_nodes);
+		}
 
 		// Initialize task.
 		uint32_t error_mutex = 0;
@@ -1061,8 +1095,10 @@ as_scan_partition_execute_async(as_async_scan_executor* se, as_partition_tracker
 		cmd->type = AS_ASYNC_TYPE_SCAN_PARTITION;
 		cmd->proto_type = AS_MESSAGE_TYPE;
 		cmd->state = AS_ASYNC_STATE_UNREGISTERED;
-		cmd->flags = AS_ASYNC_FLAGS_MASTER;
-		cmd->flags2 = se->deserialize_list_map ? AS_ASYNC_FLAGS2_DESERIALIZE : 0;
+		cmd->flags = se->deserialize_list_map ? AS_ASYNC_FLAGS_DESERIALIZE : 0;
+		cmd->replica_size = 1;
+		cmd->replica_index = 0;
+		cmd->latency_type = AS_LATENCY_TYPE_QUERY;
 		ee->commands[i] = cmd;
 	}
 
@@ -1101,6 +1137,7 @@ as_scan_partition_retry_async(as_async_scan_executor* se_old, as_error* err)
 	se->listener = se_old->listener;
 	se->cluster = se_old->cluster;
 	se->pt = se_old->pt;
+	se->parent_id = se_old->parent_id;
 	se->cmd_buf = se_old->cmd_buf;
 	se->cmd_size = se_old->cmd_size;
 	se->cmd_size_pre = se_old->cmd_size_pre;
@@ -1112,6 +1149,7 @@ as_scan_partition_retry_async(as_async_scan_executor* se_old, as_error* err)
 
 	// Must change task_id each round. Otherwise, server rejects command.
 	uint64_t task_id = as_random_get_uint64();
+	as_scan_log_iter(se->parent_id, task_id, se->pt->iteration);
 	*(uint64_t*)(se->cmd_buf + se->task_id_offset) = task_id;
 
 	uint32_t n_nodes = se->pt->node_parts.size;
@@ -1133,6 +1171,7 @@ as_scan_partition_retry_async(as_async_scan_executor* se_old, as_error* err)
 	ee->queued = 0;
 	ee->notify = true;
 	ee->valid = true;
+	as_cluster_add_retry(se->cluster);
 
 	return as_scan_partition_execute_async(se, se->pt, err);
 }
@@ -1144,6 +1183,7 @@ as_scan_partition_async(
 	as_event_loop* event_loop
 	)
 {
+	as_cluster_add_tran(cluster);
 	pt->sleep_between_retries = 0;
 	as_status status = as_partition_tracker_assign(pt, cluster, scan->ns, err);
 
@@ -1161,7 +1201,10 @@ as_scan_partition_async(
 
 	// Create scan command buffer without partition fields.
 	// The partition fields will be added later.
+	uint64_t parent_id = as_random_get_uint64();
 	uint64_t task_id = as_random_get_uint64();
+	as_scan_log_iter(parent_id, task_id, pt->iteration);
+
 	as_scan_builder sb;
 	sb.pt = NULL;
 	sb.np = NULL;
@@ -1186,6 +1229,7 @@ as_scan_partition_async(
 	se->listener = listener;
 	se->cluster = cluster;
 	se->pt = pt;
+	se->parent_id = parent_id;
 	se->cmd_buf = cmd_buf;
 	se->cmd_size = (uint32_t)cmd_size;
 	se->cmd_size_pre = sb.cmd_size_pre;
@@ -1298,7 +1342,7 @@ aerospike_scan_foreach(
 
 	as_partition_tracker pt;
 	as_partition_tracker_init_nodes(&pt, cluster, &policy->base, policy->max_records,
-		&scan->parts_all, scan->paginate, n_nodes);
+		policy->replica, &scan->parts_all, scan->paginate, n_nodes);
 
 	status = as_scan_partitions(cluster, err, policy, scan, &pt, callback, udata);
 
@@ -1337,7 +1381,7 @@ aerospike_scan_node(
 
 	as_partition_tracker pt;
 	as_partition_tracker_init_node(&pt, cluster, &policy->base, policy->max_records,
-		&scan->parts_all, scan->paginate, node);
+		policy->replica, &scan->parts_all, scan->paginate, node);
 
 	status = as_scan_partitions(cluster, err, policy, scan, &pt, callback, udata);
 
@@ -1374,7 +1418,7 @@ aerospike_scan_partitions(
 
 	as_partition_tracker pt;
 	status = as_partition_tracker_init_filter(&pt, cluster, &policy->base, policy->max_records,
-		&scan->parts_all, scan->paginate, n_nodes, pf, err);
+		policy->replica, &scan->parts_all, scan->paginate, n_nodes, pf, err);
 
 	if (status != AEROSPIKE_OK) {
 		return status;
@@ -1415,7 +1459,7 @@ aerospike_scan_async(
 
 	as_partition_tracker* pt = cf_malloc(sizeof(as_partition_tracker));
 	as_partition_tracker_init_nodes(pt, cluster, &policy->base, policy->max_records,
-		&scan->parts_all, scan->paginate, n_nodes);
+		policy->replica, &scan->parts_all, scan->paginate, n_nodes);
 
 	return as_scan_partition_async(cluster, err, policy, scan, pt, listener, udata, event_loop);
 }
@@ -1448,7 +1492,7 @@ aerospike_scan_node_async(
 
 	as_partition_tracker* pt = cf_malloc(sizeof(as_partition_tracker));
 	as_partition_tracker_init_node(pt, cluster, &policy->base, policy->max_records,
-		&scan->parts_all, scan->paginate, node);
+		policy->replica, &scan->parts_all, scan->paginate, node);
 
 	status = as_scan_partition_async(cluster, err, policy, scan, pt, listener, udata,
 									 event_loop);
@@ -1484,7 +1528,7 @@ aerospike_scan_partitions_async(
 
 	as_partition_tracker* pt = cf_malloc(sizeof(as_partition_tracker));
 	status = as_partition_tracker_init_filter(pt, cluster, &policy->base, policy->max_records,
-		&scan->parts_all, scan->paginate, n_nodes, pf, err);
+		policy->replica, &scan->parts_all, scan->paginate, n_nodes, pf, err);
 
 	if (status != AEROSPIKE_OK) {
 		cf_free(pt);
