@@ -1,5 +1,5 @@
 /*
- * Copyright 2008-2022 Aerospike, Inc.
+ * Copyright 2008-2024 Aerospike, Inc.
  *
  * Portions may be licensed to Aerospike, Inc. under one or more contributor
  * license agreements.
@@ -433,11 +433,13 @@ void
 as_event_command_execute_in_loop(as_event_loop* event_loop, as_event_command* cmd)
 {
 	// Initialize read buffer (buf) to be located after write buffer.
+	cmd->begin = 0;
 	cmd->write_offset = (uint32_t)(cmd->buf - (uint8_t*)cmd);
 	cmd->buf += cmd->write_len;
 	cmd->conn = NULL;
 	cmd->proto_type_rcv = 0;
 	cmd->event_state = &cmd->cluster->event_state[event_loop->index];
+	cmd->metrics_enabled = cmd->cluster->metrics_enabled;
 
 	if (cmd->event_state->closed) {
 		as_error err;
@@ -580,6 +582,21 @@ as_event_create_connection(as_event_command* cmd, as_async_conn_pool* pool)
 	as_event_connect(cmd, pool);
 }
 
+static inline void
+as_event_add_latency(as_event_command* cmd, as_latency_type type)
+{
+	uint64_t elapsed = cf_getns() - cmd->begin;
+	as_node_add_latency(cmd->node, type, elapsed);
+}
+
+void
+as_event_connection_complete(as_event_command* cmd)
+{
+	if (cmd->metrics_enabled) {
+		as_event_add_latency(cmd, AS_LATENCY_TYPE_CONN);
+	}
+}
+
 static void
 as_event_command_begin(as_event_loop* event_loop, as_event_command* cmd)
 {
@@ -595,7 +612,7 @@ as_event_command_begin(as_event_loop* event_loop, as_event_command* cmd)
 		// This works because the previous node is only used for pointer comparison
 		// and the previous node's contents are not examined during this call.
 		cmd->node = as_partition_get_node(cmd->cluster, cmd->ns, cmd->partition, cmd->node,
-										  cmd->replica, cmd->flags & AS_ASYNC_FLAGS_MASTER);
+										  cmd->replica, cmd->replica_size, &cmd->replica_index);
 
 		if (! cmd->node) {
 			event_loop->errors++;
@@ -611,7 +628,7 @@ as_event_command_begin(as_event_loop* event_loop, as_event_command* cmd)
 		as_node_reserve(cmd->node);
 	}
 
-	if (! as_node_valid_error_count(cmd->node)) {
+	if (! as_node_valid_error_rate(cmd->node)) {
 		event_loop->errors++;
 
 		if (as_event_command_retry(cmd, true)) {
@@ -626,6 +643,10 @@ as_event_command_begin(as_event_loop* event_loop, as_event_command* cmd)
 		return;
 	}
 
+	if (cmd->metrics_enabled) {
+		cmd->begin = cf_getns();
+	}
+
 	if (cmd->pipe_listener) {
 		as_pipe_get_connection(cmd);
 		return;
@@ -648,7 +669,7 @@ as_event_command_begin(as_event_loop* event_loop, as_event_command* cmd)
 		if (len != 0) {
 			as_log_debug("Invalid async socket from pool: %d", len);
 			as_event_release_connection(&conn->base, pool);
-			as_node_incr_error_count(cmd->node);
+			as_node_incr_error_rate(cmd->node);
 			continue;
 		}
 
@@ -805,6 +826,8 @@ as_event_socket_timeout(as_event_command* cmd)
 		return;
 	}
 
+	as_node_add_timeout(cmd->node);
+
 	if (cmd->pipe_listener) {
 		as_pipe_timeout(cmd, true);
 		return;
@@ -829,6 +852,10 @@ as_event_delay_timeout(as_event_command* cmd)
 {
 	cmd->state = AS_ASYNC_STATE_QUEUE_ERROR;
 
+	if (cmd->metrics_enabled) {
+		as_cluster_add_delay_queue_timeout(cmd->cluster);
+	}
+
 	as_error err;
 	as_error_set_message(&err, AEROSPIKE_ERR_TIMEOUT, "Delay queue timeout");
 
@@ -865,12 +892,14 @@ as_event_process_timer(as_event_command* cmd)
 void
 as_event_total_timeout(as_event_command* cmd)
 {
+	// Node should not be null at this point.
+	as_node_add_timeout(cmd->node);
+	
 	if (cmd->pipe_listener) {
 		as_pipe_timeout(cmd, false);
 		return;
 	}
 
-	// Node should not be null at this point.
 	as_event_connection_timeout(cmd, &cmd->node->async_conn_pools[cmd->event_loop->index]);
 
 	as_error err;
@@ -892,7 +921,7 @@ as_event_command_retry(as_event_command* cmd, bool timeout)
 	if (! timeout || ((cmd->flags & AS_ASYNC_FLAGS_READ) &&
 					  !(cmd->flags & AS_ASYNC_FLAGS_LINEARIZE))) {
 		// Note: SC session read will ignore this setting because it uses master only.
-		cmd->flags ^= AS_ASYNC_FLAGS_MASTER;
+		cmd->replica_index++;
 	}
 
 	// Old connection should already be closed or is closing.
@@ -962,6 +991,7 @@ as_event_execute_retry(as_event_command* cmd)
 	}
 
 	// Retry command.
+	as_cluster_add_retry(cmd->cluster);
 	as_event_command_begin(cmd->event_loop, cmd);
 }
 
@@ -978,6 +1008,10 @@ as_event_put_connection(as_event_command* cmd, as_async_conn_pool* pool)
 static inline void
 as_event_response_complete(as_event_command* cmd)
 {
+	if (cmd->metrics_enabled && cmd->latency_type != AS_LATENCY_TYPE_NONE) {
+		as_event_add_latency(cmd, cmd->latency_type);
+	}
+	
 	if (cmd->pipe_listener != NULL) {
 		as_pipe_response_complete(cmd);
 		return;
@@ -1019,11 +1053,24 @@ as_event_executor_destroy(as_event_executor* executor)
 void
 as_event_executor_error(as_event_executor* executor, as_error* err, uint32_t command_count)
 {
+	bool complete;
+
 	pthread_mutex_lock(&executor->lock);
+
 	bool first_error = executor->valid;
 	executor->valid = false;
-	executor->count += command_count;
-	bool complete = executor->count == executor->max;
+
+	if (executor->max_concurrent == 1) {
+		// Add current command that failed when running commands in sequence.
+		executor->count++;
+		complete = executor->count == executor->queued;
+	}
+	else {
+		// Add current command and any remaining commands.
+		executor->count += command_count;
+		complete = executor->count == executor->max;
+	}
+
 	pthread_mutex_unlock(&executor->lock);
 
 	if (complete) {
@@ -1053,6 +1100,8 @@ as_event_executor_cancel(as_event_executor* executor, uint32_t queued_count)
 	// Cancel group of commands that already have been queued.
 	// We are cancelling commands running in the event loop thread when this method
 	// is NOT running in the event loop thread.  Enforce thread-safety.
+	bool complete;
+
 	pthread_mutex_lock(&executor->lock);
 
 	// Do not call user listener because an error will be returned
@@ -1060,10 +1109,17 @@ as_event_executor_cancel(as_event_executor* executor, uint32_t queued_count)
 	executor->notify = false;
 	executor->valid = false;
 
-	// Add tasks that were never queued.
-	executor->count += (executor->max - queued_count);
-	
-	bool complete = executor->count == executor->max;
+	if (executor->max_concurrent == 1) {
+		// Add current task that failed when running commands in sequence.
+		executor->count++;
+		complete = executor->count == executor->queued;
+	}
+	else {
+		// Add tasks that were never queued.
+		executor->count += (executor->max - queued_count);
+		complete = executor->count == executor->max;
+	}
+
 	pthread_mutex_unlock(&executor->lock);
 
 	if (complete) {
@@ -1239,9 +1295,11 @@ as_event_response_error(as_event_command* cmd, as_error* err)
 
 	// Close socket on errors that can leave unread data in socket.
 	switch (err->code) {
+		case AEROSPIKE_ERR_CLUSTER:
 		case AEROSPIKE_ERR_DEVICE_OVERLOAD:
+			as_node_add_error(cmd->node);
+			as_node_incr_error_rate(cmd->node);
 			as_event_put_connection(cmd, pool);
-			as_node_incr_error_count(cmd->node);
 			break;
 
 		case AEROSPIKE_ERR_QUERY_ABORTED:
@@ -1251,11 +1309,27 @@ as_event_response_error(as_event_command* cmd, as_error* err)
 		case AEROSPIKE_ERR_CLIENT_ABORT:
 		case AEROSPIKE_ERR_CLIENT:
 		case AEROSPIKE_NOT_AUTHENTICATED:
+			as_node_add_error(cmd->node);
+			as_node_incr_error_rate(cmd->node);
 			as_event_release_connection(cmd->conn, pool);
-			as_node_incr_error_count(cmd->node);
+			break;
+		
+		case AEROSPIKE_ERR_TIMEOUT:
+			as_node_add_timeout(cmd->node);
+			as_event_put_connection(cmd, pool);
 			break;
 			
+		case AEROSPIKE_ERR_RECORD_NOT_FOUND:
+			// Do not increment error count on record not found.
+			// Add latency metrics instead.
+			if (cmd->metrics_enabled && cmd->latency_type != AS_LATENCY_TYPE_NONE) {
+				as_event_add_latency(cmd, cmd->latency_type);
+			}
+			as_event_put_connection(cmd, pool);
+			break;
+
 		default:
+			as_node_add_error(cmd->node);
 			as_event_put_connection(cmd, pool);
 			break;
 	}
@@ -1294,7 +1368,7 @@ as_event_command_parse_result(as_event_command* cmd)
 
 	switch (status) {
 		case AEROSPIKE_OK: {
-			if (cmd->flags2 & AS_ASYNC_FLAGS2_HEAP_REC) {
+			if (cmd->flags & AS_ASYNC_FLAGS_HEAP_REC) {
 				// Create record on heap and let user call as_record_destroy() on success.
 				as_record* rec = as_record_new(msg->n_ops);
 
@@ -1303,7 +1377,7 @@ as_event_command_parse_result(as_event_command* cmd)
 
 				p = as_command_ignore_fields(p, msg->n_fields);
 				status = as_command_parse_bins(&p, &err, rec, msg->n_ops,
-											   cmd->flags2 & AS_ASYNC_FLAGS2_DESERIALIZE);
+											   cmd->flags & AS_ASYNC_FLAGS_DESERIALIZE);
 
 				if (status == AEROSPIKE_OK) {
 					as_event_response_complete(cmd);
@@ -1331,7 +1405,7 @@ as_event_command_parse_result(as_event_command* cmd)
 				
 				p = as_command_ignore_fields(p, msg->n_fields);
 				status = as_command_parse_bins(&p, &err, &rec, msg->n_ops,
-											   cmd->flags2 & AS_ASYNC_FLAGS2_DESERIALIZE);
+											   cmd->flags & AS_ASYNC_FLAGS_DESERIALIZE);
 
 				if (status == AEROSPIKE_OK) {
 					as_event_response_complete(cmd);
@@ -1483,7 +1557,7 @@ connector_execute_command(as_event_loop* event_loop, connector_shared* cs);
 static inline void
 connector_release(as_monitor* monitor, uint32_t* loop_count)
 {
-	if (as_aaf_uint32(loop_count, -1) == 0) {
+	if (as_aaf_uint32_rls(loop_count, -1) == 0) {
 		as_monitor_notify(monitor);
 	}
 }
@@ -1577,7 +1651,9 @@ connector_execute_command(as_event_loop* event_loop, connector_shared* cs)
 	cmd->type = AS_ASYNC_TYPE_CONNECTOR;
 	cmd->proto_type = AS_MESSAGE_TYPE;
 	cmd->state = AS_ASYNC_STATE_CONNECT;
-	cmd->flags = AS_ASYNC_FLAGS_MASTER;
+	cmd->flags = 0;
+	cmd->replica_size = 1;
+	cmd->replica_index = 0;
 
 	event_loop->pending++;
 	cmd->event_state->pending++;
@@ -1732,7 +1808,7 @@ typedef struct {
 static inline void
 balancer_release(balancer_shared* bs)
 {
-	if (as_aaf_uint32(&bs->loop_count, -1) == 0) {
+	if (as_aaf_uint32_rls(&bs->loop_count, -1) == 0) {
 		as_monitor_notify(&bs->monitor);
 	}
 }
@@ -1769,7 +1845,7 @@ as_event_balance_connections_node(as_event_loop* event_loop, as_cluster* cluster
 		// Do not close idle pipeline connections because pipelines work better with a stable
 		// number of connections.
 	}
-	else if (excess < 0 && as_node_valid_error_count(node)) {
+	else if (excess < 0 && as_node_valid_error_rate(node)) {
 		create_connections(event_loop, node, pool, -excess);
 	}
 }
@@ -1779,21 +1855,11 @@ as_event_balance_connections_cluster(as_event_loop* event_loop, as_cluster* clus
 {
 	as_nodes* nodes = as_nodes_reserve(cluster);
 
-	// Reserve each node.
-	for (uint32_t i = 0; i < nodes->size; i++) {
-		as_node_reserve(nodes->array[i]);
-	}
-
 	for (uint32_t i = 0; i < nodes->size; i++) {
 		as_node* node = nodes->array[i];
 		as_event_balance_connections_node(event_loop, cluster, node);
 	}
 
-	// Release each node.
-	for (uint32_t i = 0; i < nodes->size; i++) {
-		as_node_release(nodes->array[i]);
-	}
-
 	as_nodes_release(nodes);
 }
 
@@ -1844,7 +1910,7 @@ typedef struct {
 static inline void
 balancer_release_node(balancer_shared_node* bs)
 {
-	if (as_aaf_uint32(&bs->loop_count, -1) == 0) {
+	if (as_aaf_uint32_rls(&bs->loop_count, -1) == 0) {
 		as_node_release(bs->node);
 		as_monitor_notify(&bs->monitor);
 	}
@@ -1907,7 +1973,8 @@ as_event_close_cluster_event_loop(
 {
 	event_state->closed = true;
 
-	if (as_aaf_uint32(&state->event_loop_count, -1) == 0) {
+	if (as_aaf_uint32_rls(&state->event_loop_count, -1) == 0) {
+		as_fence_acq();
 		as_cluster_destroy(state->cluster);
 
 		if (state->monitor) {
