@@ -1,5 +1,5 @@
 /*
- * Copyright 2008-2022 Aerospike, Inc.
+ * Copyright 2008-2024 Aerospike, Inc.
  *
  * Portions may be licensed to Aerospike, Inc. under one or more contributor
  * license agreements.
@@ -434,6 +434,8 @@ as_uv_command_write_start(as_event_command* cmd, uv_stream_t* stream)
 static inline void
 as_uv_command_start(as_event_command* cmd, uv_stream_t* stream)
 {
+	as_event_connection_complete(cmd);
+	
 	if (cmd->type == AS_ASYNC_TYPE_CONNECTOR) {
 		as_event_connector_success(cmd);
 		return;
@@ -935,6 +937,8 @@ as_uv_tls_command_write_start(as_event_command* cmd)
 static inline void
 as_uv_tls_command_start(as_event_command* cmd)
 {
+	as_event_connection_complete(cmd);
+
 	if (cmd->type == AS_ASYNC_TYPE_CONNECTOR) {
 		as_event_connector_success(cmd);
 		return;
@@ -1143,7 +1147,7 @@ as_uv_tls_handshake_read(uv_stream_t* stream, ssize_t nread, const uv_buf_t* buf
 		uv_read_stop(stream);
 
 		if (cmd->cluster->auth_enabled) {
-			as_session* session = (as_session*)as_load_ptr(&cmd->node->session);
+			as_session* session = as_session_load(&cmd->node->session);
 
 			if (session) {
 				as_incr_uint32(&session->ref_count);
@@ -1358,7 +1362,7 @@ as_uv_connected(uv_connect_t* req, int status)
 
 		if (!ctx) {
 			if (cmd->cluster->auth_enabled) {
-				as_session* session = (as_session*)as_load_ptr(&cmd->node->session);
+				as_session* session = as_session_load(&cmd->node->session);
 
 				if (session) {
 					as_incr_uint32(&session->ref_count);
