@@ -1,5 +1,5 @@
 /*
- * Copyright 2008-2021 Aerospike, Inc.
+ * Copyright 2008-2024 Aerospike, Inc.
  *
  * Portions may be licensed to Aerospike, Inc. under one or more contributor
  * license agreements.
@@ -23,6 +23,7 @@
 #include <aerospike/as_event_internal.h>
 #include <aerospike/as_info.h>
 #include <aerospike/as_log_macros.h>
+#include <aerospike/as_metrics.h>
 #include <aerospike/as_peers.h>
 #include <aerospike/as_queue.h>
 #include <aerospike/as_shm_cluster.h>
@@ -34,6 +35,9 @@
 // Replicas take ~2K per namespace, so this will cover most deployments:
 #define INFO_STACK_BUF_SIZE (16 * 1024)
 
+// Number of nanoseconds per millisecond
+#define NS_TO_MS 1000000
+
 /******************************************************************************
  * Function declarations.
  *****************************************************************************/
@@ -42,7 +46,7 @@ const char*
 as_cluster_get_alternate_host(as_cluster* cluster, const char* hostname);
 
 bool
-as_partition_tables_update_all(as_cluster* cluster, as_node* node, char* buf, bool has_regime);
+as_partition_tables_update_all(as_cluster* cluster, as_node* node, char* buf);
 
 static void
 as_node_create_connections(as_node* node, as_conn_pool* pool, uint32_t timeout_ms, int count);
@@ -56,11 +60,16 @@ extern uint32_t as_event_loop_capacity;
  * Functions.
  *****************************************************************************/
 
+static inline as_racks*
+as_racks_load(as_racks** racks)
+{
+	return (as_racks*)as_load_ptr((void* const*)racks);
+}
+
 static inline void
 as_racks_release(as_racks* racks)
 {
-	//as_fence_release();
-	if (as_aaf_uint32(&racks->ref_count, -1) == 0) {
+	if (as_aaf_uint32_rls(&racks->ref_count, -1) == 0) {
 		cf_free(racks);
 	}
 }
@@ -86,6 +95,48 @@ as_node_create_async_pools(uint32_t min_conns_per_node, uint32_t max_conns_per_n
 	return pools;
 }
 
+static void
+as_latency_buckets_init(as_latency_buckets* latency_buckets, uint32_t latency_columns, uint32_t latency_shift)
+{
+	latency_buckets->latency_columns = latency_columns;
+	latency_buckets->latency_shift = latency_shift;
+	latency_buckets->buckets = cf_malloc(sizeof(uint64_t) * latency_columns);
+	
+	for (uint32_t i = 0; i < latency_columns; i++) {
+		as_store_uint64(&latency_buckets->buckets[i], 0);
+	}
+}
+
+static as_node_metrics*
+as_node_metrics_init(uint32_t latency_columns, uint32_t latency_shift)
+{
+	as_node_metrics* node_metrics = (as_node_metrics*)cf_malloc(sizeof(as_node_metrics));
+	uint32_t max_latency_type = AS_LATENCY_TYPE_NONE;
+	node_metrics->latency = (as_latency_buckets*)cf_malloc(sizeof(as_latency_buckets) * max_latency_type);
+
+	for (uint32_t i = 0; i < max_latency_type; i++) {
+		as_latency_buckets_init(&node_metrics->latency[i], latency_columns, latency_shift);
+	}
+	return node_metrics;
+}
+
+void
+as_node_destroy_metrics(as_node* node)
+{
+	as_node_metrics* node_metrics = node->metrics;
+	
+	if (node_metrics) {
+		uint32_t max = AS_LATENCY_TYPE_NONE;
+		
+		for (uint32_t i = 0; i < max; i++) {
+			cf_free(node_metrics->latency[i].buckets);
+		}
+		cf_free(node_metrics->latency);
+		cf_free(node_metrics);
+		node->metrics = NULL;
+	}
+}
+
 as_node*
 as_node_create(as_cluster* cluster, as_node_info* node_info)
 {
@@ -130,12 +181,21 @@ as_node_create(as_cluster* cluster, as_node_info* node_info)
 	node->active = true;
 	node->partition_changed = true;
 	node->rebalance_changed = cluster->rack_aware;
+	node->error_rate = 0;
+	node->error_count = 0;
+	node->timeout_count = 0;
+
+	if (cluster->metrics_enabled) {
+		node->metrics = as_node_metrics_init(cluster->metrics_latency_columns, cluster->metrics_latency_shift);
+	}
+	else {
+		node->metrics = NULL;
+	}
 
 	// Create sync connection pools.
 	node->sync_conn_pools = cf_malloc(sizeof(as_conn_pool) * cluster->conn_pools_per_node);
 	node->sync_conns_opened = 1;
 	node->sync_conns_closed = 0;
-	node->error_count = 0;
 	node->conn_iter = 0;
 
 	uint32_t min = cluster->min_conns_per_node / cluster->conn_pools_per_node;
@@ -194,17 +254,19 @@ as_node_destroy(as_node* node)
 		cf_free(node->tls_name);
 	}
 
-	as_session* session = (as_session*)as_load_ptr(&node->session);
+	as_session* session = as_session_load(&node->session);
 
 	if (session) {
 		as_session_release(session);
 	}
 
-	as_racks* racks = (as_racks*)as_load_ptr(&node->racks);
+	as_racks* racks = as_racks_load(&node->racks);
 
 	if (racks) {
 		as_racks_release(racks);
 	}
+	
+	as_node_destroy_metrics(node);
 	cf_free(node);
 }
 
@@ -429,6 +491,12 @@ as_node_create_connection(
 	as_socket* sock
 	)
 {
+	uint64_t begin = 0;
+
+	if (node->cluster->metrics_enabled) {
+		begin = cf_getns();
+	}
+	
 	as_status status = as_node_create_socket(err, node, pool, sock, deadline_ms);
 
 	if (status) {
@@ -440,7 +508,7 @@ as_node_create_connection(
 
 	if (cluster->auth_enabled) {
 		// Must reserve session because not called from cluster tend thread.
-		as_session* session = (as_session*)as_load_ptr(&node->session);
+		as_session* session = as_session_load(&node->session);
 
 		if (session) {
 			as_incr_uint32(&session->ref_count);
@@ -454,6 +522,11 @@ as_node_create_connection(
 			}
 		}
 	}
+
+	if (node->cluster->metrics_enabled) {
+		uint64_t elapsed = cf_getns() - begin;
+		as_node_add_latency(node, AS_LATENCY_TYPE_CONN, elapsed);
+	}
 	return AEROSPIKE_OK;
 }
 
@@ -596,7 +669,7 @@ as_node_get_connection(as_error* err, as_node* node, uint32_t socket_timeout, ui
 static void
 as_node_close_idle_connections(as_node* node, as_conn_pool* pool, int count)
 {
-	uint64_t max_socket_idle_ns = node->cluster->max_socket_idle_ns_trim;
+	uint64_t max_socket_idle_ns = as_load_uint64(&node->cluster->max_socket_idle_ns_trim);
 	as_socket s;
 
 	while (count > 0) {
@@ -630,7 +703,7 @@ as_node_balance_connections(as_node* node)
 		if (excess > 0) {
 			as_node_close_idle_connections(node, pool, excess);
 		}
-		else if (excess < 0 && as_node_valid_error_count(node)) {
+		else if (excess < 0 && as_node_valid_error_rate(node)) {
 			as_node_create_connections(node, pool, timeout_ms, -excess);
 		}
 	}
@@ -669,7 +742,6 @@ as_node_login(as_error* err, as_node* node, as_socket* sock)
 	as_status status = as_cluster_login(cluster, err, sock, deadline_ms, &node_info);
 
 	if (status) {
-		as_fence_store();
 		as_store_uint8(&node->perform_login, 1);
 		as_error_append(err, as_node_get_address_string(node));
 		return status;
@@ -677,8 +749,7 @@ as_node_login(as_error* err, as_node* node, as_socket* sock)
 
 	as_session* old = node->session;
 
-	as_fence_store();
-	as_store_ptr(&node->session, node_info.session);
+	as_store_ptr_rls((void**)&node->session, node_info.session);
 	as_store_uint8(&node->perform_login, 0);
 
 	if (old) {
@@ -729,7 +800,7 @@ as_node_should_login(as_node* node)
 bool
 as_node_has_rack(as_node* node, const char* ns, int rack_id)
 {
-	as_racks* racks = (as_racks*)as_load_ptr(&node->racks);
+	as_racks* racks = as_racks_load(&node->racks);
 
 	if (! racks) {
 		return false;
@@ -889,7 +960,7 @@ as_node_verify_name(as_error* err, as_node* node, const char* name)
 	if (strcmp(node->name, name) != 0) {
 		// Set node to inactive immediately.
 		// Make volatile write so changes are reflected in other threads.
-		as_store_uint8(&node->active, false);
+		as_store_uint8_rls(&node->active, false);
 		return as_error_update(err, AEROSPIKE_ERR_CLIENT, "Node name has changed. Old=%s New=%s", node->name, name);
 	}
 	return AEROSPIKE_OK;
@@ -899,7 +970,7 @@ static void
 as_node_restart(as_cluster* cluster, as_node* node)
 {
 	if (cluster->max_error_rate > 0) {
-		as_node_reset_error_count(node);
+		as_node_reset_error_rate(node);
 	}
 
 	// Balance sync connections.
@@ -917,11 +988,19 @@ static as_status
 as_node_process_response(as_cluster* cluster, as_error* err, as_node* node, as_vector* values,
 						 as_peers* peers)
 {
+	as_status status;
+
 	for (uint32_t i = 0; i < values->size; i++) {
 		as_name_value* nv = as_vector_get(values, i);
 		
+		status = as_info_validate_item(err, nv->value);
+
+		if (status != AEROSPIKE_OK) {
+			return status;
+		}
+
 		if (strcmp(nv->name, "node") == 0) {
-			as_status status = as_node_verify_name(err, node, nv->value);
+			status = as_node_verify_name(err, node, nv->value);
 			
 			if (status != AEROSPIKE_OK) {
 				return status;
@@ -953,7 +1032,8 @@ as_node_process_response(as_cluster* cluster, as_error* err, as_node* node, as_v
 			}
 		}
 		else {
-			return as_error_update(err, AEROSPIKE_ERR_CLIENT, "Node %s did not request info '%s'", node->name, nv->name);
+			return as_error_update(err, AEROSPIKE_ERR_CLIENT, "Did not request info '%s'",
+				nv->name);
 		}
 	}
 	return AEROSPIKE_OK;
@@ -1122,10 +1202,7 @@ as_node_process_partitions(as_cluster* cluster, as_error* err, as_node* node, as
 			node->partition_generation = (uint32_t)strtoul(nv->value, NULL, 10);
 		}
 		else if (strcmp(nv->name, "replicas") == 0) {
-			as_partition_tables_update_all(cluster, node, nv->value, true);
-		}
-		else if (strcmp(nv->name, "replicas-all") == 0) {
-			as_partition_tables_update_all(cluster, node, nv->value, false);
+			as_partition_tables_update_all(cluster, node, nv->value);
 		}
 		else {
 			return as_error_update(err, AEROSPIKE_ERR_CLIENT, "Node %s did not request info '%s'", node->name, nv->name);
@@ -1187,8 +1264,7 @@ as_node_replace_racks(as_cluster* cluster, as_node* node, as_racks* racks)
 
 	as_racks* old = node->racks;
 
-	as_fence_store();
-	as_store_ptr(&node->racks, racks);
+	as_store_ptr_rls((void**)&node->racks, racks);
 
 	if (old) {
 		// Put old racks on garbage collector stack.
@@ -1291,12 +1367,55 @@ as_node_parse_racks(as_cluster* cluster, as_error* err, as_node* node, char* buf
 	return AEROSPIKE_OK;
 }
 
+static uint32_t
+as_latency_buckets_get_index(as_latency_buckets* latency_buckets, uint64_t elapsed_nanos)
+{
+	// Convert nanoseconds to milliseconds.
+	uint64_t elapsed = elapsed_nanos / NS_TO_MS;
+
+	// Round up elapsed to nearest millisecond.
+	if ((elapsed_nanos - (elapsed * NS_TO_MS)) > 0) {
+		elapsed++;
+	}
+
+	uint32_t last_bucket = latency_buckets->latency_columns - 1;
+	uint64_t limit = 1;
+
+	for (uint32_t i = 0; i < last_bucket; i++) {
+		if (elapsed <= limit) {
+			return i;
+		}
+		limit <<= latency_buckets->latency_shift;
+	}
+	return last_bucket;
+}
+
+void
+as_node_add_latency(as_node* node, as_latency_type latency_type, uint64_t elapsed)
+{
+	as_latency_buckets* latency_buckets = &node->metrics->latency[latency_type];
+	uint32_t index = as_latency_buckets_get_index(latency_buckets, elapsed);
+	as_incr_uint64(&latency_buckets->buckets[index]);
+}
+
+void
+as_node_enable_metrics(as_node* node, const as_metrics_policy* policy)
+{
+	 node->metrics = as_node_metrics_init(policy->latency_columns, policy->latency_shift);
+}
+
 static as_status
 as_node_process_racks(as_cluster* cluster, as_error* err, as_node* node, as_vector* values)
 {
 	for (uint32_t i = 0; i < values->size; i++) {
 		as_name_value* nv = as_vector_get(values, i);
 
+		as_status status = as_info_validate_item(err, nv->value);
+
+		if (status != AEROSPIKE_OK) {
+			return status;
+		}
+
 		if (strcmp(nv->name, "rebalance-generation") == 0) {
 			node->rebalance_generation = (uint32_t)strtoul(nv->value, NULL, 10);
 		}
@@ -1304,7 +1423,8 @@ as_node_process_racks(as_cluster* cluster, as_error* err, as_node* node, as_vect
 			return as_node_parse_racks(cluster, err, node, nv->value);
 		}
 		else {
-			return as_error_update(err, AEROSPIKE_ERR_CLIENT, "Node %s did not request info '%s'", node->name, nv->name);
+			return as_error_update(err, AEROSPIKE_ERR_CLIENT, "Did not request info '%s'",
+				nv->name);
 		}
 	}
 	return AEROSPIKE_OK;
