@@ -1,5 +1,5 @@
 /*
- * Copyright 2008-2022 Aerospike, Inc.
+ * Copyright 2008-2024 Aerospike, Inc.
  *
  * Portions may be licensed to Aerospike, Inc. under one or more contributor
  * license agreements.
@@ -19,9 +19,20 @@
 #include <aerospike/as_shm_cluster.h>
 #include <aerospike/as_string_builder.h>
 
-/******************************************************************************
- * Static Functions
- *****************************************************************************/
+//---------------------------------
+// Types
+//---------------------------------
+
+typedef struct {
+	char node_address[AS_IP_ADDRESS_SIZE];
+	as_status status;
+	uint32_t iteration;
+	uint32_t part_id;
+} query_error;
+
+//---------------------------------
+// Static Functions
+//---------------------------------
 
 static as_partitions_status*
 parts_create(uint16_t part_begin, uint16_t part_count, const as_digest* digest)
@@ -38,8 +49,16 @@ parts_create(uint16_t part_begin, uint16_t part_count, const as_digest* digest)
 	for (uint16_t i = 0; i < part_count; i++) {
 		as_partition_status* ps = &parts_all->parts[i];
 		ps->part_id = part_begin + i;
+		// as_replica_index_init_read() could be called here to support AS_POLICY_REPLICA_ANY,
+		// but AS_POLICY_REPLICA_ANY does not make sense in the scan/query context since
+		// partitions are already well distributed across nodes.  Therefore, treat
+		// AS_POLICY_REPLICA_ANY like AS_POLICY_REPLICA_SEQUENCE and start at the primary node.
+		// ps->replica_index = as_replica_index_init_read(replica);
+		ps->replica_index = 0;
 		ps->retry = true;
 		ps->digest.init = false;
+		ps->bval = 0;
+		ps->node = NULL;
 	}
 
 	if (digest && digest->init) {
@@ -51,8 +70,8 @@ parts_create(uint16_t part_begin, uint16_t part_count, const as_digest* digest)
 static void
 tracker_init(
 	as_partition_tracker* pt, const as_policy_base* policy, as_partitions_status** pp_resume,
-	uint64_t max_records, bool paginate, uint16_t part_begin, uint16_t part_count,
-	const as_digest* digest
+	uint64_t max_records, as_policy_replica replica, bool paginate, uint16_t part_begin,
+	uint16_t part_count, const as_digest* digest
 	)
 {
 	as_partitions_status* resume = *pp_resume;
@@ -74,6 +93,14 @@ tracker_init(
 		if (max_records == 0) {
 			pt->parts_all->retry = true;
 		}
+
+		// Reset replica sequence and last node used.
+		for (uint16_t i = 0; i < part_count; i++) {
+			as_partition_status* ps = &pt->parts_all->parts[i];
+			// ps->replica_index = as_replica_index_init_read(replica);
+			ps->replica_index = 0;
+			ps->node = NULL;
+		}
 	}
 
 	pthread_mutex_init(&pt->lock, NULL);
@@ -81,6 +108,9 @@ tracker_init(
 	as_vector_init(&pt->node_parts, sizeof(as_node_partitions), pt->node_capacity);
 	pt->errors = NULL;
 	pt->max_records = max_records;
+	pt->record_count = 0;
+	pt->check_max = false;
+	pt->replica = replica;
 
 	pt->sleep_between_retries = policy->sleep_between_retries;
 	pt->socket_timeout = policy->socket_timeout;
@@ -138,6 +168,45 @@ assign_partition(as_partition_tracker* pt, as_partition_status* ps, as_node* nod
 	}
 }
 
+static void
+add_error(as_partition_tracker* pt, as_node* node, as_status status, uint32_t part_id)
+{
+	query_error e;
+	as_strncpy(e.node_address, as_node_get_address_string(node), sizeof(e.node_address));
+	e.status = status;
+	e.iteration = pt->iteration;
+	e.part_id = part_id;
+	
+	// Multiple scan/query threads may call this function, so error
+	// list must be modified under lock.
+	pthread_mutex_lock(&pt->lock);
+	if (!pt->errors) {
+		pt->errors = as_vector_create(sizeof(query_error), 32);
+	}
+	as_vector_append(pt->errors, &e);
+	pthread_mutex_unlock(&pt->lock);
+}
+
+static void
+mark_retry_on_error(as_partition_tracker* pt, as_node_partitions* np)
+{
+	as_vector* list = &np->parts_full;
+
+	for (uint32_t i = 0; i < list->size; i++) {
+		as_partition_status* ps = as_partition_tracker_get_status(pt, list, i);
+		ps->retry = true;
+		ps->replica_index++;
+	}
+
+	list = &np->parts_partial;
+
+	for (uint32_t i = 0; i < list->size; i++) {
+		as_partition_status* ps = as_partition_tracker_get_status(pt, list, i);
+		ps->retry = true;
+		ps->replica_index++;
+	}
+}
+
 static void
 mark_retry(as_partition_tracker* pt, as_node_partitions* np)
 {
@@ -173,14 +242,15 @@ release_node_partitions(as_vector* list)
 	}
 }
 
-/******************************************************************************
- * Functions
- *****************************************************************************/
+//---------------------------------
+// Functions
+//---------------------------------
 
 void
 as_partition_tracker_init_nodes(
 	as_partition_tracker* pt, as_cluster* cluster, const as_policy_base* policy,
-	uint64_t max_records, as_partitions_status** parts_all, bool paginate, uint32_t cluster_size
+	uint64_t max_records, as_policy_replica replica, as_partitions_status** parts_all,
+	bool paginate, uint32_t cluster_size
 	)
 {
 	pt->node_filter = NULL;
@@ -190,26 +260,26 @@ as_partition_tracker_init_nodes(
 	uint32_t ppn = cluster->n_partitions / cluster_size;
 	ppn += ppn >> 2;
 	pt->parts_capacity = ppn;
-	tracker_init(pt, policy, parts_all, max_records, paginate, 0, cluster->n_partitions, NULL);
+	tracker_init(pt, policy, parts_all, max_records, replica, paginate, 0, cluster->n_partitions, NULL);
 }
 
 void
 as_partition_tracker_init_node(
 	as_partition_tracker* pt, as_cluster* cluster, const as_policy_base* policy,
-	uint64_t max_records, as_partitions_status** parts_all, bool paginate, as_node* node
+	uint64_t max_records, as_policy_replica replica, as_partitions_status** parts_all, bool paginate, as_node* node
 	)
 {
 	pt->node_filter = node;
 	pt->node_capacity = 1;
 	pt->parts_capacity = cluster->n_partitions;
-	tracker_init(pt, policy, parts_all, max_records, paginate, 0, cluster->n_partitions, NULL);
+	tracker_init(pt, policy, parts_all, max_records, replica, paginate, 0, cluster->n_partitions, NULL);
 }
 
 as_status
 as_partition_tracker_init_filter(
 	as_partition_tracker* pt, as_cluster* cluster, const as_policy_base* policy,
-	uint64_t max_records, as_partitions_status** parts_all, bool paginate,
-	uint32_t cluster_size, as_partition_filter* pf, as_error* err
+	uint64_t max_records, as_policy_replica replica, as_partitions_status** parts_all,
+	bool paginate, uint32_t cluster_size, as_partition_filter* pf, as_error* err
 	)
 {
 	if (pf->digest.init) {
@@ -233,7 +303,7 @@ as_partition_tracker_init_filter(
 	pt->node_filter = NULL;
 	pt->node_capacity = cluster_size;
 	pt->parts_capacity = pf->count;
-	tracker_init(pt, policy, parts_all, max_records, paginate, pf->begin, pf->count, &pf->digest);
+	tracker_init(pt, policy, parts_all, max_records, replica, paginate, pf->begin, pf->count, &pf->digest);
 	return AEROSPIKE_OK;
 }
 
@@ -242,7 +312,7 @@ as_partition_tracker_assign(
 	as_partition_tracker* pt, as_cluster* cluster, const char* ns, as_error* err
 	)
 {
-    //printf("Round %u\n", pt->iteration);
+	//printf("Round %u\n", pt->iteration);
 
 	as_partitions_status* parts_all = pt->parts_all;
 	bool retry = parts_all->retry && pt->iteration == 1;
@@ -251,20 +321,25 @@ as_partition_tracker_assign(
 		as_partition_table* table = as_partition_tables_get(&cluster->partition_tables, ns);
 
 		if (! table) {
-			return as_error_update(err, AEROSPIKE_ERR_CLIENT, "Invalid namespace: %s", ns);
+			return as_error_update(err, AEROSPIKE_ERR_NAMESPACE_NOT_FOUND, "Invalid namespace: %s",
+				ns);
 		}
 
 		for (uint16_t i = 0; i < parts_all->part_count; i++) {
 			as_partition_status* ps = &parts_all->parts[i];
 
 			if (retry || ps->retry) {
-				as_node* node = table->partitions[ps->part_id].master;
+				as_partition* part = &table->partitions[ps->part_id];
+
+				as_node* node = as_partition_get_node(cluster, ns, part, ps->node, pt->replica,
+													  table->replica_size, &ps->replica_index);
 
 				if (! node) {
 					return as_error_update(err, AEROSPIKE_ERR_INVALID_NODE,
 										   "Node not found for partition %u", ps->part_id);
 				}
 
+				ps->node = node;
 				ps->retry = false;
 
 				// Use node name to check for single node equality because
@@ -283,30 +358,25 @@ as_partition_tracker_assign(
 		as_partition_table_shm* table = as_shm_find_partition_table(cluster_shm, ns);
 
 		if (! table) {
-			return as_error_update(err, AEROSPIKE_ERR_CLIENT, "Invalid namespace: %s", ns);
+			return as_error_update(err, AEROSPIKE_ERR_NAMESPACE_NOT_FOUND, "Invalid namespace: %s",
+				ns);
 		}
 
-		as_node** local_nodes = cluster->shm_info->local_nodes;
-
 		for (uint16_t i = 0; i < parts_all->part_count; i++) {
 			as_partition_status* ps = &parts_all->parts[i];
 
 			if (retry || ps->retry) {
-				uint32_t master = as_load_uint32(&table->partitions[ps->part_id].master);
+				as_partition_shm* part = &table->partitions[ps->part_id];
 
-				// node index zero indicates unset.
-				if (master == 0) {
-					return as_error_update(err, AEROSPIKE_ERR_INVALID_NODE,
-										   "Node not found for partition %u", ps->part_id);
-				}
-
-				as_node* node = (as_node*)as_load_ptr(&local_nodes[master-1]);
+				as_node* node = as_partition_get_node(cluster, ns, part, ps->node, pt->replica,
+													  table->replica_size, &ps->replica_index);
 
 				if (! node) {
 					return as_error_update(err, AEROSPIKE_ERR_INVALID_NODE,
 										   "Node not found for partition %u", ps->part_id);
 				}
 
+				ps->node = node;
 				ps->retry = false;
 
 				// Use node name to check for single node equality because
@@ -332,34 +402,50 @@ as_partition_tracker_assign(
 	// Global retry will be set to false if the scan/query completes normally and max_records
 	// is specified.
 	parts_all->retry = true;
+	pt->record_count = 0;
+	pt->check_max = false;
 
 	if (pt->max_records > 0) {
-		// Distribute max_records across nodes.
-		if (pt->max_records < node_size) {
-			// Only include nodes that have at least 1 record requested.
-			node_size = (uint32_t)pt->max_records;
+		if (pt->max_records >= node_size) {
+			// Distribute max_records across nodes.
+			uint64_t max = pt->max_records / node_size;
+			uint32_t rem = (uint32_t)(pt->max_records - (max * node_size));
 
-			// Delete unused node partitions.
-			for (uint32_t i = node_size; i < pt->node_parts.size; i++) {
+			for (uint32_t i = 0; i < node_size; i++) {
 				as_node_partitions* np = as_vector_get(&pt->node_parts, i);
-				release_np(np);
+				np->record_max = i < rem ? max + 1 : max;
+			}
+		}
+		else {
+			// If max_records < node_size, the scan/query could consistently return 0 records even
+			// when some records are still available in nodes that were not included in the
+			// max_record distribution. Therefore, ensure each node receives at least one max record
+			// allocation and filter out excess records when receiving records from the server.
+			for (uint32_t i = 0; i < node_size; i++) {
+				as_node_partitions* np = as_vector_get(&pt->node_parts, i);
+				np->record_max = 1;
 			}
 
-			// Reset list size.
-			pt->node_parts.size = node_size;
-		}
-
-		uint64_t max = pt->max_records / node_size;
-		uint32_t rem = (uint32_t)(pt->max_records - (max * node_size));
-
-		for (uint32_t i = 0; i < node_size; i++) {
-			as_node_partitions* np = as_vector_get(&pt->node_parts, i);
-			np->record_max = i < rem ? max + 1 : max;
+			// Track records returned for this iteration.
+			pt->check_max = true;
 		}
 	}
 	return AEROSPIKE_OK;
 }
 
+void
+as_partition_tracker_part_unavailable(
+	as_partition_tracker* pt, as_node_partitions* np, uint32_t part_id
+	)
+{
+	as_partitions_status* ps = pt->parts_all;
+	as_partition_status* p = &ps->parts[part_id - ps->part_begin];
+	p->retry = true;
+	p->replica_index++;
+	np->parts_unavailable++;
+	add_error(pt, np->node, AEROSPIKE_ERR_CLUSTER, part_id);
+}
+
 as_status
 as_partition_tracker_is_complete(as_partition_tracker* pt, as_cluster* cluster, as_error* err)
 {
@@ -403,7 +489,7 @@ as_partition_tracker_is_complete(as_partition_tracker* pt, as_cluster* cluster,
 				for (uint32_t i = 0; i < list->size; i++) {
 					as_node_partitions* np = as_vector_get(list, i);
 
-					if (np->record_count >= np->record_max) {
+					if (np->retry || np->record_count >= np->record_max) {
 						mark_retry(pt, np);
 						is_done = false;
 					}
@@ -417,7 +503,7 @@ as_partition_tracker_is_complete(as_partition_tracker* pt, as_cluster* cluster,
 				for (uint32_t i = 0; i < list->size; i++) {
 					as_node_partitions* np = as_vector_get(list, i);
 
-					if (np->record_count > 0) {
+					if (np->retry || np->record_count > 0) {
 						mark_retry(pt, np);
 					}
 				}
@@ -433,25 +519,35 @@ as_partition_tracker_is_complete(as_partition_tracker* pt, as_cluster* cluster,
 
 	// Check if limits have been reached.
 	if (pt->iteration > pt->max_retries) {
-		as_error_set_message(err, AEROSPIKE_ERR_MAX_RETRIES_EXCEEDED, "");
+		// Return last sub-error code received.
+		uint32_t max = pt->errors->size;
+		query_error* last_error = as_vector_get(pt->errors, max - 1);
+		as_error_set_message(err, last_error->status, "");
 
+		// Include all sub-errors in error message.
 		as_string_builder sb;
 		as_string_builder_assign(&sb, sizeof(err->message), err->message);
-		as_string_builder_append(&sb, "Max retries exceeded: ");
-		as_string_builder_append_uint(&sb, pt->max_retries);
+		as_string_builder_append(&sb, as_error_string(err->code));
+		as_string_builder_append_newline(&sb);
+		as_string_builder_append(&sb, "sub-errors:");
 
-		if (pt->errors) {
+		for (uint32_t i = 0; i < max; i++) {
+			query_error* qe = as_vector_get(pt->errors, i);
 			as_string_builder_append_newline(&sb);
-			as_string_builder_append(&sb, "sub-errors:");
-
-			uint32_t max = pt->errors->size;
-
-			for (uint32_t i = 0; i < max; i++) {
-				as_status st = *(as_status*)as_vector_get(pt->errors, i);
-				as_string_builder_append_newline(&sb);
-				as_string_builder_append_int(&sb, st);
-				as_string_builder_append_char(&sb, ' ');
-				as_string_builder_append(&sb, as_error_string(st));
+			as_string_builder_append_int(&sb, qe->status);
+			as_string_builder_append_char(&sb, ',');
+			as_string_builder_append_uint(&sb, qe->iteration);
+			as_string_builder_append_char(&sb, ',');
+			as_string_builder_append(&sb, qe->node_address);
+			as_string_builder_append_char(&sb, ',');
+			
+			if (qe->status == AEROSPIKE_ERR_CLUSTER) {
+				as_string_builder_append(&sb, "Partition ");
+				as_string_builder_append_uint(&sb, qe->part_id);
+				as_string_builder_append(&sb, " unavailable");
+			}
+			else {
+				as_string_builder_append(&sb, as_error_string(qe->status));
 			}
 		}
 		return err->code;
@@ -491,21 +587,15 @@ as_partition_tracker_should_retry(
 	)
 {
 	switch (status) {
+	case AEROSPIKE_ERR_CLUSTER:
+	case AEROSPIKE_ERR_INVALID_NODE:
 	case AEROSPIKE_ERR_CONNECTION:
 	case AEROSPIKE_ERR_ASYNC_CONNECTION:
 	case AEROSPIKE_ERR_TIMEOUT:
 	case AEROSPIKE_ERR_INDEX_NOT_FOUND:
 	case AEROSPIKE_ERR_INDEX_NOT_READABLE:
-		// Multiple scan/query threads may call this function, so error
-		// list must be modified under lock.
-		pthread_mutex_lock(&pt->lock);
-		if (!pt->errors) {
-			pt->errors = as_vector_create(sizeof(as_status), 10);
-		}
-		as_vector_append(pt->errors, &status);
-		pthread_mutex_unlock(&pt->lock);
-
-		mark_retry(pt, np);
+		add_error(pt, np->node, status, 0);
+		mark_retry_on_error(pt, np);
 		np->parts_unavailable = np->parts_full.size + np->parts_partial.size;
 		return true;
 
@@ -517,13 +607,13 @@ as_partition_tracker_should_retry(
 void
 as_partition_tracker_destroy(as_partition_tracker* pt)
 {
-	release_node_partitions(&pt->node_parts);
-	as_vector_destroy(&pt->node_parts);
-	as_partitions_status_release(pt->parts_all);
-
 	if (pt->errors) {
 		as_vector_destroy(pt->errors);
 		pt->errors = NULL;
 	}
+
+	release_node_partitions(&pt->node_parts);
+	as_vector_destroy(&pt->node_parts);
+	as_partitions_status_release(pt->parts_all);
 	pthread_mutex_destroy(&pt->lock);
 }
