@@ -1,5 +1,5 @@
 /*
- * Copyright 2008-2019 Aerospike, Inc.
+ * Copyright 2008-2023 Aerospike, Inc.
  *
  * Portions may be licensed to Aerospike, Inc. under one or more contributor
  * license agreements.
@@ -35,7 +35,7 @@ static bool cert_blacklist_check(void* cert_blacklist,
 								 const char* issuer_name);
 static void cert_blacklist_destroy(void* cert_blacklist);
 
-static void manage_sigpipe();
+static void manage_sigpipe(void);
 
 static bool s_tls_inited = false;
 static pthread_mutex_t s_tls_init_mutex = PTHREAD_MUTEX_INITIALIZER;
@@ -117,10 +117,10 @@ protocols_parse(as_config_tls* tlscfg, uint16_t* oprotocols, as_error* errp)
 
 		if (act == '-') {
 			protocols &= ~current;
-        }
-        else if (act == '+') {
-            protocols |= current;
-        }
+		}
+		else if (act == '+') {
+			protocols |= current;
+		}
 		else {
 			if (protocols != AS_TLS_PROTOCOL_NONE) {
 				return as_error_update(errp, AEROSPIKE_ERR_TLS_ERROR,
@@ -187,11 +187,12 @@ static pthread_mutex_t *lock_cs;
 static void
 pthreads_locking_callback(int mode, int type, const char *file, int line)
 {
-    if (mode & CRYPTO_LOCK) {
-        pthread_mutex_lock(&(lock_cs[type]));
-    } else {
-        pthread_mutex_unlock(&(lock_cs[type]));
-    }
+	if (mode & CRYPTO_LOCK) {
+		pthread_mutex_lock(&(lock_cs[type]));
+	}
+	else {
+		pthread_mutex_unlock(&(lock_cs[type]));
+	}
 }
 
 static void
@@ -207,27 +208,27 @@ pthreads_thread_id(CRYPTO_THREADID *tid)
 static void
 threading_setup(void)
 {
-    int i;
+	int i;
 
-    lock_cs = cf_malloc(CRYPTO_num_locks() * sizeof(pthread_mutex_t));
-    for (i = 0; i < CRYPTO_num_locks(); i++) {
-        pthread_mutex_init(&(lock_cs[i]), NULL);
-    }
+	lock_cs = cf_malloc(CRYPTO_num_locks() * sizeof(pthread_mutex_t));
+	for (i = 0; i < CRYPTO_num_locks(); i++) {
+		pthread_mutex_init(&(lock_cs[i]), NULL);
+	}
 
-    CRYPTO_THREADID_set_callback(pthreads_thread_id);
-    CRYPTO_set_locking_callback(pthreads_locking_callback);
+	CRYPTO_THREADID_set_callback(pthreads_thread_id);
+	CRYPTO_set_locking_callback(pthreads_locking_callback);
 }
 
 static void
 threading_cleanup(void)
 {
-    int i;
+	int i;
 
-    CRYPTO_set_locking_callback(NULL);
-    for (i = 0; i < CRYPTO_num_locks(); i++) {
-        pthread_mutex_destroy(&(lock_cs[i]));
-    }
-    cf_free(lock_cs);
+	CRYPTO_set_locking_callback(NULL);
+	for (i = 0; i < CRYPTO_num_locks(); i++) {
+		pthread_mutex_destroy(&(lock_cs[i]));
+	}
+	cf_free(lock_cs);
 }
 #endif
 
@@ -259,9 +260,7 @@ as_tls_check_init()
 
 		s_ex_name_index = SSL_get_ex_new_index(0, NULL, NULL, NULL, NULL);
 		s_ex_ctxt_index = SSL_get_ex_new_index(0, NULL, NULL, NULL, NULL);
-		
-		as_fence_memory();
-		
+				
 		s_tls_inited = true;
 	}
 
@@ -301,7 +300,7 @@ as_tls_cleanup(void)
 	// http://stackoverflow.com/questions/29845527/how-to-properly-uninitialize-openssl
 	STACK_OF(SSL_COMP) *ssl_comp_methods = SSL_COMP_get_compression_methods();
 	if (ssl_comp_methods != NULL) {
-        sk_SSL_COMP_free(ssl_comp_methods);
+		sk_SSL_COMP_free(ssl_comp_methods);
 	}
 #endif
 }
@@ -325,12 +324,12 @@ static int verify_callback(int preverify_ok, X509_STORE_CTX* ctx)
 		return preverify_ok;
 	}
 
-    SSL* ssl = X509_STORE_CTX_get_ex_data(
+	SSL* ssl = X509_STORE_CTX_get_ex_data(
 					ctx, SSL_get_ex_data_X509_STORE_CTX_idx());
 
 	// The verify callback is called for each cert in the chain.
 	
-    X509* current_cert = X509_STORE_CTX_get_current_cert(ctx);
+	X509* current_cert = X509_STORE_CTX_get_current_cert(ctx);
 
 	as_tls_context* asctxt = SSL_get_ex_data(ssl, s_ex_ctxt_index);
 	if (! asctxt) {
@@ -419,6 +418,137 @@ password_cb(char* buf, int size, int rwflag, void* udata)
 	return pw_len;
 }
 
+static bool
+as_tls_load_ca_str(SSL_CTX* ctx, char* cert_str)
+{
+	BIO* cert_bio = BIO_new_mem_buf(cert_str, -1);
+
+	if (cert_bio == NULL) {
+		return false;
+	}
+
+	X509* cert;
+	int count = 0;
+
+	while ((cert = PEM_read_bio_X509(cert_bio, NULL, 0, NULL)) != NULL) {
+		X509_STORE* store = SSL_CTX_get_cert_store(ctx);
+		int rv = X509_STORE_add_cert(store, cert);
+
+		if (rv == 1) {
+			count++;
+		}
+		else {
+			as_log_warn("Failed to add TLS certificate from string");
+		}
+
+		X509_free(cert);
+	}
+
+	// Above loop always ends with an error - clear it so it doesn't affect
+	// subsequent SSL calls in this thread.
+	ERR_clear_error();
+	BIO_vfree(cert_bio);
+
+	if (count == 0) {
+		return false;
+	}
+	return true;
+}
+
+static bool
+as_tls_load_cert_chain_str(SSL_CTX* ctx, char* cert_str)
+{
+	BIO* cert_bio = BIO_new_mem_buf(cert_str, -1);
+
+	if (cert_bio == NULL) {
+		return false;
+	}
+
+	STACK_OF(X509_INFO)* inf = PEM_X509_INFO_read_bio(cert_bio, NULL, NULL, NULL);
+
+	if (!inf) {
+		BIO_free(cert_bio);
+		return false;
+	}
+
+	/* Iterate over contents of the PEM buffer, and add certs. */
+	bool first = true;
+
+	for (int i = 0; i < sk_X509_INFO_num(inf); i++) {
+		X509_INFO* itmp = sk_X509_INFO_value(inf, i);
+
+		if (itmp->x509) {
+			/* First cert is server cert. Remaining, if any, are intermediate certs. */
+			if (first) {
+				first = false;
+
+				/*
+				 * Set server certificate. Note that this operation increments the
+				 * reference count, which means that it is okay for cleanup to free it.
+				 */
+				if (!SSL_CTX_use_certificate(ctx, itmp->x509)) {
+					goto Error;
+				}
+
+				if (ERR_peek_error() != 0) {
+					goto Error;
+				}
+
+				/* Get ready to store intermediate certs, if any. */
+				SSL_CTX_clear_chain_certs(ctx);
+			}
+			else
+			{
+				/* Add intermediate cert to chain. */
+				if (!SSL_CTX_add0_chain_cert(ctx, itmp->x509)) {
+					goto Error;
+				}
+
+				/*
+				 * Above function doesn't increment cert reference count. NULL the info
+				 * reference to it in order to prevent it from being freed during cleanup.
+				 */
+				itmp->x509 = NULL;
+			}
+		}
+	}
+
+	sk_X509_INFO_pop_free(inf, X509_INFO_free);
+	BIO_free(cert_bio);
+	return true;
+
+Error:
+	sk_X509_INFO_pop_free(inf, X509_INFO_free);
+	BIO_free(cert_bio);
+	return false;
+}
+
+static bool
+as_tls_load_key_str(SSL_CTX* ctx, char* key_str, const char* key_pw)
+{
+	BIO* key_bio = BIO_new_mem_buf(key_str, -1);
+
+	if (key_bio == NULL) {
+		return false;
+	}
+
+	EVP_PKEY* pkey = PEM_read_bio_PrivateKey(key_bio, NULL, password_cb, (void*)key_pw);
+
+	BIO_vfree(key_bio);
+
+	if (pkey == NULL) {
+		if (ERR_GET_REASON(ERR_peek_error()) == EVP_R_BAD_DECRYPT) {
+			as_log_warn("Invalid password for key string");
+		}
+		return false;
+	}
+
+	int rv = SSL_CTX_use_PrivateKey(ctx, pkey);
+
+	EVP_PKEY_free(pkey);
+	return rv == 1;
+}
+
 as_status
 as_tls_context_setup(as_config_tls* tlscfg, as_tls_context* ctx, as_error* errp)
 {
@@ -495,19 +625,19 @@ as_tls_context_setup(as_config_tls* tlscfg, as_tls_context* ctx, as_error* errp)
 	}
 
 	/* always disable SSLv2, as per RFC 6176 */
-    SSL_CTX_set_options(ctx->ssl_ctx, SSL_OP_NO_SSLv2);
+	SSL_CTX_set_options(ctx->ssl_ctx, SSL_OP_NO_SSLv2);
 	SSL_CTX_set_options(ctx->ssl_ctx, SSL_OP_NO_SSLv3);
 
 	// Turn off non-enabled protocols.
 	if (! (protocols & AS_TLS_PROTOCOL_TLSV1)) {
-        SSL_CTX_set_options(ctx->ssl_ctx, SSL_OP_NO_TLSv1);
-    }
+		SSL_CTX_set_options(ctx->ssl_ctx, SSL_OP_NO_TLSv1);
+	}
 	if (! (protocols & AS_TLS_PROTOCOL_TLSV1_1)) {
-        SSL_CTX_set_options(ctx->ssl_ctx, SSL_OP_NO_TLSv1_1);
-    }
+		SSL_CTX_set_options(ctx->ssl_ctx, SSL_OP_NO_TLSv1_1);
+	}
 	if (! (protocols & AS_TLS_PROTOCOL_TLSV1_2)) {
-        SSL_CTX_set_options(ctx->ssl_ctx, SSL_OP_NO_TLSv1_2);
-    }
+		SSL_CTX_set_options(ctx->ssl_ctx, SSL_OP_NO_TLSv1_2);
+	}
 	
 	if (tlscfg->cafile || tlscfg->capath) {
 		int rv = SSL_CTX_load_verify_locations(ctx->ssl_ctx, tlscfg->cafile, tlscfg->capath);
@@ -525,6 +655,12 @@ as_tls_context_setup(as_config_tls* tlscfg, as_tls_context* ctx, as_error* errp)
 										"Unknown failure loading CAFile");
 		}
 	}
+	else if (tlscfg->castring) {
+		if (! as_tls_load_ca_str(ctx->ssl_ctx, tlscfg->castring)) {
+			return as_error_set_message(errp, AEROSPIKE_ERR_TLS_ERROR,
+				"Failed to add any TLS certificates from castring");
+		}
+	}
 
 	if (tlscfg->certfile) {
 		int rv = SSL_CTX_use_certificate_chain_file(ctx->ssl_ctx, tlscfg->certfile);
@@ -549,6 +685,12 @@ as_tls_context_setup(as_config_tls* tlscfg, as_tls_context* ctx, as_error* errp)
 			}
 		}
 	}
+	else if (tlscfg->certstring) {
+		if (! as_tls_load_cert_chain_str(ctx->ssl_ctx, tlscfg->certstring)) {
+			return as_error_set_message(errp, AEROSPIKE_ERR_TLS_ERROR,
+				"Failed to add any TLS certificate chains from certstrings");
+		}
+	}
 
 	if (tlscfg->keyfile) {
 		bool ok = false;
@@ -603,6 +745,12 @@ as_tls_context_setup(as_config_tls* tlscfg, as_tls_context* ctx, as_error* errp)
 			return AEROSPIKE_ERR_TLS_ERROR;
 		}
 	}
+	else if (tlscfg->keystring) {
+		if (! as_tls_load_key_str(ctx->ssl_ctx, tlscfg->keystring, tlscfg->keyfile_pw)) {
+			return as_error_set_message(errp, AEROSPIKE_ERR_TLS_ERROR,
+				"Failed to load private key from keystring");
+		}
+	}
 
 	if (tlscfg->cipher_suite) {
 		int rv = SSL_CTX_set_cipher_list(ctx->ssl_ctx, tlscfg->cipher_suite);
