@@ -1,5 +1,5 @@
 /*
- * Copyright 2008-2022 Aerospike, Inc.
+ * Copyright 2008-2024 Aerospike, Inc.
  *
  * Portions may be licensed to Aerospike, Inc. under one or more contributor
  * license agreements.
@@ -46,6 +46,7 @@
 #define N_KEYS 200
 
 extern aerospike* as;
+extern bool g_has_ttl;
 
 uint32_t num_threads = 0;
 pthread_rwlock_t rwlock;
@@ -563,7 +564,7 @@ batch_write_operate_cb(const as_batch_result* results, uint32_t n, void* udata)
 	return true;
 }
 
-TEST(batch_write_list_operate, "Batch write list operate")
+TEST(batch_write_list_operate, "Batch write list operate with default ttl")
 {
 	as_batch batch;
 	as_batch_inita(&batch, N_KEYS);
@@ -581,10 +582,15 @@ TEST(batch_write_list_operate, "Batch write list operate")
 	as_operations_list_insert(&ops, LIST_BIN, NULL, NULL, 0, (as_val*)&val);
 	as_operations_list_size(&ops, LIST_BIN, NULL);
 	as_operations_list_get_by_index(&ops, LIST_BIN, NULL, -1, AS_LIST_RETURN_VALUE);
+	ops.ttl = AS_RECORD_CLIENT_DEFAULT_TTL;
+
+	as_policy_batch_write p;
+	as_policy_batch_write_init(&p);
+	p.ttl = 5000;
 
 	batch_stats data = {0};
 	as_error err;
-	as_status status = aerospike_batch_operate(as, &err, NULL, NULL, &batch, &ops,
+	as_status status = aerospike_batch_operate(as, &err, NULL, &p, &batch, &ops,
 		batch_write_operate_cb, &data);
 
 	as_operations_destroy(&ops);
@@ -600,14 +606,17 @@ TEST(batch_write_complex, "Batch write complex")
 	as_operations wops1;
 	as_operations_inita(&wops1, 1);
 	as_operations_add_write_int64(&wops1, bin2, 100);
+	wops1.ttl = AS_RECORD_CLIENT_DEFAULT_TTL;
 
 	as_operations wops2;
 	as_operations_inita(&wops2, 1);
 	as_operations_exp_write(&wops2, bin3, wexp1, AS_EXP_WRITE_DEFAULT);
+	wops2.ttl = AS_RECORD_CLIENT_DEFAULT_TTL;
 
 	as_policy_batch_write wp;
 	as_policy_batch_write_init(&wp);
 	wp.key = AS_POLICY_KEY_SEND;
+	wp.ttl = 500;
 
 	as_batch_records recs;
 	as_batch_records_inita(&recs, 3);
@@ -624,9 +633,15 @@ TEST(batch_write_complex, "Batch write complex")
 	as_batch_remove_record* rm = as_batch_remove_reserve(&recs);
 	as_key_init_int64(&rm->key, NAMESPACE, SET, 10002);
 
+	// Test default ttl.
+	as->config.policies.batch_write.ttl = 1000;
+
 	as_error err;
 	as_status status = aerospike_batch_write(as, &err, NULL, &recs);
 
+	// Reset default ttl.
+	as->config.policies.batch_write.ttl = 0;
+
 	assert_int_eq(status, AEROSPIKE_OK);
 
 	assert_int_eq(wr1->result, AEROSPIKE_OK);
@@ -788,6 +803,55 @@ TEST(batch_remove, "Batch remove")
 	assert_int_eq(errors, 0);
 }
 
+TEST(batch_reset_read_ttl, "Batch reset read ttl")
+{
+	as_error err;
+	as_status status;
+	uint32_t errors;
+
+	// Define keys
+	as_batch batch;
+	as_batch_inita(&batch, 2);
+	as_key_init_int64(as_batch_keyat(&batch, 0), NAMESPACE, SET, 8888);
+	as_key_init_int64(as_batch_keyat(&batch, 1), NAMESPACE, SET, 8889);
+
+	as_operations ops;
+	as_operations_inita(&ops, 1);
+	as_operations_add_write_int64(&ops, "a", 1);
+	ops.ttl = 2;
+
+	errors = 0;
+	status = aerospike_batch_operate(as, &err, NULL, NULL, &batch, &ops, result_cb, &errors);
+	assert_int_eq(status, AEROSPIKE_OK);
+	assert_int_eq(errors, 0);
+
+	// Read the records before they expire and reset read ttl.
+	as_sleep(1010);
+	as_policy_batch pb;
+	as_policy_batch_init(&pb);
+	pb.read_touch_ttl_percent = 80;
+	
+	errors = 0;
+	status = aerospike_batch_exists(as, &err, &pb, &batch, result_cb, &errors);
+	assert_int_eq(status, AEROSPIKE_OK);
+	assert_int_eq(errors, 0);
+
+	// Read the records again, but don't reset read ttl.
+	as_sleep(1010);
+	pb.read_touch_ttl_percent = -1;
+	errors = 0;
+	status = aerospike_batch_exists(as, &err, &pb, &batch, result_cb, &errors);
+	assert_int_eq(status, AEROSPIKE_OK);
+	assert_int_eq(errors, 0);
+
+	// Read the record after it expires, showing it's gone.
+	as_sleep(2000);
+	errors = 0;
+	status = aerospike_batch_exists(as, &err, NULL, &batch, not_exists_cb, &errors);
+	assert_int_eq(status, AEROSPIKE_OK);
+	assert_int_eq(errors, 0);
+}
+
 //---------------------------------
 // Test Suite
 //---------------------------------
@@ -805,4 +869,8 @@ SUITE(batch, "aerospike batch tests")
 	suite_add(batch_write_complex);
 	suite_add(batch_write_read_all_bins);
 	suite_add(batch_remove);
+	
+	if (g_has_ttl) {
+		suite_add(batch_reset_read_ttl);
+	}
 }
