@@ -1,5 +1,5 @@
 /*
- * Copyright 2008-2018 Aerospike, Inc.
+ * Copyright 2008-2022 Aerospike, Inc.
  *
  * Portions may be licensed to Aerospike, Inc. under one or more contributor
  * license agreements.
@@ -26,5 +26,5 @@
  * FUNCTIONS
  *****************************************************************************/
 
-as_aerospike * test_aerospike_new();
+as_aerospike * test_aerospike_new(void);
 as_aerospike * test_aerospike_init(as_aerospike *);
