@@ -13,14 +13,16 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 ##########################################################################
-'''
-Module with helper functions to create dictionaries consumed by
-the :mod:`aerospike.Client.operate` and :mod:`aerospike.Client.operate_ordered` methods for the aerospike.client class.
-'''
+"""
+Module with helper functions to create dictionaries used by:
+
+* :mod:`aerospike.Client.operate` and :mod:`aerospike.Client.operate_ordered`
+* Certain batch operations listed in :mod:`aerospike_helpers.batch.records`
+"""
 import warnings
 
 import aerospike
-
+from typing import Optional
 
 
 def read(bin_name):
@@ -29,7 +31,7 @@ def read(bin_name):
     The read operation reads and returns the value in `bin_name`.
 
     Args:
-        bin (str): the name of the bin from which to read.
+        bin_name (str): the name of the bin from which to read.
     Returns:
         A dictionary to be passed to operate or operate_ordered.
     """
@@ -46,16 +48,12 @@ def write(bin_name, write_item):
     The write operation writes `write_item` into the bin specified by bin_name.
 
     Args:
-        bin (str): The name of the bin into which `write_item` will be stored.
+        bin_name (str): The name of the bin into which `write_item` will be stored.
         write_item: The value which will be written into the bin.
     Returns:
         A dictionary to be passed to operate or operate_ordered.
     """
-    return {
-        "op": aerospike.OPERATOR_WRITE,
-        "bin": bin_name,
-        "val": write_item
-    }
+    return {"op": aerospike.OPERATOR_WRITE, "bin": bin_name, "val": write_item}
 
 
 def delete():
@@ -79,16 +77,12 @@ def append(bin_name, append_item):
     The append operation appends `append_item` to the value in bin_name.
 
     Args:
-        bin (str): The name of the bin to be used.
+        bin_name (str): The name of the bin to be used.
         append_item: The value which will be appended to the item contained in the specified bin.
     Returns:
         A dictionary to be passed to operate or operate_ordered.
     """
-    return {
-        "op": aerospike.OPERATOR_APPEND,
-        "bin": bin_name,
-        "val": append_item
-    }
+    return {"op": aerospike.OPERATOR_APPEND, "bin": bin_name, "val": append_item}
 
 
 def prepend(bin_name, prepend_item):
@@ -97,16 +91,12 @@ def prepend(bin_name, prepend_item):
     The prepend operation prepends `prepend_item` to the value in bin_name.
 
     Args:
-        bin (str): The name of the bin to be used.
+        bin_name (str): The name of the bin to be used.
         prepend_item: The value which will be prepended to the item contained in the specified bin.
     Returns:
         A dictionary to be passed to operate or operate_ordered.
     """
-    return {
-        "op": aerospike.OPERATOR_PREPEND,
-        "bin": bin_name,
-        "val": prepend_item
-    }
+    return {"op": aerospike.OPERATOR_PREPEND, "bin": bin_name, "val": prepend_item}
 
 
 def increment(bin_name, amount):
@@ -116,19 +106,15 @@ def increment(bin_name, amount):
     or creates a bin with the value of amount.
 
     Args:
-        bin (str): The name of the bin to be incremented.
+        bin_name (str): The name of the bin to be incremented.
         amount: The amount by which to increment the item in the specified bin.
     Returns:
         A dictionary to be passed to operate or operate_ordered.
     """
-    return {
-        "op": aerospike.OPERATOR_INCR,
-        "bin": bin_name,
-        "val": amount
-    }
+    return {"op": aerospike.OPERATOR_INCR, "bin": bin_name, "val": amount}
 
 
-def touch(ttl: int=None):
+def touch(ttl: Optional[int] = None):
     """Create a touch operation dictionary.
 
     Using ttl here is deprecated. It should be set in the record metadata for the operate method.
@@ -142,7 +128,6 @@ def touch(ttl: int=None):
     """
     op_dict = {"op": aerospike.OPERATOR_TOUCH}
     if ttl:
-        warnings.warn(
-            "TTL should be specified in the meta dictionary for operate", DeprecationWarning)
+        warnings.warn("TTL should be specified in the meta dictionary for operate", DeprecationWarning)
         op_dict["val"] = ttl
     return op_dict
