@@ -40,31 +40,36 @@
 #define AS_PY_MAP_POLICY "map_policy"
 #define AS_EXPR_KEY "expr"
 #define AS_EXPR_FLAGS_KEY "expr_flags"
+#define AS_PY_PAD "pad"
+#define AS_PY_PERSIST_INDEX "persist_index"
+
+as_status get_bool_from_pyargs(as_error *err, char *key, PyObject *op_dict,
+                               bool *boolean);
 
 as_status get_bin(as_error *err, PyObject *op_dict, as_vector *unicodeStrVector,
-				  char **binName);
+                  char **binName);
 
 as_status get_asval(AerospikeClient *self, as_error *err, char *key,
-					PyObject *op_dict, as_val **val,
-					as_static_pool *static_pool, int serializer_type,
-					bool required);
+                    PyObject *op_dict, as_val **val,
+                    as_static_pool *static_pool, int serializer_type,
+                    bool required);
 
 as_status get_val_list(AerospikeClient *self, as_error *err,
-					   const char *list_key, PyObject *op_dict, as_list **list,
-					   as_static_pool *static_pool, int serializer_type);
+                       const char *list_key, PyObject *op_dict, as_list **list,
+                       as_static_pool *static_pool, int serializer_type);
 
 as_status get_int64_t(as_error *err, const char *key, PyObject *op_dict,
-					  int64_t *i64_valptr);
+                      int64_t *i64_valptr);
 
 as_status get_optional_int64_t(as_error *err, const char *key,
-							   PyObject *op_dict, int64_t *i64_valptr,
-							   bool *found);
+                               PyObject *op_dict, int64_t *i64_valptr,
+                               bool *found);
 
 as_status get_int_from_py_dict(as_error *err, const char *key,
-							   PyObject *op_dict, int *int_pointer);
+                               PyObject *op_dict, int *int_pointer);
 
 as_status get_list_return_type(as_error *err, PyObject *op_dict,
-							   int *return_type);
+                               int *return_type);
 
 as_status get_list_policy(as_error *err, PyObject *op_dict,
-						  as_list_policy *policy, bool *found);
\ No newline at end of file
+                          as_list_policy *policy, bool *found);
