@@ -41,7 +41,7 @@ PyTypeObject *AerospikeClient_Ready(void);
  * Create a new Aerospike client object and connect to the database.
  */
 AerospikeClient *AerospikeClient_New(PyObject *self, PyObject *args,
-									 PyObject *kwds);
+                                     PyObject *kwds);
 
 /*******************************************************************************
  * CONNECTION OPERATIONS
@@ -50,26 +50,28 @@ AerospikeClient *AerospikeClient_New(PyObject *self, PyObject *args,
 /**
  * Connect to the database.
  */
+int AerospikeClientConnect(AerospikeClient *self);
+
 PyObject *AerospikeClient_Connect(AerospikeClient *self, PyObject *args,
-								  PyObject *kwds);
+                                  PyObject *kwds);
 
 /**
  * Close the connections to the database.
  */
 PyObject *AerospikeClient_Close(AerospikeClient *self, PyObject *args,
-								PyObject *kwds);
+                                PyObject *kwds);
 
 /**
  * Checks the connection to the database.
  */
 PyObject *AerospikeClient_is_connected(AerospikeClient *self, PyObject *args,
-									   PyObject *kwds);
+                                       PyObject *kwds);
 
 /**
  * Get the shm_key to the cluster.
  */
 PyObject *AerospikeClient_shm_key(AerospikeClient *self, PyObject *args,
-								  PyObject *kwds);
+                                  PyObject *kwds);
 
 /*******************************************************************************
  * KVS OPERATIONS
@@ -82,13 +84,13 @@ PyObject *AerospikeClient_shm_key(AerospikeClient *self, PyObject *args,
  *
  */
 PyObject *AerospikeClient_Apply(AerospikeClient *self, PyObject *args,
-								PyObject *kwds);
+                                PyObject *kwds);
 
 PyObject *AerospikeClient_Apply_Invoke(AerospikeClient *self, PyObject *py_key,
-									   PyObject *py_module,
-									   PyObject *py_function,
-									   PyObject *py_arglist,
-									   PyObject *py_policy);
+                                       PyObject *py_module,
+                                       PyObject *py_function,
+                                       PyObject *py_arglist,
+                                       PyObject *py_policy);
 /**
  * Check existence of a record in the database.
  *
@@ -96,10 +98,10 @@ PyObject *AerospikeClient_Apply_Invoke(AerospikeClient *self, PyObject *py_key,
  *
  */
 PyObject *AerospikeClient_Exists(AerospikeClient *self, PyObject *args,
-								 PyObject *kwds);
+                                 PyObject *kwds);
 
 PyObject *AerospikeClient_Exists_Invoke(AerospikeClient *self, PyObject *py_key,
-										PyObject *py_policy);
+                                        PyObject *py_policy);
 
 /**
  * Read a record from the database.
@@ -108,19 +110,10 @@ PyObject *AerospikeClient_Exists_Invoke(AerospikeClient *self, PyObject *py_key,
  *
  */
 PyObject *AerospikeClient_Get(AerospikeClient *self, PyObject *args,
-							  PyObject *kwds);
+                              PyObject *kwds);
 
 PyObject *AerospikeClient_Get_Invoke(AerospikeClient *self, PyObject *py_key,
-									 PyObject *py_policy);
-
-/**
- * Async Read a record from the database.
- *
- *		client.get((x,y,z))
- *
- */
-PyObject *AerospikeClient_Get_Async(AerospikeClient *self, PyObject *args,
-									PyObject *kwds);
+                                     PyObject *py_policy);
 
 /**
  * Project specific bins of a record from the database.
@@ -129,7 +122,7 @@ PyObject *AerospikeClient_Get_Async(AerospikeClient *self, PyObject *args,
  *
  */
 PyObject *AerospikeClient_Select(AerospikeClient *self, PyObject *args,
-								 PyObject *kwds);
+                                 PyObject *kwds);
 
 /**
  * Write a record in the database.
@@ -138,21 +131,12 @@ PyObject *AerospikeClient_Select(AerospikeClient *self, PyObject *args,
  *
  */
 PyObject *AerospikeClient_Put(AerospikeClient *self, PyObject *args,
-							  PyObject *kwds);
+                              PyObject *kwds);
 
 PyObject *AerospikeClient_Put_Invoke(AerospikeClient *self, PyObject *py_key,
-									 PyObject *py_bins, PyObject *py_meta,
-									 PyObject *py_policy,
-									 long serializer_option);
-
-/**
- * Write a record async in the database.
- *
- *		client.put((x,y,z), ...)
- *
- */
-PyObject *AerospikeClient_Put_Async(AerospikeClient *self, PyObject *args,
-									PyObject *kwds);
+                                     PyObject *py_bins, PyObject *py_meta,
+                                     PyObject *py_policy,
+                                     long serializer_option);
 
 /**
  * Remove a record from the database.
@@ -161,10 +145,10 @@ PyObject *AerospikeClient_Put_Async(AerospikeClient *self, PyObject *args,
  *
  */
 PyObject *AerospikeClient_Remove(AerospikeClient *self, PyObject *args,
-								 PyObject *kwds);
+                                 PyObject *kwds);
 
 PyObject *AerospikeClient_Remove_Invoke(AerospikeClient *self, PyObject *py_key,
-										PyObject *py_meta, PyObject *py_policy);
+                                        PyObject *py_meta, PyObject *py_policy);
 
 /**
  * Remove bin from the database.
@@ -173,7 +157,7 @@ PyObject *AerospikeClient_Remove_Invoke(AerospikeClient *self, PyObject *py_key,
  *
  */
 extern PyObject *AerospikeClient_RemoveBin(AerospikeClient *self,
-										   PyObject *args, PyObject *kwds);
+                                           PyObject *args, PyObject *kwds);
 /**
  * Append a string to a string bin value of a record in the database.
  *
@@ -181,7 +165,7 @@ extern PyObject *AerospikeClient_RemoveBin(AerospikeClient *self,
  *
  */
 PyObject *AerospikeClient_Append(AerospikeClient *self, PyObject *args,
-								 PyObject *kwds);
+                                 PyObject *kwds);
 /**
  * Prepend a string to a string bin value of a record in the database.
  *
@@ -189,7 +173,7 @@ PyObject *AerospikeClient_Append(AerospikeClient *self, PyObject *args,
  *
  */
 PyObject *AerospikeClient_Prepend(AerospikeClient *self, PyObject *args,
-								  PyObject *kwds);
+                                  PyObject *kwds);
 /**
  * Increment bin value of a record to the database.
  *
@@ -197,7 +181,7 @@ PyObject *AerospikeClient_Prepend(AerospikeClient *self, PyObject *args,
  *
  */
 PyObject *AerospikeClient_Increment(AerospikeClient *self, PyObject *args,
-									PyObject *kwds);
+                                    PyObject *kwds);
 /**
  * Touch a record in the database.
  *
@@ -205,7 +189,7 @@ PyObject *AerospikeClient_Increment(AerospikeClient *self, PyObject *args,
  *
  */
 PyObject *AerospikeClient_Touch(AerospikeClient *self, PyObject *args,
-								PyObject *kwds);
+                                PyObject *kwds);
 /**
  * Performs operate operations
  *
@@ -213,7 +197,7 @@ PyObject *AerospikeClient_Touch(AerospikeClient *self, PyObject *args,
  *
  */
 PyObject *AerospikeClient_Operate(AerospikeClient *self, PyObject *args,
-								  PyObject *kwds);
+                                  PyObject *kwds);
 /**
  * Performs operate ordered operations
  *
@@ -221,210 +205,7 @@ PyObject *AerospikeClient_Operate(AerospikeClient *self, PyObject *args,
  *
  */
 PyObject *AerospikeClient_OperateOrdered(AerospikeClient *self, PyObject *args,
-										 PyObject *kwds);
-
-/*******************************************************************************
- * LIST FUNCTIONS(CDT)
- ******************************************************************************/
-/**
- * Append a single val to the list value in bin.
- *
- *		client.list_append((x,y,z))
- *
- */
-PyObject *AerospikeClient_ListAppend(AerospikeClient *self, PyObject *args,
-									 PyObject *kwds);
-
-/**
- * Extend the list value in bin with the given items.
- *
- *		client.list_extend((x,y,z))
- *
- */
-PyObject *AerospikeClient_ListExtend(AerospikeClient *self, PyObject *args,
-									 PyObject *kwds);
-
-/**
- * Insert val at the specified index of the list value in bin.
- *
- *		client.list_insert((x,y,z))
- *
- */
-PyObject *AerospikeClient_ListInsert(AerospikeClient *self, PyObject *args,
-									 PyObject *kwds);
-
-/**
- * Insert the items at the specified index of a list value in bin.
- *
- *		client.list_insert_items((x,y,z))
- *
- */
-PyObject *AerospikeClient_ListInsertItems(AerospikeClient *self, PyObject *args,
-										  PyObject *kwds);
-
-/**
- * Remove and get back a list element at a given index of a list value in bin.
- *
- *		client.list_pop((x,y,z))
- *
- */
-PyObject *AerospikeClient_ListPop(AerospikeClient *self, PyObject *args,
-								  PyObject *kwds);
-
-/**
- * Remove and get back a list element at a given index of a list value in bin.
- *
- *		client.list_pop_range((x,y,z))
- *
- */
-PyObject *AerospikeClient_ListPopRange(AerospikeClient *self, PyObject *args,
-									   PyObject *kwds);
-
-/**
- * Remove a list element at a given index of a list value in bin.
- *
- *		client.list_remove((x,y,z))
- *
- */
-PyObject *AerospikeClient_ListRemove(AerospikeClient *self, PyObject *args,
-									 PyObject *kwds);
-
-/**
- * Remove list elements at a given index of a list value in bin.
- *
- *		client.list_remove_range((x,y,z))
- *
- */
-PyObject *AerospikeClient_ListRemoveRange(AerospikeClient *self, PyObject *args,
-										  PyObject *kwds);
-
-/**
- * Remove all the elements from a list value in bin.
- *
- *		client.list_clear((x,y,z))
- *
- */
-PyObject *AerospikeClient_ListClear(AerospikeClient *self, PyObject *args,
-									PyObject *kwds);
-
-/**
- * Set list element val at the specified index of a list value in bin.
- *
- *		client.list_set((x,y,z))
- *
- */
-PyObject *AerospikeClient_ListSet(AerospikeClient *self, PyObject *args,
-								  PyObject *kwds);
-
-/**
- * Get the list element at the specified index of a list value in bin.
- *
- *		client.list_get((x,y,z))
- *
- */
-PyObject *AerospikeClient_ListGet(AerospikeClient *self, PyObject *args,
-								  PyObject *kwds);
-
-/**
- * Get the list of count elements starting at a specified index of a list value in bin.
- *
- *		client.list_get_range((x,y,z))
- *
- */
-PyObject *AerospikeClient_ListGetRange(AerospikeClient *self, PyObject *args,
-									   PyObject *kwds);
-
-/**
- * Remove elements from the list which are not within the range starting at the
- * given index plus count.
- *
- *		client.list_trim((x,y,z))
- *
- */
-PyObject *AerospikeClient_ListTrim(AerospikeClient *self, PyObject *args,
-								   PyObject *kwds);
-
-/**
- * Count the elements of the list value in bin.
- *
- *		client.list_size((x,y,z))
- *
- */
-PyObject *AerospikeClient_ListSize(AerospikeClient *self, PyObject *args,
-								   PyObject *kwds);
-
-/*******************************************************************************
- * MAP FUNCTIONS
- ******************************************************************************/
-/**
- * Append a single val to the map value in bin.
- *
- *		client.map_put((x,y,z))
- *
- */
-
-PyObject *AerospikeClient_MapSetPolicy(AerospikeClient *self, PyObject *args,
-									   PyObject *kwds);
-PyObject *AerospikeClient_MapPut(AerospikeClient *self, PyObject *args,
-								 PyObject *kwds);
-PyObject *AerospikeClient_MapPutItems(AerospikeClient *self, PyObject *args,
-									  PyObject *kwds);
-PyObject *AerospikeClient_MapIncrement(AerospikeClient *self, PyObject *args,
-									   PyObject *kwds);
-PyObject *AerospikeClient_MapDecrement(AerospikeClient *self, PyObject *args,
-									   PyObject *kwds);
-PyObject *AerospikeClient_MapSize(AerospikeClient *self, PyObject *args,
-								  PyObject *kwds);
-PyObject *AerospikeClient_MapClear(AerospikeClient *self, PyObject *args,
-								   PyObject *kwds);
-
-PyObject *AerospikeClient_MapRemoveByKey(AerospikeClient *self, PyObject *args,
-										 PyObject *kwds);
-PyObject *AerospikeClient_MapRemoveByKeyList(AerospikeClient *self,
-											 PyObject *args, PyObject *kwds);
-PyObject *AerospikeClient_MapRemoveByKeyRange(AerospikeClient *self,
-											  PyObject *args, PyObject *kwds);
-
-PyObject *AerospikeClient_MapRemoveByValue(AerospikeClient *self,
-										   PyObject *args, PyObject *kwds);
-PyObject *AerospikeClient_MapRemoveByValueList(AerospikeClient *self,
-											   PyObject *args, PyObject *kwds);
-PyObject *AerospikeClient_MapRemoveByValueRange(AerospikeClient *self,
-												PyObject *args, PyObject *kwds);
-
-PyObject *AerospikeClient_MapRemoveByIndex(AerospikeClient *self,
-										   PyObject *args, PyObject *kwds);
-PyObject *AerospikeClient_MapRemoveByIndexRange(AerospikeClient *self,
-												PyObject *args, PyObject *kwds);
-
-PyObject *AerospikeClient_MapRemoveByRank(AerospikeClient *self, PyObject *args,
-										  PyObject *kwds);
-PyObject *AerospikeClient_MapRemoveByRankRange(AerospikeClient *self,
-											   PyObject *args, PyObject *kwds);
-
-PyObject *AerospikeClient_MapGetByKey(AerospikeClient *self, PyObject *args,
-									  PyObject *kwds);
-PyObject *AerospikeClient_MapGetByKeyRange(AerospikeClient *self,
-										   PyObject *args, PyObject *kwds);
-
-PyObject *AerospikeClient_MapGetByValue(AerospikeClient *self, PyObject *args,
-										PyObject *kwds);
-PyObject *AerospikeClient_MapGetByValueRange(AerospikeClient *self,
-											 PyObject *args, PyObject *kwds);
-PyObject *AerospikeClient_MapGetByValueList(AerospikeClient *self,
-											PyObject *args, PyObject *kwds);
-PyObject *AerospikeClient_MapGetByKeyList(AerospikeClient *self, PyObject *args,
-										  PyObject *kwds);
-
-PyObject *AerospikeClient_MapGetByIndex(AerospikeClient *self, PyObject *args,
-										PyObject *kwds);
-PyObject *AerospikeClient_MapGetByIndexRange(AerospikeClient *self,
-											 PyObject *args, PyObject *kwds);
-
-PyObject *AerospikeClient_MapGetByRank(AerospikeClient *self, PyObject *args,
-									   PyObject *kwds);
-PyObject *AerospikeClient_MapGetByRankRange(AerospikeClient *self,
-											PyObject *args, PyObject *kwds);
+                                         PyObject *kwds);
 
 /*******************************************************************************
  * SCAN OPERATIONS
@@ -448,11 +229,9 @@ PyObject *AerospikeClient_MapGetByRankRange(AerospikeClient *self,
  *
  */
 AerospikeScan *AerospikeClient_Scan(AerospikeClient *self, PyObject *args,
-									PyObject *kwds);
+                                    PyObject *kwds);
 PyObject *AerospikeClient_ScanApply(AerospikeClient *self, PyObject *args,
-									PyObject *kwds);
-PyObject *AerospikeClient_ScanInfo(AerospikeClient *self, PyObject *args,
-								   PyObject *kwds);
+                                    PyObject *kwds);
 
 /*******************************************************************************
  * QUERY OPERATIONS
@@ -478,11 +257,11 @@ PyObject *AerospikeClient_ScanInfo(AerospikeClient *self, PyObject *args,
  *
  */
 AerospikeQuery *AerospikeClient_Query(AerospikeClient *self, PyObject *args,
-									  PyObject *kwds);
+                                      PyObject *kwds);
 PyObject *AerospikeClient_QueryApply(AerospikeClient *self, PyObject *args,
-									 PyObject *kwds);
+                                     PyObject *kwds);
 PyObject *AerospikeClient_JobInfo(AerospikeClient *self, PyObject *args,
-								  PyObject *kwds);
+                                  PyObject *kwds);
 
 /*******************************************************************************
  * INFO OPERATIONS
@@ -499,7 +278,7 @@ PyObject *AerospikeClient_JobInfo(AerospikeClient *self, PyObject *args,
  *
  */
 PyObject *AerospikeClient_InfoSingleNode(AerospikeClient *self, PyObject *args,
-										 PyObject *kwds);
+                                         PyObject *kwds);
 
 /**
  * Performs a `info` operation. This will invoke the info request against a random
@@ -512,27 +291,7 @@ PyObject *AerospikeClient_InfoSingleNode(AerospikeClient *self, PyObject *args,
  *
  */
 PyObject *AerospikeClient_InfoRandomNode(AerospikeClient *self, PyObject *args,
-										 PyObject *kwds);
-
-/**
- *  ----------------------------- DEPRECATED ---------------------------------
- * Please use info_single_node() instead.
- * Performs a `info` operation. This will invoke the info request against each
- * node in the cluster. The return value is a dict where the key is the node
- * name and the value is a tuple of (Error,Response). If an error occurred on
- * the node, the Error will be an object containing details, otherwise it is
- * None. If the request was successful, then the Response will contain the
- * string response from the node, otherwise it is None.
- *
- *		for node,(err,res) in client.info('statistics').items():
- *			if err == None:
- *				print "{0} - OK: {1}".format(record,res)
- *			else:
- *				print "{0} - ERR: {1}".format(record,err)
- *
- */
-PyObject *AerospikeClient_InfoNode(AerospikeClient *self, PyObject *args,
-								   PyObject *kwds);
+                                         PyObject *kwds);
 
 /*******************************************************************************
  * UDF OPERATIONS
@@ -544,7 +303,7 @@ PyObject *AerospikeClient_InfoNode(AerospikeClient *self, PyObject *args,
  *
  */
 PyObject *AerospikeClient_UDF_Put(AerospikeClient *self, PyObject *args,
-								  PyObject *kwds);
+                                  PyObject *kwds);
 
 /**
  * De-registers a UDF.
@@ -553,7 +312,7 @@ PyObject *AerospikeClient_UDF_Put(AerospikeClient *self, PyObject *args,
  *
  */
 PyObject *AerospikeClient_UDF_Remove(AerospikeClient *self, PyObject *args,
-									 PyObject *kwds);
+                                     PyObject *kwds);
 
 /**
  * Lists the UDFs
@@ -562,7 +321,7 @@ PyObject *AerospikeClient_UDF_Remove(AerospikeClient *self, PyObject *args,
  *
  */
 PyObject *AerospikeClient_UDF_List(AerospikeClient *self, PyObject *args,
-								   PyObject *kwds);
+                                   PyObject *kwds);
 
 /**
  * Gets the registered UDFs
@@ -571,7 +330,7 @@ PyObject *AerospikeClient_UDF_List(AerospikeClient *self, PyObject *args,
  *
  */
 PyObject *AerospikeClient_UDF_Get_UDF(AerospikeClient *self, PyObject *args,
-									  PyObject *kwds);
+                                      PyObject *kwds);
 
 /*******************************************************************************
  * SECONDARY INDEX OPERATIONS
@@ -583,7 +342,7 @@ PyObject *AerospikeClient_UDF_Get_UDF(AerospikeClient *self, PyObject *args,
  *
  */
 PyObject *AerospikeClient_Index_Integer_Create(AerospikeClient *self,
-											   PyObject *args, PyObject *kwds);
+                                               PyObject *args, PyObject *kwds);
 
 /**
  * Create secondary string index
@@ -592,7 +351,16 @@ PyObject *AerospikeClient_Index_Integer_Create(AerospikeClient *self,
  *
  */
 PyObject *AerospikeClient_Index_String_Create(AerospikeClient *self,
-											  PyObject *args, PyObject *kwds);
+                                              PyObject *args, PyObject *kwds);
+
+/**
+ * Create secondary string index
+ *
+ *		client.index_blob_create(namespace, set, bin, index_name, policy)
+ *
+ */
+PyObject *AerospikeClient_Index_Blob_Create(AerospikeClient *self,
+                                            PyObject *args, PyObject *kwds);
 
 /**
  * Create secondary cdt index
@@ -601,7 +369,7 @@ PyObject *AerospikeClient_Index_String_Create(AerospikeClient *self,
  *
  */
 PyObject *AerospikeClient_Index_Cdt_Create(AerospikeClient *self,
-											  PyObject *args, PyObject *kwds);
+                                           PyObject *args, PyObject *kwds);
 
 /**
  * Create secondary geospatial index
@@ -610,7 +378,7 @@ PyObject *AerospikeClient_Index_Cdt_Create(AerospikeClient *self,
  *
  */
 PyObject *AerospikeClient_Index_2dsphere_Create(AerospikeClient *self,
-												PyObject *args, PyObject *kwds);
+                                                PyObject *args, PyObject *kwds);
 
 /**
  * Remove secondary index
@@ -619,7 +387,7 @@ PyObject *AerospikeClient_Index_2dsphere_Create(AerospikeClient *self,
  *
  */
 PyObject *AerospikeClient_Index_Remove(AerospikeClient *self, PyObject *args,
-									   PyObject *kwds);
+                                       PyObject *kwds);
 /**
  * Create secondary list index
  *
@@ -627,7 +395,7 @@ PyObject *AerospikeClient_Index_Remove(AerospikeClient *self, PyObject *args,
  *
  */
 PyObject *AerospikeClient_Index_List_Create(AerospikeClient *self,
-											PyObject *args, PyObject *kwds);
+                                            PyObject *args, PyObject *kwds);
 /**
  * Create secondary map keys index
  *
@@ -635,7 +403,7 @@ PyObject *AerospikeClient_Index_List_Create(AerospikeClient *self,
  *
  */
 PyObject *AerospikeClient_Index_Map_Keys_Create(AerospikeClient *self,
-												PyObject *args, PyObject *kwds);
+                                                PyObject *args, PyObject *kwds);
 /**
  * Create secondary map values index
  *
@@ -643,8 +411,17 @@ PyObject *AerospikeClient_Index_Map_Keys_Create(AerospikeClient *self,
  *
  */
 PyObject *AerospikeClient_Index_Map_Values_Create(AerospikeClient *self,
-												  PyObject *args,
-												  PyObject *kwds);
+                                                  PyObject *args,
+                                                  PyObject *kwds);
+
+/**
+* Get the base64 representation of an aerospike CDT ctx.
+*
+* b64_string = client.get_cdtctx_base64(compiled_cdtctx)
+*
+*/
+PyObject *AerospikeClient_GetCDTCTXBase64(AerospikeClient *self, PyObject *args,
+                                          PyObject *kwds);
 
 /*******************************************************************************
  * LOG OPERATIONS
@@ -656,7 +433,7 @@ PyObject *AerospikeClient_Index_Map_Values_Create(AerospikeClient *self,
  *
  */
 PyObject *AerospikeClient_Set_Log_Level(AerospikeClient *self, PyObject *args,
-										PyObject *kwds);
+                                        PyObject *kwds);
 
 /**
  * Sets the log handler
@@ -665,7 +442,7 @@ PyObject *AerospikeClient_Set_Log_Level(AerospikeClient *self, PyObject *args,
  *
  */
 PyObject *AerospikeClient_Set_Log_Handler(AerospikeClient *self, PyObject *args,
-										  PyObject *kwds);
+                                          PyObject *kwds);
 
 /**
  * Get records in a batch
@@ -674,7 +451,7 @@ PyObject *AerospikeClient_Set_Log_Handler(AerospikeClient *self, PyObject *args,
  *
  */
 PyObject *AerospikeClient_Get_Many(AerospikeClient *self, PyObject *args,
-								   PyObject *kwds);
+                                   PyObject *kwds);
 
 /**
  * Get records in a batch
@@ -683,7 +460,7 @@ PyObject *AerospikeClient_Get_Many(AerospikeClient *self, PyObject *args,
  *
  */
 PyObject *AerospikeClient_Batch_GetOps(AerospikeClient *self, PyObject *args,
-									   PyObject *kwds);
+                                       PyObject *kwds);
 
 /**
  * Read/Write multiple records for specified batch keys in one batch call.
@@ -695,7 +472,7 @@ PyObject *AerospikeClient_Batch_GetOps(AerospikeClient *self, PyObject *args,
  *
  */
 PyObject *AerospikeClient_BatchWrite(AerospikeClient *self, PyObject *args,
-									 PyObject *kwds);
+                                     PyObject *kwds);
 
 /**
  * Perform read/write operations on multiple keys.
@@ -705,7 +482,16 @@ PyObject *AerospikeClient_BatchWrite(AerospikeClient *self, PyObject *args,
  *
  */
 PyObject *AerospikeClient_Batch_Operate(AerospikeClient *self, PyObject *args,
-										PyObject *kwds);
+                                        PyObject *kwds);
+
+/**
+ * Perform reads on multiple keys.
+ *
+ *      client.batch_read([keys], [bins], policy_batch)
+ *
+ */
+PyObject *AerospikeClient_BatchRead(AerospikeClient *self, PyObject *args,
+                                    PyObject *kwds);
 
 /**
  * Remove multiple records by key.
@@ -715,7 +501,7 @@ PyObject *AerospikeClient_Batch_Operate(AerospikeClient *self, PyObject *args,
  *
  */
 PyObject *AerospikeClient_Batch_Remove(AerospikeClient *self, PyObject *args,
-									   PyObject *kwds);
+                                       PyObject *kwds);
 
 /**
  * Apply a user defined function (UDF) to multiple keys.
@@ -725,7 +511,7 @@ PyObject *AerospikeClient_Batch_Remove(AerospikeClient *self, PyObject *args,
  *
  */
 PyObject *AerospikeClient_Batch_Apply(AerospikeClient *self, PyObject *args,
-									  PyObject *kwds);
+                                      PyObject *kwds);
 
 /**
  * Filter bins from records in a batch
@@ -734,7 +520,7 @@ PyObject *AerospikeClient_Batch_Apply(AerospikeClient *self, PyObject *args,
  *
  */
 PyObject *AerospikeClient_Select_Many(AerospikeClient *self, PyObject *args,
-									  PyObject *kwds);
+                                      PyObject *kwds);
 
 /**
  * Check existence of given keys
@@ -743,7 +529,7 @@ PyObject *AerospikeClient_Select_Many(AerospikeClient *self, PyObject *args,
  *
  */
 PyObject *AerospikeClient_Exists_Many(AerospikeClient *self, PyObject *args,
-									  PyObject *kwds);
+                                      PyObject *kwds);
 
 /**
 * Perform xdr-set-filter info operation on the database.
@@ -752,7 +538,7 @@ PyObject *AerospikeClient_Exists_Many(AerospikeClient *self, PyObject *args,
 *
 */
 PyObject *AerospikeClient_SetXDRFilter(AerospikeClient *self, PyObject *args,
-									   PyObject *kwds);
+                                       PyObject *kwds);
 
 /**
 * Get the base64 representation of an aerospike expression.
@@ -761,23 +547,14 @@ PyObject *AerospikeClient_SetXDRFilter(AerospikeClient *self, PyObject *args,
 *
 */
 PyObject *AerospikeClient_GetExpressionBase64(AerospikeClient *self,
-											  PyObject *args, PyObject *kwds);
+                                              PyObject *args, PyObject *kwds);
 
 /**
  * Send an info request to the entire cluster
  * client.info_all("statistics", {}")
 */
 PyObject *AerospikeClient_InfoAll(AerospikeClient *self, PyObject *args,
-								  PyObject *kwds);
-
-/**
-* Perform info operation on the database.
-*
-* client.info((x,y,z))
-*
-*/
-PyObject *AerospikeClient_Info(AerospikeClient *self, PyObject *args,
-							   PyObject *kwds);
+                                  PyObject *kwds);
 
 /**
 * Perform get nodes operation on the database.
@@ -786,7 +563,7 @@ PyObject *AerospikeClient_Info(AerospikeClient *self, PyObject *args,
 *
 */
 PyObject *AerospikeClient_GetNodes(AerospikeClient *self, PyObject *args,
-								   PyObject *kwds);
+                                   PyObject *kwds);
 /**
 * Perform get node name operation on the database.
 *
@@ -794,15 +571,7 @@ PyObject *AerospikeClient_GetNodes(AerospikeClient *self, PyObject *args,
 *
 */
 PyObject *AerospikeClient_GetNodeNames(AerospikeClient *self, PyObject *args,
-									   PyObject *kwds);
-/**
-* Perforrm get key digest operation on the database.
-*
-* client.get_key_digest((x,y,z))
-*
-*/
-PyObject *AerospikeClient_Get_Key_Digest(AerospikeClient *self, PyObject *args,
-										 PyObject *kwds);
+                                       PyObject *kwds);
 /**
 * Perform get key's partition id from cluster.
 *
@@ -810,7 +579,7 @@ PyObject *AerospikeClient_Get_Key_Digest(AerospikeClient *self, PyObject *args,
 *
 */
 PyObject *AerospikeClient_Get_Key_PartitionID(AerospikeClient *self,
-											  PyObject *args, PyObject *kwds);
+                                              PyObject *args, PyObject *kwds);
 /**
  * Return search string for host port combination
  */
@@ -819,16 +588,16 @@ char *return_search_string(aerospike *as);
  * Close the aerospike object depending on the global_hosts entries
  */
 void close_aerospike_object(aerospike *as, as_error *err, char *alias_to_search,
-							PyObject *py_persistent_item, bool do_destroy);
+                            PyObject *py_persistent_item, bool do_destroy);
 /**
  * Check type for 'operate' operation
  */
 int check_type(AerospikeClient *self, PyObject *py_value, int op,
-			   as_error *err);
+               as_error *err);
 
 /*******************************************************************************
  * TRUNCATE OPERATIONS
  ******************************************************************************/
 
 PyObject *AerospikeClient_Truncate(AerospikeClient *self, PyObject *args,
-								   PyObject *kwds);
+                                   PyObject *kwds);
