@@ -27,38 +27,34 @@
 #include "policy.h"
 
 PyObject *AerospikeGeospatial_Unwrap(AerospikeGeospatial *self, PyObject *args,
-									 PyObject *kwds)
+                                     PyObject *kwds)
 {
 
-	// Python function arguments
-	// Python function keyword arguments
-	static char *kwlist[] = {NULL};
+    // Python function arguments
+    // Python function keyword arguments
+    static char *kwlist[] = {NULL};
 
-	if (PyArg_ParseTupleAndKeywords(args, kwds, ":unwrap", kwlist) == false) {
-		return NULL;
-	}
+    if (PyArg_ParseTupleAndKeywords(args, kwds, ":unwrap", kwlist) == false) {
+        return NULL;
+    }
 
-	// Aerospike error object
-	as_error err;
-	// Initialize error object
-	as_error_init(&err);
+    // Aerospike error object
+    as_error err;
+    // Initialize error object
+    as_error_init(&err);
 
-	if (!self) {
-		as_error_update(&err, AEROSPIKE_ERR_PARAM, "Invalid geospatial object");
-		goto CLEANUP;
-	}
+    if (!self) {
+        as_error_update(&err, AEROSPIKE_ERR_PARAM, "Invalid geospatial object");
+        goto CLEANUP;
+    }
 
 CLEANUP:
 
-	// If an error occurred, tell Python.
-	if (err.code != AEROSPIKE_OK) {
-		PyObject *py_err = NULL;
-		error_to_pyobject(&err, &py_err);
-		PyObject *exception_type = raise_exception(&err);
-		PyErr_SetObject(exception_type, py_err);
-		Py_DECREF(py_err);
-		return NULL;
-	}
-	Py_INCREF(self->geo_data);
-	return self->geo_data;
+    // If an error occurred, tell Python.
+    if (err.code != AEROSPIKE_OK) {
+        raise_exception(&err);
+        return NULL;
+    }
+    Py_INCREF(self->geo_data);
+    return self->geo_data;
 }
